@@ -6,7 +6,7 @@
  */
 
 
-#include "allegro.h"
+#include <allegro.h>
 
 
 
@@ -92,4 +92,4 @@ int main(void)
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
