@@ -35,7 +35,7 @@
  */
 
 
-#include "allegro.h"
+#include <allegro.h>
 
 
 
@@ -112,7 +112,7 @@ BITMAP *create_light_graphic(void)
 
 	 dist = fixtoi(fixsqrt(itofix(MID(-32768, dx*dx+dy*dy, 32767))));
 
-	 dir = fixtoi(fixatan2(itofix(dy), itofix(dx)));
+	 dir = fixtoi(fixatan2(itofix(dy), itofix(dx)) + itofix(128));
 
 	 hsv_to_rgb(dir*360.0/256.0, MID(0, dist/128.0, 1), 1, &r, &g, &b);
 
@@ -198,8 +198,8 @@ void generate_conversion_tables(void)
 /* copies from our magic format data onto a normal Allegro screen bitmap */
 void blit_magic_format_to_screen(BITMAP *bmp)
 {
-   unsigned long addr;
-   unsigned long *data;
+   uintptr_t addr;
+   uint32_t *data;
    unsigned long in1, in2, in3;
    unsigned long out1, out2;
    int x, y;
@@ -260,7 +260,7 @@ void blit_magic_format_to_screen(BITMAP *bmp)
 
    for (y=0; y<SCREEN_H; y++) {
       addr = bmp_write_line(screen, y);
-      data = (unsigned long *)bmp->line[y];
+      data = (uint32_t *)bmp->line[y];
 
       for (x=0; x<SCREEN_W/4; x++) {
 	 in1 = *(data++);
@@ -336,8 +336,8 @@ void generate_conversion_tables(void)
 /* copies from our magic format data onto a normal Allegro screen bitmap */
 void blit_magic_format_to_screen(BITMAP *bmp)
 {
-   unsigned long addr;
-   unsigned long *data;
+   uintptr_t addr;
+   uint32_t *data;
    unsigned long in1, in2, in3, temp1, temp2, temp3;
    unsigned long out1, out2;
    int x, y;
@@ -366,7 +366,7 @@ void blit_magic_format_to_screen(BITMAP *bmp)
 
    for (y=0; y<SCREEN_H; y++) {
       addr = bmp_write_line(screen, y);
-      data = (unsigned long *)bmp->line[y];
+      data = (uint32_t *)bmp->line[y];
 
       for (x=0; x<SCREEN_W/4; x++) {
 	 in1 = *(data++);
@@ -419,18 +419,12 @@ int main(int argc, char *argv[])
 
    /* set a 15 or 16 bpp video mode */
    set_color_depth(16);
-   if (set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, 640, 480, 0, 0) != 0) {
+   if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
       set_color_depth(15);
-      if (set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, 640, 480, 0, 0) != 0) {
-         set_color_depth(16);
-         if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
-            set_color_depth(15);
-            if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
-	       set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-	       allegro_message("Error setting a 15 or 16 bpp 640x480 video mode\n%s\n", allegro_error);
-	       return 1;
-            }
-	 }
+      if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
+	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	 allegro_message("Error setting a 15 or 16 bpp 640x480 video mode\n%s\n", allegro_error);
+	 return 1;
       }
    }
 
@@ -547,4 +541,4 @@ int main(int argc, char *argv[])
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
