@@ -1,11 +1,17 @@
 /*
  *    Example program for the Allegro library, by Shawn Hargreaves.
  *
- *    This program demonstrates how to play samples.
+ *    This program demonstrates how to play samples. You have to
+ *    use this example from the commandline to specify as first
+ *    parameter a WAV or VOC sound file to play. If the file is
+ *    loaded successfully, the sound will be played in an infinite
+ *    loop. While it is being played, you can use the left and right
+ *    arrow keys to modify the panning of the sound. You can also
+ *    use the up and down arrow keys to modify the pitch.
  */
 
 
-#include "allegro.h"
+#include <allegro.h>
 
 
 
@@ -27,7 +33,7 @@ int main(int argc, char *argv[])
    install_timer();
 
    /* install a digital sound driver */
-   if (install_sound(DIGI_AUTODETECT, MIDI_NONE, argv[0]) != 0) {
+   if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL) != 0) {
       allegro_message("Error initialising sound system\n%s\n", allegro_error);
       return 1;
    }
@@ -89,4 +95,4 @@ int main(int argc, char *argv[])
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
