@@ -41,19 +41,6 @@
    #endif
 
    #ifndef ALLEGRO_NO_FIX_ALIASES
-      #if (defined ALLEGRO_GCC)
-         #define AL_ALIAS(DECL, CALL)                      \
-         static __attribute__((unused)) __inline__ DECL    \
-         {                                                 \
-            return CALL;                                   \
-         }
-      #else
-         #define AL_ALIAS(DECL, CALL)              \
-         static INLINE DECL                        \
-         {                                         \
-            return CALL;                           \
-         }
-      #endif
       AL_ALIAS(fixed fadd(fixed x, fixed y), fixadd(x, y))
       AL_ALIAS(fixed fsub(fixed x, fixed y), fixsub(x, y))
       AL_ALIAS(fixed fmul(fixed x, fixed y), fixmul(x, y))
@@ -69,7 +56,6 @@
       AL_ALIAS(fixed fatan2(fixed y, fixed x), fixatan2(y, x))
       AL_ALIAS(fixed fsqrt(fixed x), fixsqrt(x))
       AL_ALIAS(fixed fhypot(fixed x, fixed y), fixhypot(x, y))
-      #undef AL_ALIAS
    #endif
 
 #endif  /* !defined ALLEGRO_LIB_BUILD */
@@ -215,6 +201,32 @@ AL_INLINE_DEPRECATED(void, set_window_close_hook, (void (*proc)(void)),
 })
 
 
+/* the weird old clipping API */
+AL_FUNC_DEPRECATED(void, set_clip, (BITMAP *bitmap, int x1, int y_1, int x2, int y2));
+
+
+/* unnecessary, can use rest(0) */
+AL_INLINE_DEPRECATED(void, yield_timeslice, (void),
+{
+   ASSERT(system_driver);
+
+   if (system_driver->yield_timeslice)
+      system_driver->yield_timeslice();
+})
+
+
+/* DOS-ish monitor retrace ideas that don't work elsewhere */
+AL_FUNCPTR(void, retrace_proc, (void));
+
+#ifdef ALLEGRO_LIB_BUILD
+   AL_FUNC(int,  timer_can_simulate_retrace, (void));
+   AL_FUNC(void, timer_simulate_retrace, (int enable));
+#else
+   AL_FUNC_DEPRECATED(int,  timer_can_simulate_retrace, (void));
+   AL_FUNC_DEPRECATED(void, timer_simulate_retrace, (int enable));
+#endif
+AL_FUNC_DEPRECATED(int,  timer_is_using_retrace, (void));
+
 #ifdef __cplusplus
    }
 #endif
