@@ -33,6 +33,8 @@
 /* A static auto config */
 #define HAVE_LIBPTHREAD
 #define HAVE_DIRENT_H
+#define HAVE_INTTYPES_H
+#define HAVE_STDINT_H
 #define HAVE_SYS_DIR_H
 #define HAVE_SYS_TIME_H
 #define TIME_WITH_SYS_TIME
