@@ -38,6 +38,15 @@ AL_FUNC(int, install_sound_input, (int digi, int midi));
 AL_FUNC(void, remove_sound_input, (void));
 
 AL_FUNC(void, set_volume, (int digi_volume, int midi_volume));
+AL_FUNC(void, set_hardware_volume, (int digi_volume, int midi_volume));
+
+AL_FUNC(void, set_mixer_quality, (int quality));
+AL_FUNC(int, get_mixer_quality, (void));
+AL_FUNC(int, get_mixer_frequency, (void));
+AL_FUNC(int, get_mixer_bits, (void));
+AL_FUNC(int, get_mixer_channels, (void));
+AL_FUNC(int, get_mixer_voices, (void));
+AL_FUNC(int, get_mixer_buffer_length, (void));
 
 #ifdef __cplusplus
    }
