@@ -21,13 +21,13 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "allegro.h"
+#include "allegro/internal/aintern.h"
+
 #ifdef ALLEGRO_LINUX
    #include <sys/time.h>
 #endif
 
-#include "allegro.h"
-#include "allegro/internal/aintern.h"
-
 #ifdef ALLEGRO_INTERNAL_HEADER
    #include ALLEGRO_INTERNAL_HEADER
 #endif
@@ -216,8 +216,8 @@ static PARAMETER parameters[] =
 
  #ifdef DIGI_OSS
    { "oss_driver",         param_str,     "",      "/dev/dsp", NULL,             "Drv:",     "",         NULL,    "Name of the driver (e.g. /dev/dsp)" },
-   { "oss_numfrags",       param_int,     "",      "-1",       &_oss_numfrags,   "NumFr:",   NULL,       NULL,    "Number of fragments (use 2, other values make little difference)" },
-   { "oss_fragsize",       param_num,     "",      "-1",       &_oss_fragsize,   "Siz:",     NULL,       NULL,    "Size of fragment (small values -- choppy sound, large values -- delayed sound)" },
+   { "oss_numfrags",       param_int,     "",      "-1",       NULL,             "NumFr:",   NULL,       NULL,    "Number of fragments (use 2, other values make little difference)" },
+   { "oss_fragsize",       param_num,     "",      "-1",       NULL,             "Siz:",     NULL,       NULL,    "Size of fragment (small values -- choppy sound, large values -- delayed sound)" },
  #endif
 
  #ifdef MIDI_OSS
@@ -2854,7 +2854,9 @@ int main(void)
    set_palette(black_palette);
 
  #ifdef SETUP_USE_COMPILED_DATAFILES
+ #ifndef ALLEGRO_USE_CONSTRUCTOR
    fixup_datafile(setup_data);
+ #endif
  #else
    setup_data = load_datafile(uconvert_ascii("#", tmp1));
 
@@ -2963,5 +2965,5 @@ int main(void)
 }
 
 #ifndef SETUP_EMBEDDED
-END_OF_MAIN();
+END_OF_MAIN()
 #endif
