@@ -25,6 +25,12 @@
 
 
 
+#define PREFIX_I                "al-main INFO: "
+#define PREFIX_W                "al-main WARNING: "
+#define PREFIX_E                "al-main ERROR: "
+
+
+
 /* in case you want to report version numbers */
 char allegro_id[] = "Allegro " ALLEGRO_VERSION_STR ", " ALLEGRO_PLATFORM_STR;
 
@@ -45,6 +51,14 @@ int *allegro_errno = NULL;
 int _allegro_count = 0;
 
 
+/* flag to know whether we are being called by the exit mechanism */
+int _allegro_in_exit = FALSE;
+
+
+/* flag to decide whether to disable the screensaver */
+int _screensaver_policy = FULLSCREEN_DISABLED;
+
+
 /* the graphics driver currently in use */
 GFX_DRIVER *gfx_driver = NULL;
 
@@ -52,8 +66,6 @@ GFX_DRIVER *gfx_driver = NULL;
 /* abilities of the current graphics driver */
 int gfx_capabilities = 0;
 
-/* pointer to list of valid video modes */
-GFX_MODE_LIST *gfx_mode_list = NULL;
 
 /* a bitmap structure for accessing the physical screen */
 BITMAP *screen = NULL;
@@ -211,6 +223,7 @@ struct _AL_LINKER_MOUSE *_al_linker_mouse = NULL;
 /* dynamic registration system for cleanup code */
 struct al_exit_func {
    void (*funcptr)(void);
+   AL_CONST char *desc;
    struct al_exit_func *next;
 };
 
@@ -218,10 +231,22 @@ static struct al_exit_func *exit_func_list = NULL;
 
 
 
+/* _get_allegro_version:
+ *  Retrieves the library version.
+ */
+int _get_allegro_version(void)
+{
+   return MAKE_VERSION(ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, ALLEGRO_WIP_VERSION);
+}
+
+
+
 /* _add_exit_func:
  *  Adds a function to the list that need to be called by allegro_exit().
+ *  `desc' should point to a statically allocated string to help with
+ *  debugging.
  */
-void _add_exit_func(void (*func)(void))
+void _add_exit_func(void (*func)(void), AL_CONST char *desc)
 {
    struct al_exit_func *n;
 
@@ -235,6 +260,7 @@ void _add_exit_func(void (*func)(void))
 
    n->next = exit_func_list;
    n->funcptr = func;
+   n->desc = desc;
    exit_func_list = n;
 }
 
@@ -263,10 +289,22 @@ void _remove_exit_func(void (*func)(void))
 
 
 
-/* install_allegro:
+/* allegro_exit_stub:
+ *  Stub function registered by the library via atexit.
+ */
+static void allegro_exit_stub(void)
+{
+   _allegro_in_exit = TRUE;
+
+   allegro_exit();
+}
+
+
+
+/* _install_allegro:
  *  Initialises the Allegro library, activating the system driver.
  */
-int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func)(void)))
+int _install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func)(void)))
 {
    RGB black_rgb = {0, 0, 0, 0};
    char tmp1[64], tmp2[64];
@@ -283,6 +321,7 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
       _mouse_constructor();
       _register_bitmap_file_type_init();
       _register_sample_file_type_init();
+      _register_font_file_type_init();
    #endif
 
    if (errno_ptr)
@@ -316,6 +355,9 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
    /* nasty stuff to set up the config system before the system driver */
    system_driver = _system_driver_list[0].driver;
 
+   /* needed in case set_config_file was called before allegro_init */
+   _reload_config();
+
    reload_config_texts(NULL);
 
    if (system_id == SYSTEM_AUTODETECT)
@@ -350,14 +392,20 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
    /* detect CPU type */
    check_cpu();
 
+   /* set up screensaver policy */
+   _screensaver_policy = get_config_int(uconvert_ascii("system", tmp1),
+                                        uconvert_ascii("disable_screensaver", tmp2),
+                                        FULLSCREEN_DISABLED);
+
    /* install shutdown handler */
    if (_allegro_count == 0) {
       if (atexit_ptr)
-	 atexit_ptr(allegro_exit);
+	 atexit_ptr(allegro_exit_stub);
    }
 
    _allegro_count++;
 
+   TRACE(PREFIX_I "Allegro initialised (instance %d)\n", _allegro_count);
    return 0;
 }
 
@@ -366,10 +414,11 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
 /* allegro_exit:
  *  Closes down the Allegro system.
  */
-void allegro_exit()
+void allegro_exit(void)
 {
-   while (exit_func_list)
+   while (exit_func_list) {
       (*(exit_func_list->funcptr))();
+   }
 
    if (system_driver) {
       system_driver->exit();
@@ -499,7 +548,7 @@ void al_assert(AL_CONST char *file, int line)
 	 assert_file = NULL;
 
       if (debug_trace_virgin)
-	 _add_exit_func(debug_exit);
+	 _add_exit_func(debug_exit, "debug_exit");
 
       debug_assert_virgin = FALSE;
    }
@@ -555,7 +604,7 @@ void al_trace(AL_CONST char *msg, ...)
 	 trace_file = fopen("allegro.log", "w");
 
       if (debug_assert_virgin)
-	 _add_exit_func(debug_exit);
+	 _add_exit_func(debug_exit, "debug_exit");
 
       debug_trace_virgin = FALSE;
    }
