@@ -928,6 +928,8 @@ unsigned long _blender_screen15(unsigned long x, unsigned long y, unsigned long
 #define SET_BLENDER_FUNC(name)                                 \
    void set_##name##_blender(int r, int g, int b, int a)       \
    {                                                           \
+      if (gfx_driver && gfx_driver->set_blender_mode)          \
+         gfx_driver->set_blender_mode(blender_mode_##name, r, g, b, a);\
       set_blender_mode(BF16(_blender_##name##15),              \
 		       BF16(_blender_##name##16),              \
 		       BF24(_blender_##name##24),              \
@@ -954,11 +956,15 @@ SET_BLENDER_FUNC(screen);
 /* set_alpha_blender:
  *  Sets the special RGBA blending mode.
  */
-void set_alpha_blender()
+void set_alpha_blender(void)
 {
    BLENDER_FUNC f15, f16, f24, f32;
    int r, b;
 
+   /* Call gfx_driver->set_blender_mode() for hardware acceleration */
+   if (gfx_driver && gfx_driver->set_blender_mode)
+      gfx_driver->set_blender_mode(blender_mode_alpha, 0, 0, 0, 0);
+
    /* check which way around the 32 bit pixels are */
    if ((_rgb_g_shift_32 == 8) && (_rgb_a_shift_32 == 24)) {
       r = (_rgb_r_shift_32) ? 1 : 0;
@@ -1039,7 +1045,7 @@ unsigned long _blender_write_alpha(unsigned long x, unsigned long y, unsigned lo
 /* set_write_alpha_blender:
  *  Sets the special RGBA editing mode.
  */
-void set_write_alpha_blender()
+void set_write_alpha_blender(void)
 {
    BLENDER_FUNC f32;
 
