@@ -108,7 +108,7 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
    int x, y, w, h, c, r, g, b, a;
    int destbits, rgba;
    unsigned short *p16;
-   unsigned long *p32;
+   uint32_t *p32;
    BITMAP *bmp;
 
    if (bits < 0) {
@@ -126,7 +126,7 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
    w = pack_mgetw(f);
    h = pack_mgetw(f);
 
-   bmp = create_bitmap_ex(bits, w, h);
+   bmp = create_bitmap_ex(MAX(bits, 8), w, h);
    if (!bmp) {
       *allegro_errno = ENOMEM;
       return NULL;
@@ -148,7 +148,7 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
       case 15:
 	 /* 15bit hicolor */
 	 for (y=0; y<h; y++) {
-	    p16 = (unsigned short *)bmp->line[y];
+	    p16 = (uint16_t *)bmp->line[y];
 
 	    for (x=0; x<w; x++) {
 	       c = pack_igetw(f);
@@ -163,7 +163,7 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
       case 16:
 	 /* 16bit hicolor */
 	 for (y=0; y<h; y++) {
-	    p16 = (unsigned short *)bmp->line[y];
+	    p16 = (uint16_t *)bmp->line[y];
 
 	    for (x=0; x<w; x++) {
 	       c = pack_igetw(f);
@@ -186,9 +186,8 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
 	       if (rgba)
 		  pack_getc(f);
 
-	       bmp->line[y][x*3+_rgb_r_shift_24/8] = r;
-	       bmp->line[y][x*3+_rgb_g_shift_24/8] = g;
-	       bmp->line[y][x*3+_rgb_b_shift_24/8] = b;
+	       c = makecol24(r, g, b);
+	       WRITE3BYTES(bmp->line[y] + (x * 3), c);
 	    }
 	 }
 	 break;
@@ -196,7 +195,7 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
       case 32:
 	 /* 32bit rgba */
 	 for (y=0; y<h; y++) {
-	    p32 = (unsigned long *)bmp->line[y];
+	    p32 = (uint32_t *)bmp->line[y];
 
 	    for (x=0; x<w; x++) {
 	       r = pack_getc(f);
@@ -410,7 +409,7 @@ static FONT_MONO_DATA *read_font_mono(PACKFILE *f, int *hmax)
 /* read_font_color:
  *  Helper for read_font, below.
  */
-static FONT_COLOR_DATA *read_font_color(PACKFILE *pack, int *hmax)
+static FONT_COLOR_DATA *read_font_color(PACKFILE *pack, int *hmax, int depth)
 {
    FONT_COLOR_DATA *cf = NULL;
    int max = 0, i = 0;
@@ -433,9 +432,12 @@ static FONT_COLOR_DATA *read_font_color(PACKFILE *pack, int *hmax)
       *allegro_errno = ENOMEM;
       return NULL;
    }
-   
+
    for (i = 0; i < max; i++) {
-      bits[i] = read_bitmap(pack, 8, FALSE);
+      /* Do not allow colour conversions for 8 bit bitmaps, but require it for
+       * all other colour depths.
+       */
+      bits[i] = read_bitmap(pack, depth, depth!=8);
       if (!bits[i]) {
 	 while (i) {
 	    i--;
@@ -464,6 +466,7 @@ static FONT *read_font(PACKFILE *pack)
    FONT *f = NULL;
    int num_ranges = 0;
    int height = 0;
+   int depth;
 
    f = malloc(sizeof(FONT));
    if (!f) {
@@ -475,7 +478,8 @@ static FONT *read_font(PACKFILE *pack)
 
    num_ranges = pack_mgetw(pack);
    while (num_ranges--) {
-      if (pack_getc(pack)) {
+      depth = pack_getc(pack);
+      if (depth == 1 || depth == 255) {
 	 FONT_MONO_DATA *mf = 0, *iter = (FONT_MONO_DATA *)f->data;
 	 
 	 f->vtable = font_vtable_mono;
@@ -495,10 +499,14 @@ static FONT *read_font(PACKFILE *pack)
       } 
       else {
 	 FONT_COLOR_DATA *cf = NULL, *iter = (FONT_COLOR_DATA *)f->data;
+         
+         /* Older versions of Allegro use `0' to indicate a colour font */
+         if (depth == 0)
+            depth = 8;
 
 	 f->vtable = font_vtable_color;
 
-	 cf = read_font_color(pack, &height);
+	 cf = read_font_color(pack, &height, depth);
 	 if (!cf) {
 	    destroy_font(f);
 	    return NULL;
@@ -674,7 +682,7 @@ static RLE_SPRITE *read_rle_sprite(PACKFILE *f, int bits)
    unsigned int eol_marker;
    signed short s16;
    signed short *p16;
-   signed long *p32;
+   int32_t *p32;
 
    if (bits < 0) {
       bits = -bits;
@@ -748,13 +756,13 @@ static RLE_SPRITE *read_rle_sprite(PACKFILE *f, int bits)
       case 24:
       case 32:
 	 /* read truecolor data */
-	 p32 = (signed long *)s->dat;
+	 p32 = (int32_t *)s->dat;
 	 eol_marker = (bits == 24) ? MASK_COLOR_24 : MASK_COLOR_32;
 
 	 for (y=0; y<h; y++) {
 	    c = pack_igetl(f);
 
-	    while ((unsigned long)c != MASK_COLOR_32) {
+	    while ((uint32_t)c != MASK_COLOR_32) {
 	       if (c < 0) {
 		  /* skip count */
 		  *p32 = c;
@@ -1153,7 +1161,7 @@ static int load_object(DATAFILE *obj, PACKFILE *f, int type)
    ff = pack_fopen_chunk(f, FALSE);
 
    if (ff) {
-      d = ff->todo;
+      d = ff->normal.todo;
 
       /* look for a load function */
       for (i=0; i<MAX_DATAFILE_TYPES; i++) {
@@ -1182,11 +1190,11 @@ static int load_object(DATAFILE *obj, PACKFILE *f, int type)
 
 
 
-/* load_property:
+/* _load_property:
  *  Helper to load a property from a datafile and store it in 'prop'.
  *  Returns 0 on success and -1 on failure.
  */
-static int load_property(DATAFILE_PROPERTY *prop, PACKFILE *f)
+int _load_property(DATAFILE_PROPERTY *prop, PACKFILE *f)
 {
    int type, size;
    char *p;
@@ -1225,11 +1233,11 @@ static int load_property(DATAFILE_PROPERTY *prop, PACKFILE *f)
 
 
 
-/* add_property:
+/* _add_property:
  *  Helper to add a new property to a property list. Returns 0 on
  *  success or -1 on failure.
  */
-static int add_property(DATAFILE_PROPERTY **list, DATAFILE_PROPERTY *prop)
+int _add_property(DATAFILE_PROPERTY **list, DATAFILE_PROPERTY *prop)
 {
    DATAFILE_PROPERTY *iter;
    int length = 0;
@@ -1265,10 +1273,10 @@ static int add_property(DATAFILE_PROPERTY **list, DATAFILE_PROPERTY *prop)
 
 
 
-/* destroy_property_list:
+/* _destroy_property_list:
  *  Helper to destroy a property list.
  */
-static void destroy_property_list(DATAFILE_PROPERTY *list)
+void _destroy_property_list(DATAFILE_PROPERTY *list)
 {
    int c;
 
@@ -1307,7 +1315,7 @@ static void *load_file_object(PACKFILE *f, long size)
       type = pack_mgetl(f);
 
       if (type == DAT_PROPERTY) {
-	 if ((load_property(&prop, f) != 0) || (add_property(&list, &prop) != 0)) {
+	 if ((_load_property(&prop, f) != 0) || (_add_property(&list, &prop) != 0)) {
 	    failed = TRUE;
 	    break;
 	 }
@@ -1335,7 +1343,7 @@ static void *load_file_object(PACKFILE *f, long size)
 
    /* destroy the property list if not assigned to an object */
    if (list)
-      destroy_property_list(list);
+      _destroy_property_list(list);
 
    /* gracefully handle failure */
    if (failed) {
@@ -1376,7 +1384,7 @@ DATAFILE *load_datafile_callback(AL_CONST char *filename, void (*callback)(DATAF
    if (!f)
       return NULL;
 
-   if ((f->flags & PACKFILE_FLAG_CHUNK) && (!(f->flags & PACKFILE_FLAG_EXEDAT)))
+   if ((f->normal.flags & PACKFILE_FLAG_CHUNK) && (!(f->normal.flags & PACKFILE_FLAG_EXEDAT)))
       type = (_packfile_type == DAT_FILE) ? DAT_MAGIC : 0;
    else
       type = pack_mgetl(f);
@@ -1443,7 +1451,7 @@ DATAFILE *load_datafile_object(AL_CONST char *filename, AL_CONST char *objectnam
    if (!f)
       return NULL;
 
-   if ((f->flags & PACKFILE_FLAG_CHUNK) && (!(f->flags & PACKFILE_FLAG_EXEDAT)))
+   if ((f->normal.flags & PACKFILE_FLAG_CHUNK) && (!(f->normal.flags & PACKFILE_FLAG_EXEDAT)))
       type = (_packfile_type == DAT_FILE) ? DAT_MAGIC : 0;
    else
       type = pack_mgetl(f);
@@ -1465,7 +1473,7 @@ DATAFILE *load_datafile_object(AL_CONST char *filename, AL_CONST char *objectnam
       type = pack_mgetl(f);
 
       if (type == DAT_PROPERTY) {
-	 if ((load_property(&prop, f) != 0) || (add_property(&list, &prop) != 0))
+	 if ((_load_property(&prop, f) != 0) || (_add_property(&list, &prop) != 0))
 	    break;
 
 	 if ((prop.type == DAT_NAME) && (ustricmp(prop.dat, child) == 0))
@@ -1499,7 +1507,7 @@ DATAFILE *load_datafile_object(AL_CONST char *filename, AL_CONST char *objectnam
 
 	    /* destroy the property list */
 	    if (list) {
-	       destroy_property_list(list);
+	       _destroy_property_list(list);
 	       list = NULL;
 	    }
 
@@ -1510,7 +1518,7 @@ DATAFILE *load_datafile_object(AL_CONST char *filename, AL_CONST char *objectnam
 
    /* destroy the property list if not assigned to an object */
    if (list)
-      destroy_property_list(list);
+      _destroy_property_list(list);
 
    pack_fclose(f);
    return dat;
@@ -1527,7 +1535,7 @@ void _unload_datafile_object(DATAFILE *dat)
 
    /* destroy the property list */
    if (dat->prop)
-      destroy_property_list(dat->prop);
+      _destroy_property_list(dat->prop);
 
    /* look for a destructor function */
    for (i=0; i<MAX_DATAFILE_TYPES; i++) {
@@ -1660,9 +1668,9 @@ void fixup_datafile(DATAFILE *data)
    int bpp, depth;
    unsigned char *p8;
    unsigned short *p16;
-   unsigned long *p32;
+   uint32_t *p32;
    signed short *s16;
-   signed long *s32;
+   int32_t *s32;
    int eol_marker;
    ASSERT(data);
 
@@ -1770,7 +1778,7 @@ void fixup_datafile(DATAFILE *data)
 	       case 32:
 		  /* fix up a 32 bit truecolor bitmap */
 		  for (y=0; y<bmp->h; y++) {
-		     p32 = (unsigned long *)bmp->line[y];
+		     p32 = (uint32_t *)bmp->line[y];
 
 		     for (x=0; x<bmp->w; x++) {
 			c = p32[x];
@@ -1888,10 +1896,10 @@ void fixup_datafile(DATAFILE *data)
 		     eol_marker = MASK_COLOR_24;
 		  }
 
-		  s32 = (signed long *)rle->dat;
+		  s32 = (int32_t *)rle->dat;
 
 		  for (y=0; y<rle->h; y++) {
-		     while ((unsigned long)*s32 != MASK_COLOR_24) {
+		     while ((uint32_t)*s32 != MASK_COLOR_24) {
 			if (*s32 > 0) {
 			   x = *s32;
 			   s32++;
@@ -1929,10 +1937,10 @@ void fixup_datafile(DATAFILE *data)
 		     eol_marker = MASK_COLOR_32;
 		  }
 
-		  s32 = (signed long *)rle->dat;
+		  s32 = (int32_t *)rle->dat;
 
 		  for (y=0; y<rle->h; y++) {
-		     while ((unsigned long)*s32 != MASK_COLOR_32) {
+		     while ((uint32_t)*s32 != MASK_COLOR_32) {
 			if (*s32 > 0) {
 			   x = *s32;
 			   s32++;
@@ -1977,7 +1985,7 @@ static void initialise_bitmap(BITMAP *bmp)
    int i;
 
    for (i=0; _vtable_list[i].vtable; i++) {
-      if (_vtable_list[i].color_depth == (int)(unsigned long)bmp->vtable) {
+      if (_vtable_list[i].color_depth == (int)(uintptr_t)bmp->vtable) {
 	 bmp->vtable = _vtable_list[i].vtable;
 	 bmp->write_bank = _stub_bank_switch;
 	 bmp->read_bank = _stub_bank_switch;
@@ -2016,7 +2024,7 @@ static void initialise_datafile(DATAFILE *data)
 
 	 case DAT_FONT:
 	    f = data[c].dat;
-	    color_flag = (int)(unsigned long)f->vtable;
+	    color_flag = (int)(uintptr_t)f->vtable;
 	    if (color_flag == 1) {
 	       FONT_COLOR_DATA *cf = (FONT_COLOR_DATA *)f->data;
 	       while (cf) {
