@@ -19,8 +19,8 @@
 #include <string.h>
 
 #include "allegro.h"
-#include "allegro/aintern.h"
-#include "allegro/aintdos.h"
+#include "allegro/internal/aintern.h"
+#include "allegro/platform/aintdos.h"
 
 #ifndef ALLEGRO_DOS
    #error something is wrong with the makefile
@@ -33,13 +33,13 @@ static int fm_detect(int input);
 static int fm_init(int input, int voices);
 static void fm_exit(int input);
 static int fm_mixer_volume(int volume);
-static int fm_load_patches(char *patches, char *drums);
+static int fm_load_patches(AL_CONST char *patches, AL_CONST char *drums);
 static void fm_key_on(int inst, int note, int bend, int vol, int pan);
 static void fm_key_off(int voice);
 static void fm_set_volume(int voice, int vol);
 static void fm_set_pitch(int voice, int note, int bend);
 
-static char adlib_desc[160] = EMPTY_STRING;
+static char adlib_desc[256] = EMPTY_STRING;
 
 
 
@@ -601,7 +601,7 @@ END_OF_STATIC_FUNCTION(fm_set_pitch);
  *  Called before starting to play a MIDI file, to check if we need to be
  *  in rhythm mode or not.
  */
-static int fm_load_patches(char *patches, char *drums)
+static int fm_load_patches(AL_CONST char *patches, AL_CONST char *drums)
 {
    int i;
    int usedrums = FALSE;
@@ -653,7 +653,7 @@ static int fm_mixer_volume(int volume)
 /* fm_is_there:
  *  Checks for the presence of an OPL synth at the current port.
  */
-static int fm_is_there()
+static int fm_is_there(void)
 {
    fm_write(1, 0);                        /* init test register */
 
@@ -687,7 +687,7 @@ static int fm_detect(int input)
    static int ports[] = { 0x210, 0x220, 0x230, 0x240, 0x250, 0x260, 0x388, 0 };
    char tmp1[64], tmp2[64];
    int opl_type;
-   char *s;
+   AL_CONST char *s;
    int i;
 
    if (input)
@@ -710,7 +710,7 @@ static int fm_detect(int input)
    }
 
    if (!fm_is_there()) {
-      ustrcpy(allegro_error, get_config_text("OPL synth not found"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("OPL synth not found"));
       return FALSE;
    }
 
@@ -729,13 +729,13 @@ static int fm_detect(int input)
 
    if (midi_card == MIDI_OPL3) {
       if (opl_type != MIDI_OPL3) {
-	 ustrcpy(allegro_error, get_config_text("OPL3 synth not found"));
+	 ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("OPL3 synth not found"));
 	 return FALSE;
       }
    }
    else if (midi_card == MIDI_2XOPL2) {
       if (opl_type != MIDI_2XOPL2) {
-	 ustrcpy(allegro_error, get_config_text("Second OPL2 synth not found"));
+	 ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Second OPL2 synth not found"));
 	 return FALSE;
       }
    }
@@ -749,7 +749,7 @@ static int fm_detect(int input)
    else
       s = get_config_text("OPL3 synth");
 
-   usprintf(adlib_desc, get_config_text("%s on port %X"), s, _fm_port);
+   uszprintf(adlib_desc, sizeof(adlib_desc), get_config_text("%s on port %X"), s, _fm_port);
    midi_driver->desc = adlib_desc;
 
    return TRUE;
@@ -760,7 +760,7 @@ static int fm_detect(int input)
 /* load_ibk:
  *  Reads in a .IBK patch set file, for use by the Adlib driver.
  */
-int load_ibk(char *filename, int drums)
+int load_ibk(AL_CONST char *filename, int drums)
 {
    char sig[4];
    FM_INSTRUMENT *inst;
@@ -853,8 +853,8 @@ int load_ibk(char *filename, int drums)
  */
 static int fm_init(int input, int voices)
 {
-   char tmp1[80], tmp2[80];
-   char *s;
+   char tmp1[128], tmp2[128];
+   AL_CONST char *s;
    int i;
 
    fm_reset(1);
@@ -863,7 +863,7 @@ static int fm_init(int input, int voices)
       s = get_config_string(uconvert_ascii("sound", tmp1), uconvert_ascii(((i == 0) ? "ibk_file" : "ibk_drum_file"), tmp2), NULL);
       if ((s) && (ugetc(s))) {
 	 if (load_ibk(s, (i > 0)) != 0) {
-	    usprintf(allegro_error, get_config_text("Error reading .IBK file '%s'"), s);
+	    uszprintf(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Error reading .IBK file '%s'"), s);
 	    return -1;
 	 }
       }
