@@ -19,11 +19,13 @@
  */
 
 
-#include <dos.h>
+#ifndef SCAN_DEPEND
+   #include <dos.h>
+#endif
 
 #include "allegro.h"
-#include "allegro/aintern.h"
-#include "allegro/aintdos.h"
+#include "allegro/internal/aintern.h"
+#include "allegro/platform/aintdos.h"
 #include "../i386/asmdefs.inc"
 
 
@@ -54,17 +56,26 @@ int __djgpp_ds_alias = 0;
  *  Stop those stupid "abort, retry, fail" messages from popping up all
  *  over the place.
  */
-static int __interrupt __far my_int24()
+#if __WATCOMC__ >= 1200
+
+static void __interrupt __far my_int24(void)
+{}
+
+#else
+
+static int __interrupt __far my_int24(void)
 {
    return 3;
 }
 
+#endif
+
 
 
 /* _dos_irq_init:
  *  Initialises this module.
  */
-void _dos_irq_init()
+void _dos_irq_init(void)
 {
    int c;
 
@@ -112,7 +123,7 @@ void _dos_irq_init()
 /* _dos_irq_exit:
  *  Shuts down this module.
  */
-void _dos_irq_exit()
+void _dos_irq_exit(void)
 {
    int c;
 
@@ -573,7 +584,7 @@ void CALL_RMCB_PART2(void);
    "  mov esp, rmcb_saved_esp "        /* restore the original stack */    \
    "  mov ax, rmcb_saved_ss "                                              \
    "  mov ss, ax "                                                         \
-									   \ 
+									   \
    "  pop es "                         /* restore registers */             \
    "  pop ds "                                                             \
    "  pop edx "                                                            \
