@@ -55,14 +55,21 @@ static char *fs_dlist_getter(int, int *);
 
 #endif
 
-
-#define FLIST_SIZE      2048
+/* Number of entries is limited by available memory
+ * Initial capacity is given by FLIST_START_CAPACITY, structure can grow beyond
+ * this. Normally keeps the structure in memory between invocations, but it
+ * attempts to free memory after processing a directory with more than 
+ * FLIST_UPPER_CAPACITY 
+ */
+#define FLIST_START_CAPACITY 128
+#define FLIST_UPPER_CAPACITY 2048
 
 typedef struct FLIST
 {
    char dir[1024];
    int size;
-   char *name[FLIST_SIZE];
+   int capacity;
+   char** name;
 } FLIST;
 
 static FLIST *flist = NULL;
@@ -166,7 +173,7 @@ static int get_x_drive(int index)
    for (i=0; i<26; i++) {
       if (_al_drive_exists(i)) {
 	 c++;
-	 if (c==index)
+	 if (c == index)
 	    return i;
       }
    }
@@ -427,7 +434,7 @@ static int fs_flist_putter(AL_CONST char *str, int attrib, void *check_attrib)
       }
    }
 
-   if ((flist->size < FLIST_SIZE) && ((ugetc(s) != '.') || (ugetat(s, 1)))) {
+   if (((ugetc(s) != '.') || (ugetat(s, 1)))) {
       int size = ustrsizez(s) + ((attrib & FA_DIREC) ? ucwidth(OTHER_PATH_SEPARATOR) : 0);
       name = malloc(size);
       if (!name)
@@ -451,6 +458,18 @@ static int fs_flist_putter(AL_CONST char *str, int attrib, void *check_attrib)
 	       break;
 	 }
       }
+      /* Do we need to allocate more space in the structure? */
+      /* This doubles the capacity of the array each time, */
+      /* which gives 'linear' compexity */
+      if (flist->size == flist->capacity) {
+         flist->capacity *= 2;
+	 flist->name = _al_sane_realloc(flist->name, sizeof(char *) * flist->capacity);
+	 if (flist->name == NULL) {
+	    *allegro_error = ENOMEM;
+	    /* Stop the enumeration by returning non-zero */
+	    return -1;
+	 }
+      }
 
       /* Shift in preparation for inserting the new entry. */
       for (c2=flist->size; c2>c; c2--)
@@ -520,11 +539,31 @@ static int fs_flist_proc(int msg, DIALOG *d, int c)
 	    *allegro_errno = ENOMEM;
 	    return D_CLOSE; 
 	 }
+	 flist->capacity = FLIST_START_CAPACITY;
+	 flist->name = malloc(flist->capacity * sizeof(char *));
+	 if (!flist->name) {
+	    *allegro_errno = ENOMEM;
+	    return D_CLOSE;
+         }
       }
       else {
-	 for (i=0; i<flist->size; i++)
-	    if (flist->name[i])
+	 for (i=0; i<flist->size; i++) {
+	    if (flist->name[i]) {
 	       free(flist->name[i]);
+	       /* PH add: maybe avoid multiple frees */
+	       flist->name[i] = NULL;
+	    }
+	 }
+	 /* Maybe shrink the structure */
+	 if (flist->capacity > FLIST_UPPER_CAPACITY) {
+	    flist->capacity = FLIST_UPPER_CAPACITY;
+	    flist->name = _al_sane_realloc(flist->name, sizeof(char *) * flist->capacity);
+	    if (!flist) {
+	       /* Oops! Should never happen, I hope */
+	       *allegro_errno = ENOMEM;
+	       return D_CLOSE;
+	    }
+	 }
       }
 
       flist->size = 0;
@@ -553,6 +592,7 @@ static int fs_flist_proc(int msg, DIALOG *d, int c)
 	 for (i=0; i<flist->size; i++)
 	    if (flist->name[i])
 	       free(flist->name[i]);
+	 free(flist->name);
 	 free(flist);
 	 flist = NULL;
       }
@@ -803,11 +843,15 @@ int file_select_ex(AL_CONST char *message, char *path, AL_CONST char *ext, int s
 {
    static attrb_state_t default_attrb_state[ATTRB_MAX] = DEFAULT_ATTRB_STATE;
    int ret;
-   char *p;
+   char *p, *backup;
    char tmp[32];
    ASSERT(message);
    ASSERT(path);
 
+   backup = ustrdup(path);
+   if (!backup)
+      return FALSE;
+
    if (width == OLD_FILESEL_WIDTH)
       width = 305;
 
@@ -878,15 +922,22 @@ int file_select_ex(AL_CONST char *message, char *path, AL_CONST char *ext, int s
       fext_p = NULL;
    }
 
-   if ((ret == FS_CANCEL) || (!ugetc(get_filename(path))))
+   if (ret == FS_CANCEL) {
+      ustrcpy(path, backup);
+      free(backup);
       return FALSE;
+   }
 
-   p = get_extension(path);
-   if ((!ugetc(p)) && (ext) && (!ustrpbrk(ext, uconvert_ascii(" ,;", tmp)))) {
-      size -= ((long)p - (long)path + ucwidth('.'));
-      if (size >= uwidth_max(U_CURRENT) + ucwidth(0)) {  /* do not end with '.' */
-         p += usetc(p, '.');
-         ustrzcpy(p, size, ext);
+   free(backup);
+
+   if (ugetc(get_filename(path))) {
+      p = get_extension(path);
+      if ((!ugetc(p)) && (ext) && (ugetc(ext)) && (!ustrpbrk(ext, uconvert_ascii(" ,;", tmp)))) {
+         size -= ((long)p - (long)path + ucwidth('.'));
+         if (size >= uwidth_max(U_CURRENT) + ucwidth(0)) {  /* do not end with '.' */
+            p += usetc(p, '.');
+            ustrzcpy(p, size, ext);
+         }
       }
    }
 
