@@ -146,7 +146,7 @@ void xor_mode(int on)
 /* solid_mode:
  *  Shortcut function for selecting solid drawing mode.
  */
-void solid_mode()
+void solid_mode(void)
 {
    drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
 }
@@ -166,7 +166,7 @@ void clear_bitmap(BITMAP *bitmap)
 /* vsync:
  *  Waits for a retrace.
  */
-void vsync()
+void vsync(void)
 {
    ASSERT(gfx_driver);
 
@@ -181,6 +181,7 @@ void vsync()
  */
 void set_color(int index, AL_CONST RGB *p)
 {
+   ASSERT(index >= 0 && index < PAL_SIZE);
    set_palette_range((struct RGB *)p-index, index, index, FALSE);
 }
 
@@ -203,6 +204,9 @@ void set_palette_range(AL_CONST PALETTE p, int from, int to, int vsync)
 {
    int c;
 
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE)
+
    for (c=from; c<=to; c++) {
       _current_palette[c] = p[c];
 
@@ -225,7 +229,7 @@ void set_palette_range(AL_CONST PALETTE p, int from, int to, int vsync)
 /* previous palette, so the image loaders can restore it when they are done */
 int _got_prev_current_palette = FALSE;
 PALETTE _prev_current_palette;
-static int prev_palette_color[256];
+static int prev_palette_color[PAL_SIZE];
 
 
 
@@ -239,13 +243,13 @@ void select_palette(AL_CONST PALETTE p)
 {
    int c;
 
-   for (c=0; c<256; c++) {
+   for (c=0; c<PAL_SIZE; c++) {
       _prev_current_palette[c] = _current_palette[c];
       _current_palette[c] = p[c];
    }
 
    if (_color_depth != 8) {
-      for (c=0; c<256; c++) {
+      for (c=0; c<PAL_SIZE; c++) {
 	 prev_palette_color[c] = palette_color[c];
 	 palette_color[c] = makecol(_rgb_scale_6[p[c].r], _rgb_scale_6[p[c].g], _rgb_scale_6[p[c].b]);
       }
@@ -261,18 +265,19 @@ void select_palette(AL_CONST PALETTE p)
 /* unselect_palette:
  *  Restores palette settings from before the last call to select_palette().
  */
-void unselect_palette()
+void unselect_palette(void)
 {
    int c;
 
-   for (c=0; c<256; c++)
+   for (c=0; c<PAL_SIZE; c++)
       _current_palette[c] = _prev_current_palette[c];
 
    if (_color_depth != 8) {
-      for (c=0; c<256; c++)
+      for (c=0; c<PAL_SIZE; c++)
 	 palette_color[c] = prev_palette_color[c];
    }
 
+   ASSERT(_got_prev_current_palette == TRUE);
    _got_prev_current_palette = FALSE;
 
    _current_palette_changed = 0xFFFFFFFF & ~(1<<(_color_depth-1));
@@ -297,7 +302,7 @@ static int *palette_expansion_table(int bpp)
    }
 
    if (_current_palette_changed & (1<<(bpp-1))) {
-      for (c=0; c<256; c++) {
+      for (c=0; c<PAL_SIZE; c++) {
 	 table[c] = makecol_depth(bpp,
 				  _rgb_scale_6[_current_palette[c].r], 
 				  _rgb_scale_6[_current_palette[c].g], 
@@ -325,7 +330,7 @@ void generate_332_palette(PALETTE pal)
 {
    int c;
 
-   for (c=0; c<256; c++) {
+   for (c=0; c<PAL_SIZE; c++) {
       pal[c].r = ((c>>5)&7) * 63/7;
       pal[c].g = ((c>>2)&7) * 63/7;
       pal[c].b = (c&3) * 63/3;
@@ -345,6 +350,8 @@ void generate_332_palette(PALETTE pal)
  */
 void get_color(int index, RGB *p)
 {
+   ASSERT(index >= 0 && index < PAL_SIZE);
+   ASSERT(p);
    get_palette_range(p-index, index, index);
 }
 
@@ -367,6 +374,9 @@ void get_palette_range(PALETTE p, int from, int to)
 {
    int c;
 
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
+
    if ((system_driver) && (system_driver->read_hardware_palette))
       system_driver->read_hardware_palette();
 
@@ -386,6 +396,10 @@ void fade_interpolate(AL_CONST PALETTE source, AL_CONST PALETTE dest, PALETTE ou
 {
    int c;
 
+   ASSERT(pos >= 0 && pos <= 64);
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
+
    for (c=from; c<=to; c++) { 
       output[c].r = ((int)source[c].r * (63-pos) + (int)dest[c].r * pos) / 64;
       output[c].g = ((int)source[c].g * (63-pos) + (int)dest[c].g * pos) / 64;
@@ -405,6 +419,10 @@ void fade_from_range(AL_CONST PALETTE source, AL_CONST PALETTE dest, int speed,
    PALETTE temp;
    int c, start, last;
 
+   ASSERT(speed > 0 && speed <= 64);
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
+
    for (c=0; c<PAL_SIZE; c++)
       temp[c] = source[c];
 
@@ -440,6 +458,9 @@ void fade_from_range(AL_CONST PALETTE source, AL_CONST PALETTE dest, int speed,
  */
 void fade_in_range(AL_CONST PALETTE p, int speed, int from, int to)
 {
+   ASSERT(speed > 0 && speed <= 64);
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
    fade_from_range(black_palette, p, speed, from, to);
 }
 
@@ -454,6 +475,9 @@ void fade_in_range(AL_CONST PALETTE p, int speed, int from, int to)
 void fade_out_range(int speed, int from, int to)
 {
    PALETTE temp;
+   ASSERT(speed > 0 && speed <= 64);
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
 
    get_palette(temp);
    fade_from_range(temp, black_palette, speed, from, to);
@@ -467,6 +491,7 @@ void fade_out_range(int speed, int from, int to)
  */
 void fade_from(AL_CONST PALETTE source, AL_CONST PALETTE dest, int speed)
 {
+   ASSERT(speed > 0 && speed <= 64);
    fade_from_range(source, dest, speed, 0, PAL_SIZE-1);
 }
 
@@ -478,6 +503,7 @@ void fade_from(AL_CONST PALETTE source, AL_CONST PALETTE dest, int speed)
  */
 void fade_in(AL_CONST PALETTE p, int speed)
 {
+   ASSERT(speed > 0 && speed <= 64);
    fade_in_range(p, speed, 0, PAL_SIZE-1);
 }
 
@@ -489,6 +515,7 @@ void fade_in(AL_CONST PALETTE p, int speed)
  */
 void fade_out(int speed)
 {
+   ASSERT(speed > 0 && speed <= 64);
    fade_out_range(speed, 0, PAL_SIZE-1);
 }
 
@@ -497,7 +524,7 @@ void fade_out(int speed)
 /* rect:
  *  Draws an outline rectangle.
  */
-void rect(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
+void _soft_rect(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
 {
    int t;
 
@@ -684,13 +711,71 @@ void do_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int d, void (*proc)(BI
 
 
 /* _normal_line:
+ *  Draws a line from x1, y1 to x2, y2, using putpixel() to do the work.
+ */
+void _normal_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
+{
+   int sx, sy, dx, dy, t;
+
+   if (x1 == x2) {
+      vline(bmp, x1, y1, y2, color);
+      return;
+   }
+
+   if (y1 == y2) {
+      hline(bmp, x1, y1, x2, color);
+      return;
+   }
+
+   /* use a bounding box to check if the line needs clipping */
+   if (bmp->clip) {
+      sx = x1;
+      sy = y1;
+      dx = x2;
+      dy = y2;
+
+      if (sx > dx) {
+	 t = sx;
+	 sx = dx;
+	 dx = t;
+      }
+
+      if (sy > dy) {
+	 t = sy;
+	 sy = dy;
+	 dy = t;
+      }
+
+      if ((sx >= bmp->cr) || (sy >= bmp->cb) || (dx < bmp->cl) || (dy < bmp->ct))
+	 return;
+
+      if ((sx >= bmp->cl) && (sy >= bmp->ct) && (dx < bmp->cr) && (dy < bmp->cb))
+	 bmp->clip = FALSE;
+
+      t = TRUE;
+   }
+   else
+      t= FALSE;
+
+   acquire_bitmap(bmp);
+
+   do_line(bmp, x1, y1, x2, y2, color, bmp->vtable->putpixel);
+
+   release_bitmap(bmp);
+
+   bmp->clip = t;
+}
+
+
+
+/* _fast_line:
  *  Draws a line from x1, y1 to x2, y2, using putpixel() to do the work.
  *  This is an implementation of the Cohen-Sutherland line clipping algorithm.
  *  Loops over the line until it can be either trivially rejected or trivially
  *  accepted. If it is neither rejected nor accepted, subdivide it into two
  *  segments, one of which can be rejected.
  */
-void _normal_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
+void _fast_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
 {
    int code0, code1;
    int outcode;
@@ -876,7 +961,7 @@ void do_circle(BITMAP *bmp, int x, int y, int radius, int d, void (*proc)(BITMAP
 /* circle:
  *  Draws a circle.
  */
-void circle(BITMAP *bmp, int x, int y, int radius, int color)
+void _soft_circle(BITMAP *bmp, int x, int y, int radius, int color)
 {
    int clip, sx, sy, dx, dy;
    ASSERT(bmp);
@@ -912,7 +997,7 @@ void circle(BITMAP *bmp, int x, int y, int radius, int color)
 /* circlefill:
  *  Draws a filled circle.
  */
-void circlefill(BITMAP *bmp, int x, int y, int radius, int color)
+void _soft_circlefill(BITMAP *bmp, int x, int y, int radius, int color)
 {
    int cx = 0;
    int cy = radius;
@@ -1089,7 +1174,7 @@ void do_ellipse(BITMAP *bmp, int x, int y, int rx, int ry, int d, void (*proc)(B
 /* ellipse:
  *  Draws an ellipse.
  */
-void ellipse(BITMAP *bmp, int x, int y, int rx, int ry, int color)
+void _soft_ellipse(BITMAP *bmp, int x, int y, int rx, int ry, int color)
 {
    int clip, sx, sy, dx, dy;
    ASSERT(bmp);
@@ -1125,7 +1210,7 @@ void ellipse(BITMAP *bmp, int x, int y, int rx, int ry, int color)
 /* ellipsefill:
  *  Draws a filled ellipse.
  */
-void ellipsefill(BITMAP *bmp, int x, int y, int rx, int ry, int color)
+void _soft_ellipsefill(BITMAP *bmp, int x, int y, int rx, int ry, int color)
 {
    int ix, iy;
    int a, b, c, d;
@@ -1452,7 +1537,7 @@ void do_arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int d, voi
 /* arc:
  *  Draws an arc.
  */
-void arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int color)
+void _soft_arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int color)
 {
    ASSERT(bmp);
    acquire_bitmap(bmp);
