@@ -23,6 +23,12 @@
 
 
 
+#define PREFIX_I                "al-gfx INFO: "
+#define PREFIX_W                "al-gfx WARNING: "
+#define PREFIX_E                "al-gfx ERROR: "
+
+
+
 static int gfx_virgin = TRUE;          /* is the graphics system active? */
 
 int _sub_bitmap_id_count = 1;          /* hash value for sub-bitmaps */
@@ -133,6 +139,11 @@ static int failed_bitmap_h = BMP_MAX_SIZE;
 
 
 
+static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_config);
+static int _set_gfx_mode_safe(int card, int w, int h, int v_w, int v_h);
+
+
+
 /* lock_bitmap:
  *  Locks all the memory used by a bitmap structure.
  */
@@ -585,69 +596,48 @@ static int get_config_gfx_driver(char *gfx_card, int w, int h, int v_w, int v_h,
  */
 int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 {
-   static int allow_config = TRUE;
-   extern void blit_end(void);
+   TRACE(PREFIX_I "Called set_gfx_mode(%d, %d, %d, %d, %d).\n",
+	 card, w, h, v_w, v_h);
+         
+   /* TODO: shouldn't this be incremented only IF successful? */
+   _gfx_mode_set_count++;
+
+   /* special bodge for the GFX_SAFE driver */
+   if (card == GFX_SAFE)
+      return _set_gfx_mode_safe(card, w, h, v_w, v_h);
+   else
+      return _set_gfx_mode(card, w, h, v_w, v_h, TRUE);
+}
+
+
+
+/* _set_gfx_mode:
+ *  Called by set_gfx_mode(). Separated to make a clear difference between
+ *  the virtual GFX_SAFE driver and the rest. The allow_config parameter,
+ *  if true, allows the configuration to override the graphics card/driver
+ *  when using GFX_AUTODETECT.
+ */
+static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_config)
+{
    _DRIVER_INFO *driver_list;
    GFX_DRIVER *drv;
-   struct GFX_MODE mode;
-   char buf[ALLEGRO_ERROR_SIZE], tmp1[64], tmp2[64];
+   char tmp1[64], tmp2[64];
    AL_CONST char *dv;
    int flags = 0;
    int c, driver, ret;
    ASSERT(system_driver);
-
-   _gfx_mode_set_count++;
-
-   /* special bodge for the GFX_SAFE driver */
-   if (card == GFX_SAFE) {
-      if (system_driver->get_gfx_safe_mode) {
-	 ustrzcpy(buf, sizeof(buf), allegro_error);
-
-	 /* retrieve the safe graphics mode */
-	 system_driver->get_gfx_safe_mode(&driver, &mode);
-
-	 /* try using the specified depth and resolution */
-	 if (set_gfx_mode(driver, w, h, 0, 0) == 0)
-	    return 0;
-
-	 ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, buf);
-
-	 /* finally use the safe settings */
-	 set_color_depth(mode.bpp);
-	 if (set_gfx_mode(driver, mode.width, mode.height, 0, 0) == 0)
-	    return 0;
-
-	 ASSERT(FALSE);  /* the safe graphics mode must always work */
-      }
-      else {
-	 /* no safe graphics mode, try hard-coded autodetected modes with custom settings */
-	 allow_config = FALSE;
-	 _safe_gfx_mode_change = 1;
-
-	 ret = set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0);
-
-	 _safe_gfx_mode_change = 0;
-	 allow_config = TRUE;
-
-	 if (ret == 0)
-	    return 0;
-      }
-
-      /* failing to set GFX_SAFE is a fatal error */
-      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-      allegro_message(uconvert_ascii("%s\n", tmp1), get_config_text("Fatal error: unable to set GFX_SAFE"));
-      return -1;
-   }
+   ASSERT(card != GFX_SAFE);
 
    /* remember the current console state */
    if (gfx_virgin) {
+      TRACE(PREFIX_I "Firt call, remembering console state.\n");
       LOCK_FUNCTION(_stub_bank_switch);
       LOCK_FUNCTION(blit);
 
       if (system_driver->save_console_state)
 	 system_driver->save_console_state();
 
-      _add_exit_func(shutdown_gfx);
+      _add_exit_func(shutdown_gfx, "shutdown_gfx");
 
       gfx_virgin = FALSE;
    }
@@ -661,6 +651,8 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 
    /* close down any existing graphics driver */
    if (gfx_driver) {
+      TRACE(PREFIX_I "Closing graphics driver (%p) ", gfx_driver);
+      TRACE("%s.\n", gfx_driver->ascii_name);
       if (_al_linker_mouse)
          _al_linker_mouse->show_mouse(NULL);
 
@@ -712,6 +704,7 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 
    /* return to text mode? */
    if (card == GFX_TEXT) {
+      TRACE(PREFIX_I "Closing, restoring original console state.\n");
       if (system_driver->restore_console_state)
 	 system_driver->restore_console_state();
 
@@ -723,6 +716,7 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
       if (system_driver->display_switch_lock)
 	 system_driver->display_switch_lock(FALSE, FALSE);
 
+      TRACE(PREFIX_I "Graphic mode closed.\n");
       return 0;
    }
 
@@ -762,6 +756,7 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 
       /* go through the list of autodetected drivers if none was previously found */
       if (!found) {
+	 TRACE(PREFIX_I "Autodetecing graphic driver.\n");
 	 for (c=0; driver_list[c].driver; c++) {
 	    if (driver_list[c].autodetect) {
 	       drv = driver_list[c].driver;
@@ -775,6 +770,10 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 	    }
 	 }
       }
+      else {
+	 TRACE(PREFIX_I "GFX_AUTODETECT overriden through configuration:"
+	       " %s.\n", tmp1);
+      }
    }
    else {
       /* search the list for the requested driver */
@@ -794,6 +793,7 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
       if (system_driver->display_switch_lock)
 	 system_driver->display_switch_lock(FALSE, FALSE);
 
+      TRACE(PREFIX_E "Failed setting graphic driver %d.", card);
       return -1;
    }
 
@@ -816,6 +816,13 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
    else
       _wait_for_vsync = TRUE;
 
+   TRACE(PREFIX_I "The driver %s wait for vsync.\n",
+	 (_wait_for_vsync) ? "will" : "won't");
+
+   /* Give the gfx driver an opportunity to set the drawing mode */
+   if ((gfx_driver->drawing_mode) && (!_dispsw_status))
+      gfx_driver->drawing_mode();
+
    clear_bitmap(screen);
 
    /* set up the default colors */
@@ -845,11 +852,77 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
    if (system_driver->display_switch_lock)
       system_driver->display_switch_lock(FALSE, FALSE);
 
+   TRACE(PREFIX_I "set_gfx_card success for %dx%dx%d.\n",
+	 screen->w, screen->h, bitmap_color_depth(screen));
    return 0;
 } 
 
 
 
+/* _set_gfx_mode_safe:
+ *  Special wrapper used when the card parameter of set_gfx_mode()
+ *  is GFX_SAFE. In this case the function tries to query the
+ *  system driver for a "safe" resolution+driver it knows it will
+ *  work, and set it. If the system driver cannot get a "safe"
+ *  resolution+driver, it will try the given parameters.
+ */
+static int _set_gfx_mode_safe(int card, int w, int h, int v_w, int v_h)
+{
+   char buf[ALLEGRO_ERROR_SIZE], tmp1[64];
+   struct GFX_MODE mode;
+   static int allow_config = TRUE;
+   int ret, driver;
+
+   ASSERT(card == GFX_SAFE);
+   ASSERT(system_driver);
+   TRACE(PREFIX_I "Trying to set a safe graphics mode.\n");
+   
+   if (system_driver->get_gfx_safe_mode) {
+      ustrzcpy(buf, sizeof(buf), allegro_error);
+
+      /* retrieve the safe graphics mode */
+      system_driver->get_gfx_safe_mode(&driver, &mode);
+      TRACE(PREFIX_I "The system driver suggests %dx%dx%d\n",
+	    mode.width, mode.height, mode.bpp);
+
+      /* try using the specified resolution but current depth */
+      if (_set_gfx_mode(driver, w, h, 0, 0, TRUE) == 0)
+         return 0;
+
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, buf);
+
+      /* finally use the safe settings */
+      set_color_depth(mode.bpp);
+      if (_set_gfx_mode(driver, mode.width, mode.height, 0, 0, TRUE) == 0)
+         return 0;
+
+      ASSERT(FALSE);  /* the safe graphics mode must always work */
+   }
+   else {
+      TRACE(PREFIX_W "The system driver was unable to get a safe mode, "
+	    "I'll try with the specified parameters...\n");
+      /* no safe graphics mode, try hard-coded autodetected modes with
+       * custom settings */
+      _safe_gfx_mode_change = 1;
+
+      ret = _set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0, TRUE);
+
+      _safe_gfx_mode_change = 0;
+
+      if (ret == 0)
+         return 0;
+   }
+
+   /* failing to set GFX_SAFE is a fatal error */
+   TRACE(PREFIX_E "Bad bad, not even GFX_SAFE works?\n");
+   _set_gfx_mode(GFX_TEXT, 0, 0, 0, 0, TRUE);
+   allegro_message(uconvert_ascii("%s\n", tmp1),
+		   get_config_text("Fatal error: unable to set GFX_SAFE"));
+   return -1;
+}
+
+
+
 /* _sort_out_virtual_width:
  *  Decides how wide the virtual screen really needs to be. That is more 
  *  complicated than it sounds, because the Allegro graphics primitives 
@@ -888,7 +961,7 @@ void _sort_out_virtual_width(int *width, GFX_DRIVER *driver)
  *  switching table using bank size/granularity information from the 
  *  specified graphics driver.
  */
-BITMAP *_make_bitmap(int w, int h, unsigned long addr, GFX_DRIVER *driver, int color_depth, int bpl)
+BITMAP *_make_bitmap(int w, int h, uintptr_t addr, GFX_DRIVER *driver, int color_depth, int bpl)
 {
    GFX_VTABLE *vtable = _get_vtable(color_depth);
    int i, bank, size;
@@ -970,7 +1043,7 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
    BITMAP *bitmap;
    int i;
    ASSERT(width >= 0);
-   ASSERT(height >= 0);
+   ASSERT(height > 0);
    ASSERT(system_driver);
 
    if (system_driver->create_bitmap)
@@ -1002,9 +1075,11 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
    bitmap->y_ofs = 0;
    bitmap->seg = _default_ds();
 
-   bitmap->line[0] = bitmap->dat;
-   for (i=1; i<height; i++)
-      bitmap->line[i] = bitmap->line[i-1] + width * BYTES_PER_PIXEL(color_depth);
+   if (height > 0) {
+      bitmap->line[0] = bitmap->dat;
+      for (i=1; i<height; i++)
+         bitmap->line[i] = bitmap->line[i-1] + width * BYTES_PER_PIXEL(color_depth);
+   }
 
    if (system_driver->created_bitmap)
       system_driver->created_bitmap(bitmap);
@@ -1020,7 +1095,7 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
 BITMAP *create_bitmap(int width, int height)
 {
    ASSERT(width >= 0);
-   ASSERT(height >= 0);
+   ASSERT(height > 0);
    return create_bitmap_ex(_color_depth, width, height);
 }
 
@@ -1194,6 +1269,9 @@ BITMAP *create_video_bitmap(int width, int height)
    BITMAP *bmp;
    int x = 0, y = 0;
 
+   ASSERT(width >= 0);
+   ASSERT(height > 0);
+   
    if (_dispsw_status)
       return NULL;
 
@@ -1319,6 +1397,8 @@ BITMAP *create_system_bitmap(int width, int height)
 {
    BITMAP *bmp;
 
+   ASSERT(width >= 0);
+   ASSERT(height > 0);
    ASSERT(gfx_driver != NULL);
 
    if (gfx_driver->create_system_bitmap)
@@ -1408,6 +1488,53 @@ void destroy_bitmap(BITMAP *bitmap)
 
 
 
+/* set_clip_rect:
+ *  Sets the two opposite corners of the clipping rectangle to be used when
+ *  drawing to the bitmap. Nothing will be drawn to positions outside of this 
+ *  rectangle. When a new bitmap is created the clipping rectangle will be 
+ *  set to the full area of the bitmap.
+ */
+void set_clip_rect(BITMAP *bitmap, int x1, int y1, int x2, int y2)
+{
+   ASSERT(bitmap);
+
+   /* internal clipping is inclusive-exclusive */
+   x2++;
+   y2++;
+
+   bitmap->cl = MID(0, x1, bitmap->w-1);
+   bitmap->ct = MID(0, y1, bitmap->h-1);
+   bitmap->cr = MID(0, x2, bitmap->w);
+   bitmap->cb = MID(0, y2, bitmap->h);
+
+   if (bitmap->vtable->set_clip)
+      bitmap->vtable->set_clip(bitmap);
+}
+
+
+
+/* add_clip_rect:
+ *  Makes the new clipping rectangle the intersection between the given
+ *  rectangle and the current one.
+ */
+void add_clip_rect(BITMAP *bitmap, int x1, int y1, int x2, int y2)
+{
+   int cx1, cy1, cx2, cy2;
+
+   ASSERT(bitmap);
+
+   get_clip_rect(bitmap, &cx1, &cy1, &cx2, &cy2);
+
+   x1 = MAX(x1, cx1);
+   y1 = MAX(y1, cy1);
+   x2 = MIN(x2, cx2);
+   y2 = MIN(y2, cy2);
+
+   set_clip_rect(bitmap, x1, y1, x2, y2);
+}
+
+
+
 /* set_clip:
  *  Sets the two opposite corners of the clipping rectangle to be used when
  *  drawing to the bitmap. Nothing will be drawn to positions outside of this 
@@ -1424,14 +1551,8 @@ void set_clip(BITMAP *bitmap, int x1, int y1, int x2, int y2)
    ASSERT(bitmap);
 
    if ((!x1) && (!y1) && (!x2) && (!y2)) {
-      bitmap->clip = FALSE;
-      bitmap->cl = bitmap->ct = 0;
-      bitmap->cr = bitmap->w;
-      bitmap->cb = bitmap->h;
-
-      if (bitmap->vtable->set_clip)
-	 bitmap->vtable->set_clip(bitmap);
-
+      set_clip_rect(bitmap, 0, 0, bitmap->w-1, bitmap->h-1);
+      set_clip_state(bitmap, FALSE);
       return;
    }
 
@@ -1447,17 +1568,8 @@ void set_clip(BITMAP *bitmap, int x1, int y1, int x2, int y2)
       y2 = t;
    }
 
-   x2++;
-   y2++;
-
-   bitmap->clip = TRUE;
-   bitmap->cl = MID(0, x1, bitmap->w-1);
-   bitmap->ct = MID(0, y1, bitmap->h-1);
-   bitmap->cr = MID(0, x2, bitmap->w);
-   bitmap->cb = MID(0, y2, bitmap->h);
-
-   if (bitmap->vtable->set_clip)
-      bitmap->vtable->set_clip(bitmap);
+   set_clip_rect(bitmap, x1, y1, x2, y2);
+   set_clip_state(bitmap, TRUE);
 }
 
 
@@ -1559,7 +1671,7 @@ int request_scroll(int x, int y)
 /* poll_scroll:
  *  Checks whether a requested triple buffer flip has actually taken place.
  */
-int poll_scroll()
+int poll_scroll(void)
 {
    if ((!gfx_driver->poll_scroll) || (_dispsw_status))
       return FALSE;
@@ -1611,7 +1723,7 @@ int request_video_bitmap(BITMAP *bitmap)
  *  Asks a driver to turn on triple buffering mode, if it is capable
  *  of that.
  */
-int enable_triple_buffer()
+int enable_triple_buffer(void)
 {
    if (gfx_capabilities & GFX_CAN_TRIPLE_BUFFER)
       return 0;
