@@ -25,7 +25,7 @@
  *  Draws a lit or tinted sprite, interpolating the four corner colors
  *  over the surface of the image.
  */
-void draw_gouraud_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int c1, int c2, int c3, int c4)
+void _soft_draw_gouraud_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int c1, int c2, int c3, int c4)
 {
    fixed mc1, mc2, mh;
    fixed lc, rc, hc;
@@ -35,7 +35,7 @@ void draw_gouraud_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int c1, int
    int y2 = y + sprite->h;
    int i, j;
    int pixel;
-   unsigned long addr;
+   uintptr_t addr;
 
    ASSERT(bmp);
    ASSERT(sprite);
@@ -70,7 +70,7 @@ void draw_gouraud_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int c1, int
       if ((bmp->clip) && (x < bmp->cl))
 	 hc += mh * (bmp->cl - x);
 
-   #ifdef GFX_MODEX
+   #ifdef GFX_HAS_VGA
 
       /* modex version */
       if (is_planar_bitmap(bmp)) {
@@ -155,7 +155,7 @@ void draw_gouraud_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int c1, int
 	 #ifdef ALLEGRO_COLOR32
 
 	    case 32:
-	       addr = bmp_write_line(bmp, j) + x1*sizeof(long);
+	       addr = bmp_write_line(bmp, j) + x1*sizeof(int32_t);
 	       for (i=x1; i<x2; i++) {
 		  pixel = ((unsigned long *)sprite->line[j-y])[i-x];
 		  if (pixel != MASK_COLOR_32) {
@@ -163,7 +163,7 @@ void draw_gouraud_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int c1, int
 		     bmp_write32(addr, pixel);
 		  }
 		  hc += mh;
-		  addr += sizeof(long);
+		  addr += sizeof(int32_t);
 	       }
 	       break;
 
@@ -178,4 +178,3 @@ void draw_gouraud_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int c1, int
 
    bmp_unwrite_line(bmp);
 }
-
