@@ -18,6 +18,7 @@
  */
 #define ALLEGRO_NO_CLEAR_BITMAP_ALIAS
 #define ALLEGRO_NO_FIX_ALIASES
+#define ALLEGRO_NO_VHLINE_ALIAS 
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -117,7 +118,7 @@ offset_entry_t list[] = {
   {"BMP_ID_MASK",     BMP_ID_MASK},
   {"#endif", 0 },
   {"NEWLINE", 0},
-#ifndef ALLEGRO_USE_C
+#if !defined ALLEGRO_USE_C && defined ALLEGRO_I386
   {"CMP_PLANAR",      (int)offsetof(COMPILED_SPRITE, planar)},
   {"CMP_COLOR_DEPTH", (int)offsetof(COMPILED_SPRITE, color_depth)},
   {"CMP_DRAW",        (int)offsetof(COMPILED_SPRITE, proc)},
@@ -235,6 +236,8 @@ offset_entry_t list[] = {
 #endif
 #ifdef ALLEGRO_WATCOM
   {"#define FUNC(name)            .globl " PREFIX "name ; nop ; _align_ ; " PREFIX "name:", 0},
+#elif defined ALLEGRO_UNIX
+  {"#define FUNC(name)            .globl " PREFIX "name ; _align_ ; .type " PREFIX "name,@function ; " PREFIX "name:", 0},
 #else
   {"#define FUNC(name)            .globl " PREFIX "name ; _align_ ; " PREFIX "name:", 0},
 #endif
