@@ -28,7 +28,7 @@ _DRIVER_INFO _linux_mouse_driver_list[] =
    {  MOUSEDRV_LINUX_IMS,      &mousedrv_linux_ims,      TRUE  },
    {  MOUSEDRV_LINUX_PS2,      &mousedrv_linux_ps2,      TRUE  },
    {  MOUSEDRV_LINUX_IPS2,     &mousedrv_linux_ips2,     TRUE  },
-#ifdef ALLEGRO_HAVE_LINUX_INPUT_H
+#ifdef HAVE_LINUX_INPUT_H
    {  MOUSEDRV_LINUX_EVDEV,    &mousedrv_linux_evdev,    TRUE  },
 #endif
    {  MOUSEDRV_NONE,           &mousedrv_none,           TRUE  },
