@@ -17,13 +17,17 @@
 
 
 #include "allegro.h"
-#include "allegro/aintunix.h"
+#include "allegro/platform/aintunix.h"
 
 
 /* list the available drivers */
 _DRIVER_INFO _linux_timer_driver_list[] =
 {
-   {  TIMERDRV_UNIX,     &timerdrv_unix,     TRUE  },
-   {  0,                 NULL,               0     }
+#ifdef HAVE_LIBPTHREAD
+   {  TIMERDRV_UNIX_PTHREADS,  &timerdrv_unix_pthreads, TRUE  },
+#else
+   {  TIMERDRV_UNIX_SIGALRM,   &timerdrv_unix_sigalrm,  TRUE  },
+#endif
+   {  0,                       NULL,                    0     }
 };
 
