@@ -85,7 +85,8 @@ KEYBOARD_DRIVER keyboard_macosx =
    NULL,   // AL_METHOD(void, set_rate, (int delay, int rate));
    NULL,   // AL_METHOD(void, wait_for_input, (void));
    NULL,   // AL_METHOD(void, stop_waiting_for_input, (void));
-   NULL    // AL_METHOD(int,  scancode_to_ascii, (int scancode));
+   NULL,   // AL_METHOD(int,  scancode_to_ascii, (int scancode));
+   NULL    // scancode_to_name
 };
 
 
@@ -108,11 +109,9 @@ void osx_keyboard_handler(int pressed, NSEvent *event)
 	 else
             _handle_key_press(character, scancode);
       }
-      if (((three_finger_flag) &&
-	  (scancode == KEY_END) && (_key_shifts & (KB_CTRL_FLAG | KB_ALT_FLAG))) ||
-	  ((scancode == KEY_Q) && (_key_shifts & KB_COMMAND_FLAG))) {
-	 [NSApp sendEvent: event];
-	 [[NSApp delegate] app_quit: nil];
+      if ((three_finger_flag) &&
+	  (scancode == KEY_END) && (_key_shifts & (KB_CTRL_FLAG | KB_ALT_FLAG))) {
+	 raise(SIGTERM);
       }
    }
    else
