@@ -37,18 +37,6 @@ static char *arg0, *arg1 = NULL;
 static int refresh_rate = 70;
 
 
-/* These are used to warn the dock about the application */
-typedef struct CPSProcessSerNum
-{
-   UInt32 lo;
-   UInt32 hi;
-} CPSProcessSerNum;
-
-extern OSErr CPSGetCurrentProcess( CPSProcessSerNum *psn);
-extern OSErr CPSEnableForegroundOperation( CPSProcessSerNum *psn, UInt32 _arg2, UInt32 _arg3, UInt32 _arg4, UInt32 _arg5);
-extern OSErr CPSSetFrontProcess( CPSProcessSerNum *psn);
-
-
 
 @implementation AllegroAppDelegate
 
@@ -71,10 +59,12 @@ extern OSErr CPSSetFrontProcess( CPSProcessSerNum *psn);
    FSCatalogInfo processInfo;
    ProcessSerialNumber psn = { 0, kCurrentProcess };
    CFDictionaryRef mode;
-   char path[1024], *p;
-   int i;
+   NSString* exename, *resdir;
+   NSFileManager* fm;
+   BOOL isDir;
 
-   pthread_mutex_init(&osx_event_mutex, NULL);
+   /* create mutex */
+    osx_event_mutex=_unix_create_mutex();
    
    pool = [[NSAutoreleasePool alloc] init];
    
@@ -84,14 +74,25 @@ extern OSErr CPSSetFrontProcess( CPSProcessSerNum *psn);
    GetProcessBundleLocation(&psn, &processRef);
    FSGetCatalogInfo(&processRef, kFSCatInfoNodeFlags, &processInfo, NULL, NULL, NULL);
    if (processInfo.nodeFlags & kFSNodeIsDirectoryMask) {
-      strncpy(path, __crt0_argv[0], sizeof(path));
-      for (i = 0; i < 4; i++) {
-         for (p = path + strlen(path); (p >= path) && (*p != '/'); p--);
-         *p = '\0';
-      }
-      chdir(path);
+      /* In a bundle, so chdir to the containing directory,
+       * or to the 'magic' resource directory if it exists.
+       * (see the readme.osx file for more info)
+       */
       osx_bundle = [NSBundle mainBundle];
-      arg0 = strdup([[osx_bundle bundlePath] lossyCString]);
+      exename = [[osx_bundle executablePath] lastPathComponent];
+      resdir = [[osx_bundle resourcePath] stringByAppendingPathComponent: exename];
+      resdir = [resdir stringByStandardizingPath];
+      fm = [NSFileManager defaultManager];
+      if ([fm fileExistsAtPath: resdir isDirectory: &isDir] && isDir) {
+          /* Yes, it exists inside the bundle */
+          [fm changeCurrentDirectoryPath: resdir];
+      }
+      else {
+          /* No, change to the directory containing the bundle*/
+          [fm changeCurrentDirectoryPath: [osx_bundle bundlePath]];
+      }
+
+      arg0 = strdup([[osx_bundle bundlePath] fileSystemRepresentation]);
       if (arg1) {
          static char *args[2];
 	 args[0] = arg0;
@@ -104,6 +105,7 @@ extern OSErr CPSSetFrontProcess( CPSProcessSerNum *psn);
          __crt0_argc = 1;
       }
    }
+   /* else: not in a bundle so don't chdir */
    
    mode = CGDisplayCurrentMode(kCGDirectMainDisplay);
    CFNumberGetValue(CFDictionaryGetValue(mode, kCGDisplayRefreshRate), kCFNumberSInt32Type, &refresh_rate);
@@ -115,22 +117,22 @@ extern OSErr CPSSetFrontProcess( CPSProcessSerNum *psn);
       withObject: nil];
    
    while (1) {
-      pthread_mutex_lock(&osx_event_mutex);
-      osx_event_handler();
       if (osx_gfx_mode == OSX_GFX_WINDOW)
          osx_update_dirty_lines();
-      else if (osx_gfx_mode == OSX_GFX_FULL) {
+      _unix_lock_mutex(osx_event_mutex);
+      if (osx_gfx_mode == OSX_GFX_FULL) {
          if ((osx_palette) && (osx_palette_dirty)) {
             CGDisplaySetPalette(kCGDirectMainDisplay, osx_palette);
 	    osx_palette_dirty = FALSE;
 	 }
       }
-      pthread_mutex_unlock(&osx_event_mutex);
+      osx_event_handler();
+      _unix_unlock_mutex(osx_event_mutex);
       usleep(1000000 / refresh_rate);
    }
    
    [pool release];
-   pthread_mutex_destroy(&osx_event_mutex);
+   _unix_destroy_mutex(osx_event_mutex);
 }
 
 
@@ -156,34 +158,51 @@ extern OSErr CPSSetFrontProcess( CPSProcessSerNum *psn);
 
 
 
+/* Call the user main() */
+static void call_user_main(void)
+{
+   int (*real_main)(int, char*[]) = (int (*)(int, char*[])) _mangled_main_address;
+   exit(real_main(__crt0_argc, __crt0_argv));
+}
+
+
+
 /* app_main:
  *  Thread dedicated to the user program; real main() gets called here.
  */
 + (void)app_main: (id)arg
 {
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
-   int (*real_main) (int, char*[]) = (int (*) (int, char*[])) _mangled_main_address;
-   
-   /* Call the user main() */
-   exit(real_main(__crt0_argc, __crt0_argv));
-   
+   call_user_main();
    [pool release];
 }
 
 
 
 /* app_quit:
- *  Forces application to quit by raising a SIGTERM signal.
+ *  Called upon Command-Q or "Quit" menu item selection.
+ *  If the window close callback is set, calls it, otherwise behaves like
+ *  Ctrl-Alt-End.
  */
 - (void)app_quit: (id)sender
 {
-   raise(SIGTERM);
+   if (osx_window_close_hook)
+      osx_window_close_hook();
 }
 
 @end
 
 
 
+/* This prevents warnings that 'NSApplication might not
+ * respond to setAppleMenu' on OS X 10.4
+ */
+@interface NSApplication(AllegroOSX)
+- (void)setAppleMenu:(NSMenu *)menu;
+@end
+
+
+
 /* main:
  *  Replacement for main function.
  */
@@ -191,22 +210,15 @@ int main(int argc, char *argv[])
 {
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    AllegroAppDelegate *app_delegate = [[AllegroAppDelegate alloc] init];
-   CPSProcessSerNum psn;
    NSMenu *menu;
    NSMenuItem *menu_item, *temp_item;
    
    __crt0_argc = argc;
    __crt0_argv = argv;
    
-   [NSApplication sharedApplication];
+   if (!osx_bootstrap_ok()) /* not safe to use NSApplication */
+      call_user_main();
       
-   /* Tell the dock about us; the origins of this hack are unknown, but it's
-    * currently the only way to make a Cocoa app to work when started from a
-    * console.
-    */
-   if ((!CPSGetCurrentProcess(&psn)) &&
-       (!CPSEnableForegroundOperation(&psn, 0x03, 0x3C, 0x2C, 0x1103)) &&
-       (!CPSSetFrontProcess(&psn)))
       [NSApplication sharedApplication];
    
    /* Creates a custom application menu */
