@@ -29,8 +29,9 @@ static BITMAP *osx_qz_window_init(int, int, int, int, int);
 static void osx_qz_window_exit(BITMAP *);
 static void osx_qz_window_vsync(void);
 static void osx_qz_window_set_palette(AL_CONST struct RGB *, int, int, int);
+static void osx_signal_vsync(void);
 
-
+static pthread_mutex_t vsync_mutex;
 static pthread_cond_t vsync_cond;
 static int lock_nesting = 0;
 static AllegroWindowDelegate *window_delegate = NULL;
@@ -49,7 +50,7 @@ static char *dirty_lines = NULL;
 static GFX_VTABLE _special_vtable; /* special vtable for offscreen bitmap */
 
 
-pthread_mutex_t osx_window_mutex;
+void* osx_window_mutex;
 
 
 GFX_DRIVER gfx_quartz_window =
@@ -72,13 +73,14 @@ GFX_DRIVER gfx_quartz_window =
    NULL,                         /* AL_METHOD(int, request_video_bitmap, (BITMAP *bitmap)); */
    NULL,                         /* AL_METHOD(BITMAP *, create_system_bitmap, (int width, int height)); */
    NULL,                         /* AL_METHOD(void, destroy_system_bitmap, (BITMAP *bitmap)); */
-   NULL,                         /* AL_METHOD(int, set_mouse_sprite, (BITMAP *sprite, int xfocus, int yfocus)); */
-   NULL,                         /* AL_METHOD(int, show_mouse, (BITMAP *bmp, int x, int y)); */
-   NULL,                         /* AL_METHOD(void, hide_mouse, (void)); */
-   NULL,                         /* AL_METHOD(void, move_mouse, (int x, int y)); */
+   osx_mouse_set_sprite,         /* AL_METHOD(int, set_mouse_sprite, (BITMAP *sprite, int xfocus, int yfocus)); */
+   osx_mouse_show,               /* AL_METHOD(int, show_mouse, (BITMAP *bmp, int x, int y)); */
+   osx_mouse_hide,               /* AL_METHOD(void, hide_mouse, (void)); */
+   osx_mouse_move,               /* AL_METHOD(void, move_mouse, (int x, int y)); */
    NULL,                         /* AL_METHOD(void, drawing_mode, (void)); */
    NULL,                         /* AL_METHOD(void, save_video_state, (void)); */
    NULL,                         /* AL_METHOD(void, restore_video_state, (void)); */
+   NULL,                         /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    NULL,                         /* AL_METHOD(int, fetch_mode_list, (void)); */
    0, 0,                         /* physical (not virtual!) screen size */
    TRUE,                         /* true if video memory is linear */
@@ -105,14 +107,14 @@ static void prepare_window_for_animation(int refresh_view)
    unsigned int *addr;
    int pitch, y, x;
    
-   pthread_mutex_lock(&osx_window_mutex);
+   _unix_lock_mutex(osx_window_mutex);
    while (![qd_view lockFocusIfCanDraw]);
    while (!QDDone([qd_view qdPort]));
    LockPortBits([qd_view qdPort]);
    pitch = GetPixRowBytes(GetPortPixMap([qd_view qdPort])) / 4;
    addr = (unsigned int *)GetPixBaseAddr(GetPortPixMap([qd_view qdPort])) +
       ((int)([osx_window frame].size.height) - gfx_quartz_window.h) * pitch;
-   if (refresh_view) {
+   if (refresh_view && colorconv_blitter) {
       src_gfx_rect.width  = gfx_quartz_window.w;
       src_gfx_rect.height = gfx_quartz_window.h;
       src_gfx_rect.pitch  = pseudo_screen_pitch;
@@ -128,7 +130,7 @@ static void prepare_window_for_animation(int refresh_view)
    }
    UnlockPortBits([qd_view qdPort]);
    [qd_view unlockFocus];
-   pthread_mutex_unlock(&osx_window_mutex);
+   _unix_unlock_mutex(osx_window_mutex);
 }
 
 
@@ -182,9 +184,9 @@ static void prepare_window_for_animation(int refresh_view)
  */
 - (void)windowDidDeminiaturize: (NSNotification *)aNotification
 {
-   pthread_mutex_lock(&osx_window_mutex);
+   _unix_lock_mutex(osx_window_mutex);
    memset(dirty_lines, 1, gfx_quartz_window.h);
-   pthread_mutex_unlock(&osx_window_mutex);
+   _unix_unlock_mutex(osx_window_mutex);
 }
 
 @end
@@ -216,10 +218,11 @@ static void osx_qz_acquire_win(BITMAP *bmp)
 {
    /* to prevent the drawing threads and the rendering proc
       from concurrently accessing the dirty lines array */
-   if (lock_nesting == 0)
-      pthread_mutex_lock(&osx_window_mutex);
-   lock_nesting++;
+  if (lock_nesting == 0) {
+      _unix_lock_mutex(osx_window_mutex);
    bmp->id |= BMP_ID_LOCKED;
+  }
+   lock_nesting++;
 }
 
  
@@ -231,10 +234,11 @@ static void osx_qz_release_win(BITMAP *bmp)
 {
    if (lock_nesting > 0) {
       lock_nesting--;
-      if (!lock_nesting)
-         pthread_mutex_unlock(&osx_window_mutex);
+      if (!lock_nesting) {
+	bmp->id &= ~BMP_ID_LOCKED;
+         _unix_unlock_mutex(osx_window_mutex);
+      }
    }
-   bmp->id &= ~BMP_ID_LOCKED;
 }
 
 
@@ -290,70 +294,72 @@ void osx_update_dirty_lines(void)
       return;
    
    /* Skip everything if there are no dirty lines */
-   pthread_mutex_lock(&osx_window_mutex);
+   _unix_lock_mutex(osx_window_mutex);
    for (rect.top = 0; (rect.top < gfx_quartz_window.h) && (!dirty_lines[rect.top]); rect.top++)
       ;
    if (rect.top >= gfx_quartz_window.h) {
-      pthread_mutex_unlock(&osx_window_mutex);
-      pthread_cond_broadcast(&vsync_cond);
+      _unix_unlock_mutex(osx_window_mutex);
+      osx_signal_vsync();
       return;
    }
    
    /* Dirty lines need to be updated */
-   while (![qd_view lockFocusIfCanDraw]);
-   while (!QDDone([qd_view qdPort]));
-   LockPortBits([qd_view qdPort]);
+   if ([qd_view lockFocusIfCanDraw] == YES) {
+      while (!QDDone([qd_view qdPort]));
+      LockPortBits([qd_view qdPort]);
    
-   qd_view_port = [qd_view qdPort];
+      qd_view_port = [qd_view qdPort];
    
-   qd_view_pitch = GetPixRowBytes(GetPortPixMap(qd_view_port));
-   qd_view_addr = GetPixBaseAddr(GetPortPixMap(qd_view_port)) +
-      ((int)([osx_window frame].size.height) - gfx_quartz_window.h) * qd_view_pitch;
+      qd_view_pitch = GetPixRowBytes(GetPortPixMap(qd_view_port));
+      qd_view_addr = GetPixBaseAddr(GetPortPixMap(qd_view_port)) +
+         ((int)([osx_window frame].size.height) - gfx_quartz_window.h) * qd_view_pitch;
    
-   SetEmptyRgn(update_region);
+      if (colorconv_blitter || (osx_setup_colorconv_blitter() == 0)) {
+         SetEmptyRgn(update_region);
    
-   rect.left = 0;
-   rect.right = gfx_quartz_window.w;
+         rect.left = 0;
+         rect.right = gfx_quartz_window.w;
 
-   while (rect.top < gfx_quartz_window.h) {
-      while ((!dirty_lines[rect.top]) && (rect.top < gfx_quartz_window.h))
-         rect.top++;
-      if (rect.top >= gfx_quartz_window.h)
-         break;
-      rect.bottom = rect.top;
-      while ((dirty_lines[rect.bottom]) && (rect.bottom < gfx_quartz_window.h)) {
-         dirty_lines[rect.bottom] = 0;
-         rect.bottom++;
-      }
-      /* fill in source graphics rectangle description */
-      src_gfx_rect.width  = rect.right - rect.left;
-      src_gfx_rect.height = rect.bottom - rect.top;
-      src_gfx_rect.pitch  = pseudo_screen_pitch;
-      src_gfx_rect.data   = pseudo_screen->line[0] +
-                            (rect.top * pseudo_screen_pitch) +
-                            (rect.left * BYTES_PER_PIXEL(pseudo_screen_depth));
+         while (rect.top < gfx_quartz_window.h) {
+            while ((!dirty_lines[rect.top]) && (rect.top < gfx_quartz_window.h))
+               rect.top++;
+            if (rect.top >= gfx_quartz_window.h)
+               break;
+            rect.bottom = rect.top;
+            while ((dirty_lines[rect.bottom]) && (rect.bottom < gfx_quartz_window.h)) {
+               dirty_lines[rect.bottom] = 0;
+               rect.bottom++;
+            }
+            /* fill in source graphics rectangle description */
+            src_gfx_rect.width  = rect.right - rect.left;
+            src_gfx_rect.height = rect.bottom - rect.top;
+            src_gfx_rect.pitch  = pseudo_screen_pitch;
+            src_gfx_rect.data   = pseudo_screen->line[0] +
+               (rect.top * pseudo_screen_pitch) +
+               (rect.left * BYTES_PER_PIXEL(pseudo_screen_depth));
 
-      /* fill in destination graphics rectangle description */
-      dest_gfx_rect.pitch = qd_view_pitch;
-      dest_gfx_rect.data  = qd_view_addr +
-                            (rect.top * qd_view_pitch) + 
-                            (rect.left * BYTES_PER_PIXEL(desktop_depth));
+            /* fill in destination graphics rectangle description */
+            dest_gfx_rect.pitch = qd_view_pitch;
+            dest_gfx_rect.data  = qd_view_addr +
+               (rect.top * qd_view_pitch) + 
+               (rect.left * BYTES_PER_PIXEL(desktop_depth));
       
-      /* function doing the hard work */
-      colorconv_blitter(&src_gfx_rect, &dest_gfx_rect);
+            /* function doing the hard work */
+            colorconv_blitter(&src_gfx_rect, &dest_gfx_rect);
       
-      RectRgn(temp_region, &rect);
-      UnionRgn(temp_region, update_region, update_region);
-      rect.top = rect.bottom;
+            RectRgn(temp_region, &rect);
+            UnionRgn(temp_region, update_region, update_region);
+            rect.top = rect.bottom;
+         }
+      }   
+      QDFlushPortBuffer(qd_view_port, update_region);
+   
+      UnlockPortBits([qd_view qdPort]);
+      [qd_view unlockFocus];
    }
+   _unix_unlock_mutex(osx_window_mutex);
    
-   QDFlushPortBuffer(qd_view_port, update_region);
-   
-   UnlockPortBits([qd_view qdPort]);
-   [qd_view unlockFocus];
-   pthread_mutex_unlock(&osx_window_mutex);
-   
-   pthread_cond_broadcast(&vsync_cond);
+   osx_signal_vsync();
 }
 
 
@@ -365,21 +371,29 @@ void osx_update_dirty_lines(void)
 int osx_setup_colorconv_blitter()
 {
    CFDictionaryRef mode;
-   
-   mode = CGDisplayCurrentMode(kCGDirectMainDisplay);
-   CFNumberGetValue(CFDictionaryGetValue(mode, kCGDisplayBitsPerPixel), kCFNumberSInt32Type, &desktop_depth);
-   desktop_depth = (desktop_depth == 16) ? 15 : desktop_depth;
-   
-   pthread_mutex_lock(&osx_window_mutex);
+   void *vp;
+   int dd;
+   if (qd_view && (vp = [qd_view qdPort]))
+   {
+      dd = GetPixDepth(GetPortPixMap(vp));
+   }   
+   else
+   {
+       mode = CGDisplayCurrentMode(kCGDirectMainDisplay);
+       CFNumberGetValue(CFDictionaryGetValue(mode, kCGDisplayBitsPerPixel), kCFNumberSInt32Type, &desktop_depth);
+       dd = desktop_depth;
+   }
+   if (dd == 16) dd = 15;
+   _unix_lock_mutex(osx_window_mutex);
    if (colorconv_blitter)
       _release_colorconv_blitter(colorconv_blitter);
-   colorconv_blitter = _get_colorconv_blitter(requested_color_depth, desktop_depth);
+   colorconv_blitter = _get_colorconv_blitter(requested_color_depth, dd);
    /* We also need to update the color conversion palette to reflect the change */
    if (colorconv_blitter)
       _set_colorconv_palette(_current_palette, 0, 255);
    /* Mark all the window as dirty */
    memset(dirty_lines, 1, gfx_quartz_window.h);
-   pthread_mutex_unlock(&osx_window_mutex);
+   _unix_unlock_mutex(osx_window_mutex);
    
    return (colorconv_blitter ? 0 : -1);
 }
@@ -398,7 +412,8 @@ static BITMAP *private_osx_qz_window_init(int w, int h, int v_w, int v_h, int co
    char tmp1[128], tmp2[128];
    
    pthread_cond_init(&vsync_cond, NULL);
-   pthread_mutex_init(&osx_window_mutex, NULL);
+   pthread_mutex_init(&vsync_mutex, NULL);
+   osx_window_mutex=_unix_create_mutex();
    lock_nesting = 0;
    
    if (1
@@ -468,11 +483,7 @@ static BITMAP *private_osx_qz_window_init(int w, int h, int v_w, int v_h, int co
    gfx_quartz_window.vid_mem = w * h * BYTES_PER_PIXEL(color_depth);
    
    requested_color_depth = color_depth;
-   if (osx_setup_colorconv_blitter()) {
-      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Unsupported color depth"));
-      return NULL;
-   }
-   
+   colorconv_blitter=NULL;   
    mode = CGDisplayCurrentMode(kCGDirectMainDisplay);
    CFNumberGetValue(CFDictionaryGetValue(mode, kCGDisplayBitsPerPixel), kCFNumberSInt32Type, &desktop_depth);
    CFNumberGetValue(CFDictionaryGetValue(mode, kCGDisplayRefreshRate), kCFNumberSInt32Type, &refresh_rate);
@@ -526,9 +537,9 @@ static BITMAP *private_osx_qz_window_init(int w, int h, int v_w, int v_h, int co
 static BITMAP *osx_qz_window_init(int w, int h, int v_w, int v_h, int color_depth)
 {
    BITMAP *bmp;
-   pthread_mutex_lock(&osx_event_mutex);
+   _unix_lock_mutex(osx_event_mutex);
    bmp = private_osx_qz_window_init(w, h, v_w, v_h, color_depth);
-   pthread_mutex_unlock(&osx_event_mutex);
+   _unix_unlock_mutex(osx_event_mutex);
    if (!bmp)
       osx_qz_window_exit(bmp);
    return bmp;
@@ -541,7 +552,7 @@ static BITMAP *osx_qz_window_init(int w, int h, int v_w, int v_h, int color_dept
  */
 static void osx_qz_window_exit(BITMAP *bmp)
 {
-   pthread_mutex_lock(&osx_event_mutex);
+   _unix_lock_mutex(osx_event_mutex);
 
    if (update_region) {
       DisposeRgn(update_region);
@@ -572,14 +583,15 @@ static void osx_qz_window_exit(BITMAP *bmp)
       colorconv_blitter = NULL;
    }
    
-   pthread_mutex_destroy(&osx_window_mutex);
+   _unix_destroy_mutex(osx_window_mutex);
    pthread_cond_destroy(&vsync_cond);
+   pthread_mutex_destroy(&vsync_mutex);
    
    osx_mouse_tracking_rect = -1;
    
    osx_gfx_mode = OSX_GFX_NONE;
    
-   pthread_mutex_unlock(&osx_event_mutex);
+   _unix_unlock_mutex(osx_event_mutex);
 }
 
 
@@ -589,13 +601,14 @@ static void osx_qz_window_exit(BITMAP *bmp)
  */
 static void osx_qz_window_vsync(void)
 {
-   if (lock_nesting > 0)
-      pthread_cond_wait(&vsync_cond, &osx_window_mutex);
-   else {
-      pthread_mutex_lock(&osx_window_mutex);
-      pthread_cond_wait(&vsync_cond, &osx_window_mutex);
-      pthread_mutex_unlock(&osx_window_mutex);
-   }
+  if (lock_nesting==0) {
+    pthread_mutex_trylock(&vsync_mutex);
+    pthread_cond_wait(&vsync_cond, &vsync_mutex);
+    pthread_mutex_unlock(&vsync_mutex);
+  }
+  else {
+    ASSERT(0); /* Screen already acquired, don't call vsync() */
+  }
 }
 
 
@@ -608,11 +621,24 @@ static void osx_qz_window_set_palette(AL_CONST struct RGB *p, int from, int to,
    if (vsync)
       osx_qz_window_vsync();
    
-   pthread_mutex_lock(&osx_window_mutex);
+   _unix_lock_mutex(osx_window_mutex);
    _set_colorconv_palette(p, from, to);
          
    /* invalidate the whole screen */
    memset(dirty_lines, 1, gfx_quartz_window.h);
    
-   pthread_mutex_unlock(&osx_window_mutex);
+   _unix_unlock_mutex(osx_window_mutex);
 }
+
+void osx_signal_vsync(void) 
+{
+   pthread_mutex_lock(&vsync_mutex);
+   pthread_cond_broadcast(&vsync_cond);
+   pthread_mutex_unlock(&vsync_mutex);
+}
+
+/* Local variables:       */
+/* c-basic-offset: 3      */
+/* indent-tabs-mode: nil  */
+/* c-file-style: "linux" */
+/* End:                   */
