@@ -24,6 +24,24 @@
    #error something is wrong with the makefile
 #endif
 
+#ifndef SCAN_DEPEND
+#include <CoreFoundation/CoreFoundation.h>
+#include <mach/mach_port.h>
+#include <servers/bootstrap.h>
+#endif
+
+
+/* These are used to warn the dock about the application */
+struct CPSProcessSerNum
+{
+   UInt32 lo;
+   UInt32 hi;
+};
+extern OSErr CPSGetCurrentProcess(struct CPSProcessSerNum *psn);
+extern OSErr CPSEnableForegroundOperation(struct CPSProcessSerNum *psn, UInt32 _arg2, UInt32 _arg3, UInt32 _arg4, UInt32 _arg5);
+extern OSErr CPSSetFrontProcess(struct CPSProcessSerNum *psn);
+
+
 
 static int osx_sys_init(void);
 static void osx_sys_exit(void);
@@ -34,7 +52,6 @@ static void osx_sys_set_window_title(AL_CONST char *);
 static int osx_sys_set_close_button_callback(void (*proc)(void));
 static int osx_sys_set_display_switch_mode(int mode);
 static void osx_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode);
-static void osx_sys_yield_timeslice(void);
 static int osx_sys_desktop_color_depth(void);
 static int osx_sys_get_desktop_resolution(int *width, int *height);
 
@@ -43,13 +60,15 @@ static int osx_sys_get_desktop_resolution(int *width, int *height);
 int    __crt0_argc;
 char **__crt0_argv;
 NSBundle *osx_bundle = NULL;
-pthread_mutex_t osx_event_mutex;
+void* osx_event_mutex;
 NSCursor *osx_cursor = NULL;
+NSCursor *osx_blank_cursor = NULL;
 AllegroWindow *osx_window = NULL;
 char osx_window_title[ALLEGRO_MESSAGE_SIZE];
 void (*osx_window_close_hook)(void) = NULL;
 int osx_gfx_mode = OSX_GFX_NONE;
 int osx_emulate_mouse_buttons = FALSE;
+int osx_window_first_expose = FALSE;
 
 
 static RETSIGTYPE (*old_sig_abrt)(int num);
@@ -63,8 +82,6 @@ static RETSIGTYPE (*old_sig_quit)(int num);
 static unsigned char *cursor_data = NULL;
 static NSBitmapImageRep *cursor_rep = NULL;
 static NSImage *cursor_image = NULL;
-static int old_x, old_y;
-static int buttons = 0;
 static int skip_events_processing = FALSE;
 
 
@@ -97,7 +114,7 @@ SYSTEM_DRIVER system_macosx =
    osx_sys_desktop_color_depth,
    osx_sys_get_desktop_resolution,
    osx_sys_get_gfx_safe_mode,
-   osx_sys_yield_timeslice,
+   _unix_yield_timeslice,
    _unix_create_mutex,
    _unix_destroy_mutex,
    _unix_lock_mutex,
@@ -118,13 +135,13 @@ SYSTEM_DRIVER system_macosx =
  */
 static RETSIGTYPE osx_signal_handler(int num)
 {
-   pthread_mutex_unlock(&osx_event_mutex);
-   pthread_mutex_unlock(&osx_window_mutex);
+   _unix_unlock_mutex(osx_event_mutex);
+   _unix_unlock_mutex(osx_window_mutex);
    
    allegro_exit();
    
-   pthread_mutex_destroy(&osx_event_mutex);
-   pthread_mutex_destroy(&osx_window_mutex);
+   _unix_destroy_mutex(osx_event_mutex);
+   _unix_destroy_mutex(osx_window_mutex);
    
    fprintf(stderr, "Shutting down Allegro due to signal #%d\n", num);
    raise(num);
@@ -143,32 +160,42 @@ void osx_event_handler()
    NSPoint point;
    NSRect frame, view;
    CGMouseDelta fdx, fdy;
-   int x, y, dx = 0, dy = 0, dz = 0;
+   int dx = 0, dy = 0, dz = 0;
+   int mx=_mouse_x;
+   int my=_mouse_y;
+   static int buttons = 0;
+   int old_buttons = buttons;
+   int event_type;
+   BOOL gotmouseevent = NO;
    
-   while (1) {
-      event = [NSApp nextEventMatchingMask: NSAnyEventMask
+   while ((event = [NSApp nextEventMatchingMask: NSAnyEventMask
          untilDate: distant_past
          inMode: NSDefaultRunLoopMode
-         dequeue: YES];
-      if (event == nil)
-         break;
-      
+         dequeue: YES]) != nil)
+     {
       if ((skip_events_processing) || (osx_gfx_mode == OSX_GFX_NONE)) {
          [NSApp sendEvent: event];
 	 continue;
       }
-
-      if (osx_window) {
-         point = [event locationInWindow];
-         frame = [[osx_window contentView] frame];
-         view = NSMakeRect(0, 0, gfx_driver->w, gfx_driver->h);
-      }
       
-      switch ([event type]) {
+      view = NSMakeRect(0, 0, gfx_driver->w, gfx_driver->h);
+      point = [event locationInWindow];
+      if (osx_window) 
+      {
+	 frame = [[osx_window contentView] frame];
+      }
+      else
+      {
+	 frame = [[NSScreen mainScreen] frame];
+      }
+      event_type = [event type];
+      switch (event_type) {
 	 
          case NSKeyDown:
 	    if (_keyboard_installed)
 	       osx_keyboard_handler(TRUE, event);
+	    if (([[event charactersIgnoringModifiers] characterAtIndex: 0] == 'q') && ([event modifierFlags] & NSCommandKeyMask)) 
+	       [NSApp sendEvent: event];
 	    break;
 	
          case NSKeyUp:
@@ -188,10 +215,9 @@ void osx_event_handler()
 	       /* App is regaining focus */
 	       if (_mouse_installed) {
 	          if ((osx_window) && (NSPointInRect(point, view))) {
-                     _mouse_x = old_x = point.x;
-	             _mouse_y = old_y = frame.size.height - point.y;
-		     _mouse_b = 0;
-	             _handle_mouse_input();
+                     mx = point.x;
+	             my = frame.size.height - point.y;
+		     buttons = 0;
                      _mouse_on = TRUE;
 	          }
 	       }
@@ -200,6 +226,7 @@ void osx_event_handler()
 	       if (_keyboard_installed)
 	          osx_keyboard_focused(TRUE, 0);
 	       _switch_in();
+	       gotmouseevent = YES;
 	       [NSApp sendEvent: event];
 	       break;
 	    }
@@ -208,7 +235,7 @@ void osx_event_handler()
          case NSOtherMouseUp:
          case NSRightMouseUp:
 	    if (osx_emulate_mouse_buttons) {
-	       if ([event type] == NSLeftMouseDown) {
+	       if (event_type == NSLeftMouseDown) {
                   if ((!osx_window) || (NSPointInRect(point, view))) {
 		     buttons = 0x1;
 		     if (key[KEY_ALT])
@@ -217,22 +244,21 @@ void osx_event_handler()
 		        buttons = 0x2;
 		  }
 	       }
-	       else if ([event type] == NSLeftMouseUp)
+	       else if (event_type == NSLeftMouseUp)
 	          buttons &= ~0x7;
 	    }
 	    else {
 	       if ((!osx_window) || (NSPointInRect(point, view))) {
 	          /* Deliver mouse downs only if cursor is on the window */
-	          buttons |= (([event type] == NSLeftMouseDown) ? 0x1 : 0);
-	          buttons |= (([event type] == NSRightMouseDown) ? 0x2 : 0);
-	          buttons |= (([event type] == NSOtherMouseDown) ? 0x4 : 0);
+	          buttons |= ((event_type == NSLeftMouseDown) ? 0x1 : 0);
+	          buttons |= ((event_type == NSRightMouseDown) ? 0x2 : 0);
+	          buttons |= ((event_type == NSOtherMouseDown) ? 0x4 : 0);
 	       }
-	       buttons &= ~(([event type] == NSLeftMouseUp) ? 0x1 : 0);
-	       buttons &= ~(([event type] == NSRightMouseUp) ? 0x2 : 0);
-	       buttons &= ~(([event type] == NSOtherMouseUp) ? 0x4 : 0);
+	       buttons &= ~((event_type == NSLeftMouseUp) ? 0x1 : 0);
+	       buttons &= ~((event_type == NSRightMouseUp) ? 0x2 : 0);
+	       buttons &= ~((event_type == NSOtherMouseUp) ? 0x4 : 0);
 	    }
-	    if (_mouse_installed)
-	       osx_mouse_handler(0, 0, 0, buttons);
+	    gotmouseevent = YES;
 	    [NSApp sendEvent: event];
 	    break;
 	    
@@ -240,53 +266,46 @@ void osx_event_handler()
          case NSRightMouseDragged:
          case NSOtherMouseDragged:
          case NSMouseMoved:
-	    if (osx_window) {
-               if (NSPointInRect(point, view)) {
-	          x = point.x;
-	          y = (frame.size.height - point.y);
-	          dx += x - old_x;
-	          dy += y - old_y;
-	          old_x = x;
-	          old_y = y;
-	       }
-	    }
-	    else
-	       CGGetLastMouseDelta(&dx, &dy);
+	    dx += [event deltaX];
+	    dy += [event deltaY];
+
+	    mx=point.x;
+	    my=frame.size.height-point.y;
+
 	    [NSApp sendEvent: event];
+	    gotmouseevent = YES;
 	    break;
             
          case NSScrollWheel:
 	    dz += [event deltaY];
+	    gotmouseevent = YES;
             break;
 	    
 	 case NSMouseEntered:
 	    if (([event trackingNumber] == osx_mouse_tracking_rect) && ([NSApp isActive])) {
 	       if (_mouse_installed) {
-                  _mouse_x = old_x = point.x;
-	          _mouse_y = old_y = frame.size.height - point.y;
-		  _mouse_b = 0;
-	          _handle_mouse_input();
+		  mx = point.x;
+	          my = frame.size.height - point.y;
+		  buttons = 0;
                   _mouse_on = TRUE;
+		  gotmouseevent = YES;
 	       }
 	    }
-	    else
-	       [NSApp sendEvent: event];
+	    [NSApp sendEvent: event];
 	    break;
             
 	 case NSMouseExited:
 	    if ([event trackingNumber] == osx_mouse_tracking_rect) {
 	       if (_mouse_installed) {
 	          _mouse_on = FALSE;
-                  _handle_mouse_input();
+		  gotmouseevent = YES;
 	       }
 	    }
-	    else
-	       [NSApp sendEvent: event];
+            [NSApp sendEvent: event];
 	    break;
             
 	 case NSAppKitDefined:
             switch ([event subtype]) {
-	       
                case NSApplicationActivatedEventType:
 	          if (osx_window) {
 		     [osx_window invalidateCursorRectsForView: [osx_window contentView]];
@@ -304,12 +323,13 @@ void osx_event_handler()
 	       
 	       case NSWindowMovedEventType:
                   /* This is needed to ensure the shadow gets drawn when the window is
-		   * created. It's weird, but sometimes it doesn't get drawn if we omit
-		   * the next two lines... The same applies to the cursor rectangle,
-		   * which doesn't seem to be set at window creation (it works once you
-		   * move the mouse though)
+		   * created. It's weird, but when the window is created on another
+		   * thread, sometimes its shadow doesn't get drawn. The same applies
+		   * to the cursor rectangle, which doesn't seem to be set at window
+		   * creation (it works once you move the mouse though).
 		   */
-	          if (osx_window) {
+	          if ((osx_window) && (osx_window_first_expose)) {
+		     osx_window_first_expose = FALSE;
                      [osx_window setHasShadow: NO];
                      [osx_window setHasShadow: YES];
 		     [osx_window invalidateCursorRectsForView: [osx_window contentView]];
@@ -324,27 +344,57 @@ void osx_event_handler()
 	    break;
       }
    }
-   if ((_mouse_installed) && (osx_gfx_mode != OSX_GFX_NONE) && (!skip_events_processing)) {
-      if (osx_mouse_warped) {
-         old_x = _mouse_x;
-         old_y = _mouse_y;
-         dx = dy = 0;
-         osx_mouse_warped = FALSE;
-      }
-      else {
-	 if (dx || dy || dz) {
-            if (osx_skip_mouse_move)
-	       osx_skip_mouse_move = FALSE;
-	    else
-               osx_mouse_handler(dx, dy, dz, buttons);
-	 }
-      }
-   }
+   if (gotmouseevent == YES)
+      osx_mouse_handler(mx, my, dx, dy, dz, buttons);
    [pool release];
 }
 
 
 
+/* osx_tell_dock:
+ *  Tell the dock about us; the origins of this hack are unknown, but it's
+ *  currently the only way to make a Cocoa app to work when started from a
+ *  console.
+ *  For the future, (10.3 and above) investigate TranformProcessType in the 
+ *  HIServices framework.
+ */
+static void osx_tell_dock(void)
+{
+   struct CPSProcessSerNum psn;
+
+   if ((!CPSGetCurrentProcess(&psn)) &&
+       (!CPSEnableForegroundOperation(&psn, 0x03, 0x3C, 0x2C, 0x1103)) &&
+       (!CPSSetFrontProcess(&psn)))
+      [NSApplication sharedApplication];
+}
+
+
+
+/* osx_bootstrap_ok:
+ *  Check if the current bootstrap context is privilege. If it's not, we can't
+ *  use NSApplication, and instead have to go directly to main.
+ *  Returns 1 if ok, 0 if not.
+ */
+int osx_bootstrap_ok(void)
+{
+   static int _ok = -1;
+   mach_port_t bp;
+   kern_return_t ret;
+   CFMachPortRef cfport;
+
+   /* If have tested once, just return that answer */
+   if (_ok >= 0)
+      return _ok;
+   cfport = CFMachPortCreate(NULL, NULL, NULL, NULL);
+   task_get_bootstrap_port(mach_task_self(), &bp);
+   ret = bootstrap_register(bp, "bootstrap-ok-test", CFMachPortGetPort(cfport));
+   CFRelease(cfport);
+   _ok = (ret == KERN_SUCCESS) ? 1 : 0;
+   return _ok;
+}
+
+
+
 /* osx_sys_init:
  *  Initalizes the MacOS X system driver.
  */
@@ -354,6 +404,11 @@ static int osx_sys_init(void)
    AL_CONST char *exe_name;
    char resource_dir[1024];
    
+   /* If we're in the 'dead bootstrap' environment, the Mac driver won't work. */
+   if (!osx_bootstrap_ok()) {
+      return -1;
+   }
+   
    /* Install emergency-exit signal handlers */
    old_sig_abrt = signal(SIGABRT, osx_signal_handler);
    old_sig_fpe  = signal(SIGFPE,  osx_signal_handler);
@@ -363,12 +418,11 @@ static int osx_sys_init(void)
    old_sig_int  = signal(SIGINT,  osx_signal_handler);
    old_sig_quit = signal(SIGQUIT, osx_signal_handler);
    
-   /* Get into bundle resource directory if appropriate */
-   if (osx_bundle) {
-      exe_name = [[osx_bundle executablePath] lossyCString];
-      snprintf(resource_dir, 1023, "%s/%s", [[osx_bundle resourcePath] lossyCString], get_filename(exe_name));
-      if (file_exists(resource_dir, FA_DIREC, NULL))
-         chdir(resource_dir);
+    
+
+   if (osx_bundle == NULL) {
+       /* If in a bundle, the dock will recognise us automatically */
+       osx_tell_dock();
    }
    
    /* Setup OS type & version */
@@ -392,8 +446,9 @@ static int osx_sys_init(void)
       bitsPerPixel: 32];
    cursor_image = [[NSImage alloc] initWithSize: NSMakeSize(16, 16)];
    [cursor_image addRepresentation: cursor_rep];
-   osx_cursor = [[NSCursor alloc] initWithImage: cursor_image
+   osx_blank_cursor = [[NSCursor alloc] initWithImage: cursor_image
       hotSpot: NSMakePoint(0, 0)];
+   osx_cursor = osx_blank_cursor;
    
    osx_gfx_mode = OSX_GFX_NONE;
    
@@ -418,8 +473,8 @@ static void osx_sys_exit(void)
    signal(SIGINT,  old_sig_int);
    signal(SIGQUIT, old_sig_quit);
    
-   if (osx_cursor)
-      [osx_cursor release];
+   if (osx_blank_cursor)
+      [osx_blank_cursor release];
    if (cursor_image)
       [cursor_image release];
    if (cursor_rep)
@@ -484,15 +539,15 @@ static void osx_sys_message(AL_CONST char *msg)
    ns_title = [NSString stringWithUTF8String: osx_window_title];
    ns_msg = [NSString stringWithUTF8String: tmp];
    
-   pthread_mutex_lock(&osx_event_mutex);
+   _unix_lock_mutex(osx_event_mutex);
    skip_events_processing = TRUE;
-   pthread_mutex_unlock(&osx_event_mutex);
+   _unix_unlock_mutex(osx_event_mutex);
    
    NSRunAlertPanel(ns_title, ns_msg, nil, nil, nil);
    
-   pthread_mutex_lock(&osx_event_mutex);
+   _unix_lock_mutex(osx_event_mutex);
    skip_events_processing = FALSE;
-   pthread_mutex_unlock(&osx_event_mutex);
+   _unix_unlock_mutex(osx_event_mutex);
 }
 
 
@@ -504,7 +559,7 @@ static void osx_sys_set_window_title(AL_CONST char *title)
 {
    char tmp[ALLEGRO_MESSAGE_SIZE];
    
-   strcpy(osx_window_title, title);
+   _al_sane_strncpy(osx_window_title, title, ALLEGRO_MESSAGE_SIZE);
    do_uconvert(title, U_CURRENT, tmp, U_UTF8, ALLEGRO_MESSAGE_SIZE);
 
    NSString *ns_title = [NSString stringWithUTF8String: tmp];
@@ -516,7 +571,8 @@ static void osx_sys_set_window_title(AL_CONST char *title)
 
 
 /* osx_sys_set_close_button_callback:
- *  Sets the window close callback.
+ *  Sets the window close callback. Also used when user hits Command-Q or
+ *  selects "Quit" from the application menu.
  */
 static int osx_sys_set_close_button_callback(void (*proc)(void))
 {
@@ -551,16 +607,6 @@ static void osx_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
 
 
 
-/* osx_sys_yield_timeslice:
- *  Used to play nice with multitasking.
- */
-static void osx_sys_yield_timeslice(void)
-{
-   usleep(30000);
-}
-
-
-
 /* osx_sys_desktop_color_depth:
  *  Queries the desktop color depth.
  */
