@@ -22,14 +22,8 @@
 
 
 
-#ifndef M_PI
-   #define M_PI   3.14159265358979323846
-#endif
-
-
-#define floatcos(x)     cos((x) * M_PI / 128.0)
-#define floatsin(x)     sin((x) * M_PI / 128.0)
-#define floattan(x)     tan((x) * M_PI / 128.0)
+#define FLOATSINCOS(x, s, c)  _AL_SINCOS((x) * AL_PI / 128.0, s ,c)
+#define floattan(x)           tan((x) * AL_PI / 128.0)
 
 
 
@@ -69,6 +63,7 @@ MATRIX_f identity_matrix_f =
  */
 void get_translation_matrix(MATRIX *m, fixed x, fixed y, fixed z)
 {
+   ASSERT(m);
    *m = identity_matrix;
 
    m->t[0] = x;
@@ -83,6 +78,7 @@ void get_translation_matrix(MATRIX *m, fixed x, fixed y, fixed z)
  */
 void get_translation_matrix_f(MATRIX_f *m, float x, float y, float z)
 {
+   ASSERT(m);
    *m = identity_matrix_f;
 
    m->t[0] = x;
@@ -98,6 +94,7 @@ void get_translation_matrix_f(MATRIX_f *m, float x, float y, float z)
  */
 void get_scaling_matrix(MATRIX *m, fixed x, fixed y, fixed z)
 {
+   ASSERT(m);
    *m = identity_matrix;
 
    m->v[0][0] = x;
@@ -112,6 +109,7 @@ void get_scaling_matrix(MATRIX *m, fixed x, fixed y, fixed z)
  */
 void get_scaling_matrix_f(MATRIX_f *m, float x, float y, float z)
 {
+   ASSERT(m);
    *m = identity_matrix_f;
 
    m->v[0][0] = x;
@@ -128,8 +126,9 @@ void get_scaling_matrix_f(MATRIX_f *m, float x, float y, float z)
  */
 void get_x_rotate_matrix(MATRIX *m, fixed r)
 {
-   fixed c = fcos(r);
-   fixed s = fsin(r);
+   fixed c = fixcos(r);
+   fixed s = fixsin(r);
+   ASSERT(m);
 
    *m = identity_matrix;
 
@@ -147,9 +146,10 @@ void get_x_rotate_matrix(MATRIX *m, fixed r)
  */
 void get_x_rotate_matrix_f(MATRIX_f *m, float r)
 {
-   float c = floatcos(r);
-   float s = floatsin(r);
+   float c, s;
+   ASSERT(m);
 
+   FLOATSINCOS(r, s, c);
    *m = identity_matrix_f;
 
    m->v[1][1] = c;
@@ -168,8 +168,9 @@ void get_x_rotate_matrix_f(MATRIX_f *m, float r)
  */
 void get_y_rotate_matrix(MATRIX *m, fixed r)
 {
-   fixed c = fcos(r);
-   fixed s = fsin(r);
+   fixed c = fixcos(r);
+   fixed s = fixsin(r);
+   ASSERT(m);
 
    *m = identity_matrix;
 
@@ -187,9 +188,10 @@ void get_y_rotate_matrix(MATRIX *m, fixed r)
  */
 void get_y_rotate_matrix_f(MATRIX_f *m, float r)
 {
-   float c = floatcos(r);
-   float s = floatsin(r);
+   float c, s;
+   ASSERT(m);
 
+   FLOATSINCOS(r, s, c);
    *m = identity_matrix_f;
 
    m->v[0][0] = c;
@@ -208,8 +210,9 @@ void get_y_rotate_matrix_f(MATRIX_f *m, float r)
  */
 void get_z_rotate_matrix(MATRIX *m, fixed r)
 {
-   fixed c = fcos(r);
-   fixed s = fsin(r);
+   fixed c = fixcos(r);
+   fixed s = fixsin(r);
+   ASSERT(m);
 
    *m = identity_matrix;
 
@@ -227,9 +230,10 @@ void get_z_rotate_matrix(MATRIX *m, fixed r)
  */
 void get_z_rotate_matrix_f(MATRIX_f *m, float r)
 {
-   float c = floatcos(r);
-   float s = floatsin(r);
+   float c, s;
+   ASSERT(m);
 
+   FLOATSINCOS(r, s, c);
    *m = identity_matrix_f;
 
    m->v[0][0] = c;
@@ -243,46 +247,46 @@ void get_z_rotate_matrix_f(MATRIX_f *m, float r)
 
 /* magical formulae for constructing rotation matrices */
 #define MAKE_ROTATION(x, y, z)                  \
-   fixed sin_x = fsin(x);                       \
-   fixed cos_x = fcos(x);                       \
+   fixed sin_x = fixsin(x);                     \
+   fixed cos_x = fixcos(x);                     \
 						\
-   fixed sin_y = fsin(y);                       \
-   fixed cos_y = fcos(y);                       \
+   fixed sin_y = fixsin(y);                     \
+   fixed cos_y = fixcos(y);                     \
 						\
-   fixed sin_z = fsin(z);                       \
-   fixed cos_z = fcos(z);                       \
+   fixed sin_z = fixsin(z);                     \
+   fixed cos_z = fixcos(z);                     \
 						\
-   fixed sinx_siny = fmul(sin_x, sin_y);        \
-   fixed cosx_siny = fmul(cos_x, sin_y);
+   fixed sinx_siny = fixmul(sin_x, sin_y);      \
+   fixed cosx_siny = fixmul(cos_x, sin_y);
 
 
 
 #define MAKE_ROTATION_f(x, y, z)                \
-   float sin_x = floatsin(x);                   \
-   float cos_x = floatcos(x);                   \
+   float sin_x, cos_x;				\
+   float sin_y, cos_y;				\
+   float sin_z, cos_z;				\
+   float sinx_siny, cosx_siny;			\
 						\
-   float sin_y = floatsin(y);                   \
-   float cos_y = floatcos(y);                   \
+   FLOATSINCOS(x, sin_x, cos_x);		\
+   FLOATSINCOS(y, sin_y, cos_y);		\
+   FLOATSINCOS(z, sin_z, cos_z);		\
 						\
-   float sin_z = floatsin(z);                   \
-   float cos_z = floatcos(z);                   \
-						\
-   float sinx_siny = sin_x * sin_y;             \
-   float cosx_siny = cos_x * sin_y;
+   sinx_siny = sin_x * sin_y;			\
+   cosx_siny = cos_x * sin_y;
 
 
 
-#define R00 (fmul(cos_y, cos_z))
-#define R10 (fmul(sinx_siny, cos_z) - fmul(cos_x, sin_z))
-#define R20 (fmul(cosx_siny, cos_z) + fmul(sin_x, sin_z))
+#define R00 (fixmul(cos_y, cos_z))
+#define R10 (fixmul(sinx_siny, cos_z) - fixmul(cos_x, sin_z))
+#define R20 (fixmul(cosx_siny, cos_z) + fixmul(sin_x, sin_z))
 
-#define R01 (fmul(cos_y, sin_z))
-#define R11 (fmul(sinx_siny, sin_z) + fmul(cos_x, cos_z))
-#define R21 (fmul(cosx_siny, sin_z) - fmul(sin_x, cos_z))
+#define R01 (fixmul(cos_y, sin_z))
+#define R11 (fixmul(sinx_siny, sin_z) + fixmul(cos_x, cos_z))
+#define R21 (fixmul(cosx_siny, sin_z) - fixmul(sin_x, cos_z))
 
 #define R02 (-sin_y)
-#define R12 (fmul(sin_x, cos_y))
-#define R22 (fmul(cos_x, cos_y))
+#define R12 (fixmul(sin_x, cos_y))
+#define R22 (fixmul(cos_x, cos_y))
 
 
 
@@ -308,6 +312,7 @@ void get_z_rotate_matrix_f(MATRIX_f *m, float r)
 void get_rotation_matrix(MATRIX *m, fixed x, fixed y, fixed z)
 {
    MAKE_ROTATION(x, y, z);
+   ASSERT(m);
 
    m->v[0][0] = R00;
    m->v[0][1] = R01;
@@ -332,6 +337,7 @@ void get_rotation_matrix(MATRIX *m, fixed x, fixed y, fixed z)
 void get_rotation_matrix_f(MATRIX_f *m, float x, float y, float z)
 {
    MAKE_ROTATION_f(x, y, z);
+   ASSERT(m);
 
    m->v[0][0] = R00_f;
    m->v[0][1] = R01_f;
@@ -356,12 +362,17 @@ void get_rotation_matrix_f(MATRIX_f *m, float x, float y, float z)
 void get_align_matrix(MATRIX *m, fixed xfront, fixed yfront, fixed zfront, fixed xup, fixed yup, fixed zup)
 {
    fixed xright, yright, zright;
+   ASSERT(m);
+
+   xfront = -xfront;
+   yfront = -yfront;
+   zfront = -zfront;
 
    normalize_vector(&xfront, &yfront, &zfront);
-   normalize_vector(&xup, &yup, &zup);
-
-   cross_product(xfront, yfront, zfront, xup, yup, zup, &xright, &yright, &zright);
-   cross_product(xright, yright, zright, xfront, yfront, zfront, &xup, &yup, &zup);
+   cross_product(xup, yup, zup, xfront, yfront, zfront, &xright, &yright, &zright);
+   normalize_vector(&xright, &yright, &zright);
+   cross_product(xfront, yfront, zfront, xright, yright, zright, &xup, &yup, &zup);
+   /* No need to normalize up here, since right and front are perpendicular and normalized. */
 
    m->v[0][0] = xright; 
    m->v[0][1] = xup; 
@@ -386,12 +397,17 @@ void get_align_matrix(MATRIX *m, fixed xfront, fixed yfront, fixed zfront, fixed
 void get_align_matrix_f(MATRIX_f *m, float xfront, float yfront, float zfront, float xup, float yup, float zup)
 {
    float xright, yright, zright;
+   ASSERT(m);
+
+   xfront = -xfront;
+   yfront = -yfront;
+   zfront = -zfront;
 
    normalize_vector_f(&xfront, &yfront, &zfront);
-   normalize_vector_f(&xup, &yup, &zup);
-
-   cross_product_f(xfront, yfront, zfront, xup, yup, zup, &xright, &yright, &zright);
-   cross_product_f(xright, yright, zright, xfront, yfront, zfront, &xup, &yup, &zup);
+   cross_product_f(xup, yup, zup, xfront, yfront, zfront, &xright, &yright, &zright);
+   normalize_vector_f(&xright, &yright, &zright);
+   cross_product_f(xfront, yfront, zfront, xright, yright, zright, &xup, &yup, &zup);
+   /* No need to normalize up here, since right and front are perpendicular and normalized. */
 
    m->v[0][0] = xright; 
    m->v[0][1] = xup; 
@@ -419,6 +435,7 @@ void get_vector_rotation_matrix(MATRIX *m, fixed x, fixed y, fixed z, fixed a)
 {
    MATRIX_f rotation;
    int i, j;
+   ASSERT(m);
 
    get_vector_rotation_matrix_f(&rotation, fixtof(x), fixtof(y), fixtof(z), fixtof(a));
 
@@ -436,10 +453,11 @@ void get_vector_rotation_matrix(MATRIX *m, fixed x, fixed y, fixed z, fixed a)
  */
 void get_vector_rotation_matrix_f(MATRIX_f *m, float x, float y, float z, float a)
 {
-   float c = floatcos(a);
-   float s = floatsin(a);
-   float cc = 1 - c;
+   float c, s, cc;
+   ASSERT(m);
 
+   FLOATSINCOS(a, s, c);
+   cc = 1 - c;
    normalize_vector_f(&x, &y, &z);
 
    m->v[0][0] = (cc * x * x) + c;
@@ -469,18 +487,19 @@ void get_vector_rotation_matrix_f(MATRIX_f *m, float x, float y, float z, float
 void get_transformation_matrix(MATRIX *m, fixed scale, fixed xrot, fixed yrot, fixed zrot, fixed x, fixed y, fixed z)
 {
    MAKE_ROTATION(xrot, yrot, zrot);
+   ASSERT(m);
 
-   m->v[0][0] = fmul(R00, scale);
-   m->v[0][1] = fmul(R01, scale);
-   m->v[0][2] = fmul(R02, scale);
+   m->v[0][0] = fixmul(R00, scale);
+   m->v[0][1] = fixmul(R01, scale);
+   m->v[0][2] = fixmul(R02, scale);
 
-   m->v[1][0] = fmul(R10, scale);
-   m->v[1][1] = fmul(R11, scale);
-   m->v[1][2] = fmul(R12, scale);
+   m->v[1][0] = fixmul(R10, scale);
+   m->v[1][1] = fixmul(R11, scale);
+   m->v[1][2] = fixmul(R12, scale);
 
-   m->v[2][0] = fmul(R20, scale);
-   m->v[2][1] = fmul(R21, scale);
-   m->v[2][2] = fmul(R22, scale);
+   m->v[2][0] = fixmul(R20, scale);
+   m->v[2][1] = fixmul(R21, scale);
+   m->v[2][2] = fixmul(R22, scale);
 
    m->t[0] = x;
    m->t[1] = y;
@@ -495,6 +514,7 @@ void get_transformation_matrix(MATRIX *m, fixed scale, fixed xrot, fixed yrot, f
 void get_transformation_matrix_f(MATRIX_f *m, float scale, float xrot, float yrot, float zrot, float x, float y, float z)
 {
    MAKE_ROTATION_f(xrot, yrot, zrot);
+   ASSERT(m);
 
    m->v[0][0] = R00_f * scale;
    m->v[0][1] = R01_f * scale;
@@ -536,6 +556,7 @@ void get_camera_matrix(MATRIX *m, fixed x, fixed y, fixed z, fixed xfront, fixed
 {
    MATRIX_f camera;
    int i, j;
+   ASSERT(m);
 
    get_camera_matrix_f(&camera,
 		       fixtof(x), fixtof(y), fixtof(z), 
@@ -560,6 +581,7 @@ void get_camera_matrix_f(MATRIX_f *m, float x, float y, float z, float xfront, f
 {
    MATRIX_f camera, scale;
    float xside, yside, zside, width, d;
+   ASSERT(m);
 
    /* make 'in-front' into a unit vector, and negate it */
    normalize_vector_f(&xfront, &yfront, &zfront);
@@ -597,7 +619,7 @@ void get_camera_matrix_f(MATRIX_f *m, float x, float y, float z, float xfront, f
 
    /* construct a scaling matrix to deal with aspect ratio and FOV */
    width = floattan(64.0 - fov/2);
-   get_scaling_matrix_f(&scale, width, -aspect*width*4/3, -1.0);
+   get_scaling_matrix_f(&scale, width, -aspect*width, -1.0);
 
    /* combine the camera and scaling matrices */
    matrix_mul_f(&camera, &scale, m);
@@ -610,6 +632,7 @@ void get_camera_matrix_f(MATRIX_f *m, float x, float y, float z, float xfront, f
  */
 void qtranslate_matrix(MATRIX *m, fixed x, fixed y, fixed z)
 {
+   ASSERT(m);
    m->t[0] += x;
    m->t[1] += y;
    m->t[2] += z;
@@ -622,6 +645,7 @@ void qtranslate_matrix(MATRIX *m, fixed x, fixed y, fixed z)
  */
 void qtranslate_matrix_f(MATRIX_f *m, float x, float y, float z)
 {
+   ASSERT(m);
    m->t[0] += x;
    m->t[1] += y;
    m->t[2] += z;
@@ -635,10 +659,11 @@ void qtranslate_matrix_f(MATRIX_f *m, float x, float y, float z)
 void qscale_matrix(MATRIX *m, fixed scale)
 {
    int i, j;
+   ASSERT(m);
 
    for (i=0; i<3; i++)
       for (j=0; j<3; j++)
-	 m->v[i][j] = fmul(m->v[i][j], scale);
+	 m->v[i][j] = fixmul(m->v[i][j], scale);
 }
 
 
@@ -649,6 +674,7 @@ void qscale_matrix(MATRIX *m, fixed scale)
 void qscale_matrix_f(MATRIX_f *m, float scale)
 {
    int i, j;
+   ASSERT(m);
 
    for (i=0; i<3; i++)
       for (j=0; j<3; j++)
@@ -664,10 +690,13 @@ void qscale_matrix_f(MATRIX_f *m, float scale)
  *  applied to a vector v, (v * out) = ((v * m1) * m2). Any number of
  *  transformations can be concatenated in this way.
  */
-void matrix_mul(MATRIX *m1, MATRIX *m2, MATRIX *out)
+void matrix_mul(AL_CONST MATRIX *m1, AL_CONST MATRIX *m2, MATRIX *out)
 {
    MATRIX temp;
    int i, j;
+   ASSERT(m1);
+   ASSERT(m2);
+   ASSERT(out);
 
    if (m1 == out) {
       temp = *m1;
@@ -680,14 +709,14 @@ void matrix_mul(MATRIX *m1, MATRIX *m2, MATRIX *out)
 
    for (i=0; i<3; i++) {
       for (j=0; j<3; j++) {
-	 out->v[i][j] = fmul(m1->v[0][j], m2->v[i][0]) +
-			fmul(m1->v[1][j], m2->v[i][1]) +
-			fmul(m1->v[2][j], m2->v[i][2]);
+	 out->v[i][j] = fixmul(m1->v[0][j], m2->v[i][0]) +
+			fixmul(m1->v[1][j], m2->v[i][1]) +
+			fixmul(m1->v[2][j], m2->v[i][2]);
       }
 
-      out->t[i] = fmul(m1->t[0], m2->v[i][0]) +
-		  fmul(m1->t[1], m2->v[i][1]) +
-		  fmul(m1->t[2], m2->v[i][2]) +
+      out->t[i] = fixmul(m1->t[0], m2->v[i][0]) +
+		  fixmul(m1->t[1], m2->v[i][1]) +
+		  fixmul(m1->t[2], m2->v[i][2]) +
 		  m2->t[i];
    } 
 }
@@ -697,10 +726,13 @@ void matrix_mul(MATRIX *m1, MATRIX *m2, MATRIX *out)
 /* matrix_mul_f:
  *  Floating point version of matrix_mul().
  */
-void matrix_mul_f(MATRIX_f *m1, MATRIX_f *m2, MATRIX_f *out)
+void matrix_mul_f(AL_CONST MATRIX_f *m1, AL_CONST MATRIX_f *m2, MATRIX_f *out)
 {
    MATRIX_f temp;
    int i, j;
+   ASSERT(m1);
+   ASSERT(m2);
+   ASSERT(out);
 
    if (m1 == out) {
       temp = *m1;
@@ -736,7 +768,7 @@ fixed vector_length(fixed x, fixed y, fixed z)
    y >>= 8;
    z >>= 8;
 
-   return (fsqrt(fmul(x,x) + fmul(y,y) + fmul(z,z)) << 8);
+   return (fixsqrt(fixmul(x,x) + fixmul(y,y) + fixmul(z,z)) << 8);
 }
 
 
@@ -759,9 +791,9 @@ void normalize_vector(fixed *x, fixed *y, fixed *z)
 {
    fixed length = vector_length(*x, *y, *z);
 
-   *x = fdiv(*x, length);
-   *y = fdiv(*y, length);
-   *z = fdiv(*z, length);
+   *x = fixdiv(*x, length);
+   *y = fixdiv(*y, length);
+   *z = fixdiv(*z, length);
 }
 
 
@@ -785,9 +817,13 @@ void normalize_vector_f(float *x, float *y, float *z)
  */
 void cross_product(fixed x1, fixed y1, fixed z1, fixed x2, fixed y2, fixed z2, fixed *xout, fixed *yout, fixed *zout)
 {
-    *xout = fmul(y1, z2) - fmul(z1, y2);
-    *yout = fmul(z1, x2) - fmul(x1, z2);
-    *zout = fmul(x1, y2) - fmul(y1, x2);
+   ASSERT(xout);
+   ASSERT(yout);
+   ASSERT(zout);
+
+   *xout = fixmul(y1, z2) - fixmul(z1, y2);
+   *yout = fixmul(z1, x2) - fixmul(x1, z2);
+   *zout = fixmul(x1, y2) - fixmul(y1, x2);
 }
 
 
@@ -797,9 +833,13 @@ void cross_product(fixed x1, fixed y1, fixed z1, fixed x2, fixed y2, fixed z2, f
  */
 void cross_product_f(float x1, float y1, float z1, float x2, float y2, float z2, float *xout, float *yout, float *zout)
 {
-    *xout = (y1 * z2) - (z1 * y2);
-    *yout = (z1 * x2) - (x1 * z2);
-    *zout = (x1 * y2) - (y1 * x2);
+   ASSERT(xout);
+   ASSERT(yout);
+   ASSERT(zout);
+
+   *xout = (y1 * z2) - (z1 * y2);
+   *yout = (z1 * x2) - (x1 * z2);
+   *zout = (x1 * y2) - (y1 * x2);
 }
 
 
@@ -808,9 +848,12 @@ void cross_product_f(float x1, float y1, float z1, float x2, float y2, float z2,
  *  Helper function for backface culling: returns the z component of the
  *  normal vector to the polygon formed from the three vertices.
  */
-fixed polygon_z_normal(V3D *v1, V3D *v2, V3D *v3)
+fixed polygon_z_normal(AL_CONST V3D *v1, AL_CONST V3D *v2, AL_CONST V3D *v3)
 {
-   return (fmul(v2->x-v1->x, v3->y-v2->y) - fmul(v3->x-v2->x, v2->y-v1->y));
+   ASSERT(v1);
+   ASSERT(v2);
+   ASSERT(v3);
+   return (fixmul(v2->x-v1->x, v3->y-v2->y) - fixmul(v3->x-v2->x, v2->y-v1->y));
 }
 
 
@@ -818,8 +861,11 @@ fixed polygon_z_normal(V3D *v1, V3D *v2, V3D *v3)
 /* polygon_z_normal_f:
  *  Floating point version of polygon_z_normal().
  */
-float polygon_z_normal_f(V3D_f *v1, V3D_f *v2, V3D_f *v3)
+float polygon_z_normal_f(AL_CONST V3D_f *v1, AL_CONST V3D_f *v2, AL_CONST V3D_f *v3)
 {
+   ASSERT(v1);
+   ASSERT(v2);
+   ASSERT(v3);
    return ((v2->x-v1->x) * (v3->y-v2->y)) - ((v3->x-v2->x) * (v2->y-v1->y));
 }
 
@@ -844,6 +890,9 @@ float _persp_yoffset_f = 100.0;
  */
 void set_projection_viewport(int x, int y, int w, int h)
 {
+   ASSERT(w > 0);
+   ASSERT(h > 0);
+   
    _persp_xscale = itofix(w/2);
    _persp_yscale = itofix(h/2);
    _persp_xoffset = itofix(x + w/2);
