@@ -14,6 +14,8 @@
  *
  *      Pause and seek functions by George Foot.
  *
+ *      get_midi_length by Elias Pschernig.
+ *
  *      See readme.txt for copyright information.
  */
 
@@ -29,6 +31,9 @@
 /* maximum number of layers in a single voice */
 #define MIDI_LAYERS  4
 
+/* how often the midi callback gets called maximally / second */
+#define MIDI_TIMER_FREQUENCY 40
+
 
 typedef struct MIDI_TRACK                       /* a track in the MIDI file */
 {
@@ -76,7 +81,9 @@ typedef struct PATCH_TABLE                      /* GM -> external synth */
 } PATCH_TABLE;
 
 
-volatile long midi_pos = -1;                    /* position in MIDI file */
+volatile long midi_pos = -1;                    /* current position in MIDI file */
+volatile long midi_time = 0;                    /* current position in seconds */
+static volatile long midi_timers;               /* current position in allegro-timer-ticks */
 static long midi_pos_counter;                   /* delta for midi_pos */
 
 volatile long _midi_tick = 0;                   /* counter for killing notes */
@@ -129,6 +136,7 @@ void (*midi_sysex_callback)(AL_CONST unsigned char *data, int length) = NULL;
 void lock_midi(MIDI *midi)
 {
    int c;
+   ASSERT(midi);
 
    LOCK_DATA(midi, sizeof(MIDI));
 
@@ -170,7 +178,26 @@ MIDI *load_midi(AL_CONST char *filename)
       midi->track[c].len = 0;
    }
 
-   pack_fread(buf, 4, fp);                   /* read midi header */
+   pack_fread(buf, 4, fp); /* read midi header */
+
+   /* Is the midi inside a .rmi file? */
+   if (memcmp(buf, "RIFF", 4) == 0) { /* check for RIFF header */
+      pack_mgetl(fp);
+
+      while (!pack_feof(fp)) {
+         pack_fread(buf, 4, fp); /* RMID chunk? */
+         if (memcmp(buf, "RMID", 4) == 0) break;
+
+         pack_fseek(fp, pack_igetl(fp)); /* skip to next chunk */
+      }
+
+      if (pack_feof(fp)) goto err;
+
+      pack_mgetl(fp);
+      pack_mgetl(fp);
+      pack_fread(buf, 4, fp); /* read midi header */
+   }
+
    if (memcmp(buf, "MThd", 4))
       goto err;
 
@@ -874,14 +901,17 @@ static void midi_player(void)
       return;
 
    if (midi_semaphore) {
-      midi_timer_speed += BPS_TO_TIMER(40);
-      install_int_ex(midi_player, BPS_TO_TIMER(40));
+      midi_timer_speed += BPS_TO_TIMER(MIDI_TIMER_FREQUENCY);
+      install_int_ex(midi_player, BPS_TO_TIMER(MIDI_TIMER_FREQUENCY));
       return;
    }
 
    midi_semaphore = TRUE;
    _midi_tick++;
 
+   midi_timers += midi_timer_speed;
+   midi_time = midi_timers / TIMERS_PER_SECOND;
+
    do_it_all_again:
 
    for (c=0; c<MIDI_VOICES; c++)
@@ -975,8 +1005,8 @@ static void midi_player(void)
    }
 
    /* reprogram the timer */
-   if (midi_timer_speed < BPS_TO_TIMER(40))
-      midi_timer_speed = BPS_TO_TIMER(40);
+   if (midi_timer_speed < BPS_TO_TIMER(MIDI_TIMER_FREQUENCY))
+      midi_timer_speed = BPS_TO_TIMER(MIDI_TIMER_FREQUENCY);
 
    if (!midi_seeking) 
       install_int_ex(midi_player, midi_timer_speed);
@@ -1073,6 +1103,7 @@ static int load_patches(MIDI *midi)
    unsigned char running_status, event;
    long l;
    int c;
+   ASSERT(midi);
 
    for (c=0; c<128; c++)                        /* initialise to unused */
       patches[c] = drums[c] = FALSE;
@@ -1173,6 +1204,7 @@ static int load_patches(MIDI *midi)
 static void prepare_to_play(MIDI *midi)
 {
    int c;
+   ASSERT(midi);
 
    for (c=0; c<16; c++)
       reset_controllers(c);
@@ -1181,6 +1213,8 @@ static void prepare_to_play(MIDI *midi)
 
    midifile = midi;
    midi_pos = 0;
+   midi_timers = 0;
+   midi_time = 0;
    midi_pos_counter = 0;
    midi_speed = TIMERS_PER_SECOND / 2 / midifile->divisions;   /* 120 bpm */
    midi_new_speed = -1;
@@ -1284,7 +1318,7 @@ int play_looped_midi(MIDI *midi, int loop_start, int loop_end)
 /* stop_midi:
  *  Stops whatever MIDI file is currently playing.
  */
-void stop_midi()
+void stop_midi(void)
 {
    play_midi(NULL, FALSE);
 }
@@ -1296,7 +1330,7 @@ END_OF_FUNCTION(stop_midi);
 /* midi_pause:
  *  Pauses the currently playing MIDI file.
  */
-void midi_pause()
+void midi_pause(void)
 {
    int c;
 
@@ -1318,7 +1352,7 @@ END_OF_FUNCTION(midi_pause);
 /* midi_resume:
  *  Resumes playing a paused MIDI file.
  */
-void midi_resume() 
+void midi_resume(void)
 {
    if (!midifile)
       return;
@@ -1443,6 +1477,21 @@ END_OF_FUNCTION(midi_seek);
 
 
 
+/* get_midi_length:
+ *  Returns the length, in seconds, of the specified midi. This will stop any
+ *  currently playing midi. Don't call it too often, since it simulates playing
+ *  all of the midi to get the time even if the midi contains tempo changes.
+ */
+int get_midi_length(MIDI *midi)
+{
+    play_midi(midi, 0);
+    while (midi_pos < 0); /* Without this, midi_seek won't work. */
+    midi_seek(INT_MAX);
+    return midi_time;
+}
+
+
+
 /* midi_out:
  *  Inserts MIDI command bytes into the output stream, in realtime.
  */
@@ -1451,6 +1500,7 @@ void midi_out(unsigned char *data, int length)
    unsigned char *pos = data;
    unsigned char running_status = 0;
    long timer = 0;
+   ASSERT(data);
 
    midi_semaphore = TRUE;
    _midi_tick++;
@@ -1468,7 +1518,7 @@ void midi_out(unsigned char *data, int length)
 /* load_midi_patches:
  *  Tells the MIDI driver to preload the entire sample set.
  */
-int load_midi_patches()
+int load_midi_patches(void)
 {
    char patches[128], drums[128];
    int c, ret;
@@ -1494,6 +1544,8 @@ int load_midi_patches()
 static void midi_lock_mem(void)
 {
    LOCK_VARIABLE(midi_pos);
+   LOCK_VARIABLE(midi_time);
+   LOCK_VARIABLE(midi_timers);
    LOCK_VARIABLE(midi_pos_counter);
    LOCK_VARIABLE(_midi_tick);
    LOCK_VARIABLE(midifile);
