@@ -33,8 +33,8 @@ extern unsigned char *_colorconv_rgb_map;  /* for conversion from 8/12-bit to 8-
 
 void _colorconv_blit_8_to_8(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -50,30 +50,18 @@ void _colorconv_blit_8_to_8(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT
    for (y = src_rect->height; y; y--) {
       for (x = width >> 2; x; x--) {
          src_data = *(unsigned int *)src;
-#ifdef ALLEGRO_LITTLE_ENDIAN
 	 dest_data = _colorconv_rgb_map[src_data & 0xff];
 	 dest_data |= (_colorconv_rgb_map[(src_data >> 8) & 0xff] << 8);
 	 dest_data |= (_colorconv_rgb_map[(src_data >> 16) & 0xff] << 16);
 	 dest_data |= (_colorconv_rgb_map[src_data >> 24] << 24);
-#else
-	 dest_data = _colorconv_rgb_map[(src_data >> 24) & 0xff];
-	 dest_data |= (_colorconv_rgb_map[(src_data >> 16) & 0xff] << 8);
-	 dest_data |= (_colorconv_rgb_map[(src_data >> 8) & 0xff] << 16);
-	 dest_data |= (_colorconv_rgb_map[src_data & 0xff] << 24);
-#endif
 	 *(unsigned int *)dest = dest_data;
 	 src += 4;
 	 dest += 4;
       }
       if (width & 0x2) {
          src_data = *(unsigned short *)src;
-#ifdef ALLEGRO_LITTLE_ENDIAN
 	 dest_data = _colorconv_rgb_map[src_data & 0xff];
 	 dest_data |= (_colorconv_rgb_map[src_data >> 8] << 8);
-#else
-	 dest_data = _colorconv_rgb_map[src_data & 0xff] << 8;
-	 dest_data |= _colorconv_rgb_map[src_data >> 8];
-#endif
 	 *(unsigned short *)dest = (unsigned short)dest_data;
 	 src += 2;
 	 dest += 2;
@@ -101,8 +89,8 @@ void _colorconv_blit_8_to_15(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_REC
 
 void _colorconv_blit_8_to_16(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -166,8 +154,8 @@ void _colorconv_blit_8_to_16(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_REC
 
 void _colorconv_blit_8_to_24(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -232,8 +220,8 @@ void _colorconv_blit_8_to_24(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_REC
 
 void _colorconv_blit_8_to_32(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -306,8 +294,8 @@ void _colorconv_blit_8_to_32(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_REC
 
 void _colorconv_blit_15_to_8(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -357,8 +345,8 @@ void _colorconv_blit_15_to_8(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_REC
 
 void _colorconv_blit_15_to_16(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -395,8 +383,8 @@ void _colorconv_blit_15_to_16(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RE
 
 void _colorconv_blit_15_to_24(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -464,8 +452,8 @@ void _colorconv_blit_15_to_24(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RE
 
 void _colorconv_blit_15_to_32(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -509,8 +497,8 @@ void _colorconv_blit_15_to_32(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RE
 
 void _colorconv_blit_16_to_8(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -562,8 +550,8 @@ void _colorconv_blit_16_to_8(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_REC
 
 void _colorconv_blit_16_to_15(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -618,8 +606,8 @@ void _colorconv_blit_16_to_32(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RE
 
 static void colorconv_blit_true_to_8(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect, int bpp)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -646,8 +634,8 @@ static void colorconv_blit_true_to_8(struct GRAPHICS_RECT *src_rect, struct GRAP
 
 static void colorconv_blit_true_to_15(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect, int bpp)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -705,8 +693,8 @@ static void colorconv_blit_true_to_15(struct GRAPHICS_RECT *src_rect, struct GRA
 
 static void colorconv_blit_true_to_16(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect, int bpp)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
@@ -789,13 +777,12 @@ void _colorconv_blit_24_to_16(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RE
 
 void _colorconv_blit_24_to_32(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
    int y, x;
-   unsigned int src_data;
    unsigned int temp;
    
    width = src_rect->width;
@@ -848,13 +835,12 @@ void _colorconv_blit_32_to_16(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RE
 
 void _colorconv_blit_32_to_24(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
    int y, x;
-   unsigned int src_data;
    unsigned int temp;
    
    width = src_rect->width;
@@ -890,14 +876,12 @@ void _colorconv_blit_32_to_24(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RE
 
 static void colorcopy(struct GRAPHICS_RECT *src_rect, struct GRAPHICS_RECT *dest_rect, int bpp)
 {
-   void *src;
-   void *dest;
+   unsigned char *src;
+   unsigned char *dest;
    int width;
    int src_feed;
    int dest_feed;
    int y, x;
-   unsigned int src_data;
-   unsigned int temp;
    
    width = src_rect->width * bpp;
    src_feed = src_rect->pitch - width;
