@@ -24,7 +24,7 @@
 
 #include "allegro.h"
 
-#ifdef GFX_MODEX
+#ifdef GFX_HAS_VGA
 
 #include "allegro/internal/aintern.h"
 #include "allegro/internal/aintvga.h"
@@ -33,6 +33,8 @@
    #include ALLEGRO_INTERNAL_HEADER
 #endif
 
+#include "modexsms.h"
+
 #if (!defined ALLEGRO_LINUX) || ((defined ALLEGRO_LINUX_VGA) && ((!defined ALLEGRO_WITH_MODULES) || (defined ALLEGRO_MODULE)))
 
 
@@ -41,10 +43,12 @@ void _x_draw_sprite_end(void);
 void _x_blit_from_memory_end(void);
 void _x_blit_to_memory_end(void);
 
+static void really_split_modex_screen(int line);
+
 
 
 /* table of functions for drawing onto the mode-X screen */
-GFX_VTABLE __modex_vtable =
+static GFX_VTABLE __modex_vtable =
 {
    8,
    MASK_COLOR_8,
@@ -60,8 +64,9 @@ GFX_VTABLE __modex_vtable =
    _x_hline,
    _x_hline,
    _normal_line,
+   _fast_line,
    _normal_rectfill,
-   NULL,
+   _soft_triangle,
    _x_draw_sprite,
    _x_draw_sprite,
    _x_draw_sprite_v_flip,
@@ -87,8 +92,26 @@ GFX_VTABLE __modex_vtable =
    _x_masked_blit,
    _x_clear_to_color,
    _pivot_scaled_sprite_flip,
+   NULL,    /* AL_METHOD(void, do_stretch_blit, (struct BITMAP *source, struct BITMAP *dest, int source_x, int source_y, int source_width, int source_height, int dest_x, int dest_y, int dest_width, int dest_height, int masked)); */
+   _soft_draw_gouraud_sprite,
    _x_draw_sprite_end,
-   _x_blit_from_memory_end
+   _x_blit_from_memory_end,
+   _soft_polygon,
+   _soft_rect,
+   _soft_circle,
+   _soft_circlefill,
+   _soft_ellipse,
+   _soft_ellipsefill,
+   _soft_arc,
+   _soft_spline,
+   _soft_floodfill,
+
+   _soft_polygon3d,
+   _soft_polygon3d_f,
+   _soft_triangle3d,
+   _soft_triangle3d_f,
+   _soft_quad3d,
+   _soft_quad3d_f
 };
 
 
@@ -122,6 +145,7 @@ GFX_DRIVER gfx_modex =
    NULL,                         /* no drawing mode hook */
    _save_vga_mode,
    _restore_vga_mode,
+   NULL,                         /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    modex_fetch_mode_list,
    0, 0,
    TRUE,
@@ -133,7 +157,7 @@ GFX_DRIVER gfx_modex =
 
 
 
-GFX_MODE modex_gfx_modes[] = {
+static GFX_MODE modex_gfx_modes[] = {
    { 256, 200, 8 },
    { 256, 224, 8 },
    { 256, 240, 8 },
@@ -185,6 +209,7 @@ GFX_DRIVER gfx_xtended =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
+   NULL,                         /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    xtended_fetch_mode_list,
    640, 400,
    TRUE,
@@ -618,6 +643,9 @@ static void modex_exit(BITMAP *b)
 
    #endif
    _unset_vga_mode();
+
+   /* see modexsms.c */
+   _split_modex_screen_ptr = NULL;
 }
 
 
@@ -633,6 +661,17 @@ static BITMAP *modex_init(int w, int h, int v_w, int v_h, int color_depth)
    unsigned long addr;
    int c;
 
+   /* Do not continue if this version of Allegro was built in C-only mode.
+    * The bank switchers assume asm-mode calling conventions, but the
+    * library would try to call them with C calling conventions.
+    */
+#ifdef ALLEGRO_USE_C
+   return NULL;
+#endif
+
+   /* see modexsms.c */
+   _split_modex_screen_ptr = really_split_modex_screen;
+
    /* check it is a valid resolution */
    if (color_depth != 8) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Mode-X only supports 8 bit color"));
@@ -778,6 +817,17 @@ static BITMAP *xtended_init(int w, int h, int v_w, int v_h, int color_depth)
    unsigned long addr;
    BITMAP *b;
 
+   /* Do not continue if this version of Allegro was built in C-only mode.
+    * The bank switchers assume asm-mode calling conventions, but the
+    * library would try to call them with C calling conventions.
+    */
+#ifdef ALLEGRO_USE_C
+   return NULL;
+#endif
+
+   /* see modexsms.c */
+   _split_modex_screen_ptr = really_split_modex_screen;
+
    /* check it is a valid resolution */
    if (color_depth != 8) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Xtended mode only supports 8 bit color"));
@@ -1596,16 +1646,12 @@ static GFX_MODE_LIST *modex_fetch_mode_list(void)
 
 
 
-/* split_modex_screen:
+/* really_split_modex_screen:
  *  Enables a horizontal split screen at the specified line.
  *  Based on code from Paul Fenwick's XLIBDJ, which was in turn based 
  *  on Michael Abrash's routines in PC Techniques, June 1991.
  */
-#ifdef ALLEGRO_MODULE
-static void module_split_modex_screen(int line)
-#else
-void split_modex_screen(int line)
-#endif
+static void really_split_modex_screen(int line)
 {
    if (gfx_driver != &gfx_modex)
       return;
@@ -1637,18 +1683,6 @@ void split_modex_screen(int line)
 #endif      /* (!defined ALLEGRO_LINUX) || ((defined ALLEGRO_LINUX_VGA) && ... */
 
 
-#if (defined ALLEGRO_LINUX_VGA) && (defined ALLEGRO_WITH_MODULES)
-
-void (*_split_modex_screen_ptr)(int);
-
-void (split_modex_screen)(int line)
-{
-   _split_modex_screen_ptr(line);
-}
-
-#endif
-
-
 #if (defined ALLEGRO_LINUX_VGA) && (defined ALLEGRO_MODULE)
 
 /* _module_init:
@@ -1658,12 +1692,9 @@ void _module_init_modex(int system_driver)
 {
    if (system_driver == SYSTEM_LINUX)
       _unix_register_gfx_driver(GFX_MODEX, &gfx_modex, TRUE, FALSE);
-
-   _split_modex_screen_ptr = module_split_modex_screen;
 }
 
-#endif
+#endif	    /* (defined ALLEGRO_LINUX_VGA) && (defined ALLEGRO_MODULE) */
 
 
-
-#endif      /* ifdef GFX_MODEX */
+#endif	    /* GFX_HAS_VGA */
