@@ -107,7 +107,7 @@ static void vbeaf_vline_a(BITMAP *bmp, int x, int y1, int y2, int color);
 static void vbeaf_vline_b(BITMAP *bmp, int x, int y1, int y2, int color);
 static void vbeaf_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int color);
 static void vbeaf_rectfill(BITMAP *bmp, int x1, int y1, int x2, int y2, int color);
-static int  vbeaf_triangle(BITMAP *bmp, int x1, int y1, int x2, int y2, int x3, int y3, int color);
+static void vbeaf_triangle(BITMAP *bmp, int x1, int y1, int x2, int y2, int x3, int y3, int color);
 static void vbeaf_draw_glyph(BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color, int bg);
 static void vbeaf_draw_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y);
 static void vbeaf_blit_from_memory(BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height);
@@ -158,6 +158,7 @@ GFX_DRIVER gfx_vbeaf =
    NULL,                         /* no drawing mode hook */
    vbeaf_save,
    vbeaf_restore,
+   NULL,                         /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    vbeaf_fetch_mode_list,        /* fetch mode hook */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -1178,6 +1179,14 @@ static BITMAP *vbeaf_init(int w, int h, int v_w, int v_h, int color_depth)
    char tmp1[512];
    void *vaddr;
 
+   /* Do not continue if this version of Allegro was built in C-only mode.
+    * The bank switchers assume asm-mode calling conventions, but the
+    * library would try to call them with C calling conventions.
+    */
+#ifdef ALLEGRO_USE_C
+   return NULL;
+#endif
+
    /* locate and load VBE/AF driver from disk */
    if (!vbeaf_locate_driver())
       return NULL;
@@ -1583,7 +1592,7 @@ static void vbeaf_exit(BITMAP *b)
 /* vbeaf_save:
  *  Saves the graphics state.
  */
-static void vbeaf_save()
+static void vbeaf_save(void)
 {
    if (af_driver->EnableDirectAccess)
       af_driver->EnableDirectAccess(af_driver);
@@ -1600,7 +1609,7 @@ static void vbeaf_save()
 /* vbeaf_restore:
  *  Restores the graphics state.
  */
-static void vbeaf_restore()
+static void vbeaf_restore(void)
 {
    long wret = saved_wret;
 
@@ -1629,7 +1638,7 @@ static void vbeaf_restore()
  *  Attempts to create a list of valid video modes for the VBE/AF driver.
  *  Returns the mode list on success or NULL on failure.
  */
-static GFX_MODE_LIST *vbeaf_fetch_mode_list()
+static GFX_MODE_LIST *vbeaf_fetch_mode_list(void)
 {
    GFX_MODE_LIST *gfx_mode_list;
    AF_MODE_INFO *mode_info;
@@ -1720,7 +1729,7 @@ static GFX_MODE_LIST *vbeaf_fetch_mode_list()
  *  can emulate it by altering the display start address with the vsync 
  *  flag set.
  */
-static void vbeaf_vsync()
+static void vbeaf_vsync(void)
 {
    vbeaf_scroll(vbeaf_xscroll, vbeaf_yscroll);
 }
@@ -1800,7 +1809,7 @@ static int vbeaf_request_scroll(int x, int y)
 /* vbeaf_poll_scroll:
  *  Triple buffering test routine.
  */
-static int vbeaf_poll_scroll()
+static int vbeaf_poll_scroll(void)
 {
    int ret;
 
@@ -1948,7 +1957,7 @@ static int vbeaf_show_mouse(BITMAP *bmp, int x, int y)
 /* vbeaf_hide_mouse:
  *  Gets rid of the hardware cursor.
  */
-static void vbeaf_hide_mouse()
+static void vbeaf_hide_mouse(void)
 {
    SAFISH_CALL(
       af_driver->ShowCursor(af_driver, 0);
@@ -2002,7 +2011,7 @@ END_OF_STATIC_FUNCTION(vbeaf_move_mouse);
  *  Hook to tell us that the drawing mode has been changed, so we may need
  *  to alter some of our vtable entries.
  */
-static void vbeaf_drawing_mode()
+static void vbeaf_drawing_mode(void)
 {
    vbeaf_pattern = NULL;
 
@@ -2534,23 +2543,31 @@ static void vbeaf_rectfill(BITMAP *bmp, int x1, int y1, int x2, int y2, int colo
 /* vbeaf_triangle:
  *  Hardware accelerated triangle drawing function.
  */
-static int vbeaf_triangle(BITMAP *bmp, int x1, int y1, int x2, int y2, int x3, int y3, int color)
+static void vbeaf_triangle(BITMAP *bmp, int x1, int y1, int x2, int y2, int x3, int y3, int color)
 {
    AF_TRAP trap;
 
    /* bounding box test */
    if (bmp->clip) {
-      if ((x1 < bmp->cl) || (x2 < bmp->cl) || (x3 < bmp->cl))
-	 return ((x1 < bmp->cl) && (x2 < bmp->cl) && (x3 < bmp->cl));
+      if ((x1 < bmp->cl) || (x2 < bmp->cl) || (x3 < bmp->cl)) {
+         _soft_triangle(bmp, x1, y1, x2, y2, x3, y3, color);
+	 return;
+      }
+      
+      if ((y1 < bmp->ct) || (y2 < bmp->ct) || (y3 < bmp->ct)) {
+         _soft_triangle(bmp, x1, y1, x2, y2, x3, y3, color);
+	 return;
+      }
 
-      if ((y1 < bmp->ct) || (y2 < bmp->ct) || (y3 < bmp->ct))
-	 return ((y1 < bmp->ct) && (y2 < bmp->ct) && (y3 < bmp->ct));
+      if ((x1 >= bmp->cr) || (x2 >= bmp->cr) || (x3 >= bmp->cr)) {
+         _soft_triangle(bmp, x1, y1, x2, y2, x3, y3, color);
+	 return;
+      }
 
-      if ((x1 >= bmp->cr) || (x2 >= bmp->cr) || (x3 >= bmp->cr))
-	 return ((x1 >= bmp->cr) && (x2 >= bmp->cr) && (x3 >= bmp->cr));
-
-      if ((y1 >= bmp->cb) || (y2 >= bmp->cb) || (y3 >= bmp->cb))
-	 return ((y1 >= bmp->cb) && (y2 >= bmp->cb) && (y3 >= bmp->cb));
+      if ((y1 >= bmp->cb) || (y2 >= bmp->cb) || (y3 >= bmp->cb)) {
+         _soft_triangle(bmp, x1, y1, x2, y2, x3, y3, color);
+	 return;
+      }
    }
 
    /* sort along the vertical axis */
@@ -2642,8 +2659,6 @@ static int vbeaf_triangle(BITMAP *bmp, int x1, int y1, int x2, int y2, int x3, i
 	 af_driver->DrawTrap(af_driver, color, &trap);
       }
    );
-
-   return TRUE;
 }
 
 
