@@ -401,7 +401,7 @@ static int qnx_sys_init(void)
       pthread_setschedparam(pthread_self(), spolicy, &sparam);
    }
    
-   _read_os_type();
+   _unix_read_os_type();
 
    set_display_switch_mode(SWITCH_BACKGROUND);
 
@@ -460,7 +460,7 @@ static void qnx_sys_set_window_title(AL_CONST char *name)
 /* qnx_sys_set_close_button_callback:
  *  Sets the close button callback function.
  */
-static int qnx_sys_set_close_button_callback(void (*proc)(void));
+static int qnx_sys_set_close_button_callback(void (*proc)(void))
 {
    window_close_hook = proc;
 
@@ -563,7 +563,7 @@ static void qnx_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
  */
 static void qnx_sys_yield_timeslice(void)
 {
-   usleep(10000);
+   usleep(0);
 }
 
 
