@@ -102,7 +102,7 @@ int save_bitmap(AL_CONST char *filename, BITMAP *bmp, AL_CONST RGB *pal)
 
    for (iter = bitmap_type_list; iter; iter = iter->next) {
       if (stricmp(iter->ext, aext) == 0) {
-	 if (iter->load)
+	 if (iter->save)
 	    return iter->save(filename, bmp, pal);
 	 return 1;
       }
@@ -202,7 +202,8 @@ void _register_bitmap_file_type_init(void)
 {
    char buf[32];
 
-   _add_exit_func(register_bitmap_file_type_exit);
+   _add_exit_func(register_bitmap_file_type_exit,
+		  "register_bitmap_file_type_exit");
 
    register_bitmap_file_type(uconvert_ascii("bmp", buf), load_bmp, save_bmp);
    register_bitmap_file_type(uconvert_ascii("lbm", buf), load_lbm, NULL);
