@@ -82,7 +82,7 @@ SCANLINE_DRAWER_GENERIC(generic
 				       fixed spr_dx, fixed spr_dy)    \
    {                                                                  \
       int c;                                                          \
-      unsigned long addr, end_addr;                                   \
+      uintptr_t addr, end_addr;                                   \
       unsigned char **spr_line = spr->line;                           \
                                                                       \
       r_bmp_x >>= 16;                                                 \
@@ -137,24 +137,24 @@ SCANLINE_DRAWER_GENERIC(generic
 
 #ifdef ALLEGRO_COLOR32
    SCANLINE_DRAWER(32,
-		   c = ((unsigned long *)spr_line[l_spr_y>>16])
+		   c = ((uint32_t *)spr_line[l_spr_y>>16])
 		       [l_spr_x>>16])
 #endif
 
-#ifdef GFX_MODEX
+#ifdef GFX_HAS_VGA
    static void draw_scanline_modex(
     BITMAP *bmp, BITMAP *spr, fixed l_bmp_x, int bmp_y_i, fixed r_bmp_x,
     fixed l_spr_x, fixed l_spr_y, fixed spr_dx, fixed spr_dy)
    {
       int c;
-      unsigned long start_addr, addr, end_addr;
+      uintptr_t start_addr, addr, end_addr;
       unsigned char **spr_line = spr->line;
       int plane;
       fixed spr_x, spr_y;
 
       r_bmp_x >>= 16;
       l_bmp_x >>= 16;
-      start_addr = (unsigned long)bmp->line[bmp_y_i];
+      start_addr = (uintptr_t)bmp->line[bmp_y_i];
       spr_dx <<= 2;
       spr_dy <<= 2;
       for (plane = 0; plane < 4; plane++) {
@@ -262,13 +262,18 @@ void _parallelogram_map(BITMAP *bmp, BITMAP *spr, fixed xs[4], fixed ys[4],
       top_index = 3;
 
    /* Get direction of points: clockwise or anti-clockwise. */
-   if (fixmul(xs[(top_index+1) & 3] - xs[top_index],
-	    ys[(top_index-1) & 3] - ys[top_index]) >
-       fixmul(xs[(top_index-1) & 3] - xs[top_index],
-	    ys[(top_index+1) & 3] - ys[top_index]))
+   right_index = (double)(xs[(top_index+1) & 3] - xs[top_index]) *
+      (double)(ys[(top_index-1) & 3] - ys[top_index]) >
+      (double)(xs[(top_index-1) & 3] - xs[top_index]) *
+      (double)(ys[(top_index+1) & 3] - ys[top_index]) ? 1 : -1;
+   //FIXME: why does fixmul overflow below?
+   /*if (fixmul(xs[(top_index+1) & 3] - xs[top_index],
+      ys[(top_index-1) & 3] - ys[top_index]) >
+         fixmul(xs[(top_index-1) & 3] - xs[top_index],
+            ys[(top_index+1) & 3] - ys[top_index]))
       right_index = 1;
    else
-      right_index = -1;
+      right_index = -1;*/
 
    /*
     * Get coordinates of the corners.
@@ -317,8 +322,12 @@ void _parallelogram_map(BITMAP *bmp, BITMAP *spr, fixed xs[4], fixed ys[4],
       clip_right = (bmp->cr << 16) - 1;
    }
    else {
+      ASSERT(left_bmp_x >= 0 && top_bmp_x >= 0 && bottom_bmp_x >= 0
+	     && right_bmp_x < (bmp->w << 16)
+	     && top_bmp_x < (bmp->w << 16)
+	     && bottom_bmp_x < (bmp->w << 16));
       clip_left = 0;
-      clip_right = bmp->w << 16;
+      clip_right = (bmp->w << 16) - 1;
    }
 
    /* Quit if we're totally outside. */
@@ -336,18 +345,26 @@ void _parallelogram_map(BITMAP *bmp, BITMAP *spr, fixed xs[4], fixed ys[4],
       clip_bottom_i = (bottom_bmp_y + 0xffff) >> 16;
    else
       clip_bottom_i = (bottom_bmp_y + 0x8000) >> 16;
-   if (bmp->clip)
+   if (bmp->clip) {
       if (clip_bottom_i > bmp->cb)
 	 clip_bottom_i = bmp->cb;
+   }
+   else {
+      ASSERT(clip_bottom_i <= bmp->h);
+   }
 
    /* Calculate y coordinate of first scanline. */
    if (sub_pixel_accuracy)
       bmp_y_i = top_bmp_y >> 16;
    else
       bmp_y_i = (top_bmp_y + 0x8000) >> 16;
-   if (bmp->clip)
+   if (bmp->clip) {
       if (bmp_y_i < bmp->ct)
 	 bmp_y_i = bmp->ct;
+   }
+   else {
+      ASSERT(bmp_y_i >= 0);
+   }
 
    /* Sprite is above or below bottom clipping area. */
    if (bmp_y_i >= clip_bottom_i)
@@ -683,7 +700,7 @@ void _parallelogram_map_standard(BITMAP *bmp, BITMAP *sprite,
 	    ASSERT(0);
       }
    }
-   #ifdef GFX_MODEX
+   #ifdef GFX_HAS_VGA
       else {
 	 _parallelogram_map(bmp, sprite, xs, ys,
 			    draw_scanline_modex, FALSE);
@@ -718,15 +735,7 @@ void _rotate_scale_flip_coordinates(fixed w, fixed h,
    if (angle >= 0x800000)
       angle -= 0x1000000;
 
-   #if (defined ALLEGRO_DJGPP) && (DJGPP >= 2) && (DJGPP_MINOR >= 3)
-      /* Faster way to obtain both sin and cos with one call under djgpp.
-	 Todo: Do other platforms support this? (not watcom, mingw or msvc)
-      */
-      sincos(&cos_angle, &sin_angle, angle * (AL_PI / (double)0x800000));
-   #else
-      cos_angle = cos(angle * (AL_PI / (double)0x800000));
-      sin_angle = sin(angle * (AL_PI / (double)0x800000));
-   #endif
+   _AL_SINCOS(angle * (AL_PI / (double)0x800000), sin_angle, cos_angle);
 
    if (cos_angle >= 0)
       fix_cos = (int)(cos_angle * 0x10000 + 0.5);
