@@ -17,7 +17,7 @@
 
 #include "allegro.h"
 
-#if (defined MIDI_ALSA) && ((!defined ALLEGRO_WITH_MODULES) || (defined ALLEGRO_MODULE))
+#if (defined ALLEGRO_WITH_ALSAMIDI) && ((!defined ALLEGRO_WITH_MODULES) || (defined ALLEGRO_MODULE))
 
 #include "allegro/internal/aintern.h"
 
@@ -157,7 +157,7 @@ static int alsa_rawmidi_init(int input, int voices)
 #if ALLEGRO_ALSA_VERSION == 9
       device = get_config_string(uconvert_ascii("sound", tmp1),
 				 uconvert_ascii("alsa_rawmidi_device", tmp2),
-				 0);
+				 "default");
 
       err = snd_rawmidi_open(NULL, &rawmidi_handle, device, 0);
 #else  /* ALLEGRO_ALSA_VERSION == 5 */
@@ -185,11 +185,11 @@ static int alsa_rawmidi_init(int input, int voices)
       snd_rawmidi_nonblock(rawmidi_handle, 0);
       snd_rawmidi_info_malloc(&info);
       snd_rawmidi_info(rawmidi_handle, info);
-      strcpy(alsa_rawmidi_desc, snd_rawmidi_info_get_name(info));
+      _al_sane_strncpy(alsa_rawmidi_desc, snd_rawmidi_info_get_name(info), sizeof(alsa_rawmidi_desc));
 #else  /* ALLEGRO_ALSA_VERSION == 5 */
       snd_rawmidi_block_mode(rawmidi_handle, 1);
       snd_rawmidi_info(rawmidi_handle, &info);
-      strcpy(alsa_rawmidi_desc, info.name);
+      _al_sane_strncpy(alsa_rawmidi_desc, info.name, sizeof(alsa_rawmidi_desc));
 #endif
       midi_alsa.desc = alsa_rawmidi_desc;
    }
