@@ -37,7 +37,8 @@ static void bg_man_pthreads_disable_interrupts(void);
 static bg_func funcs[MAX_FUNCS];
 static int max_func; /* highest+1 used entry */
 
-static pthread_t thread;
+static pthread_t thread = 0;
+static int thread_alive = FALSE;
 static pthread_mutex_t cli_mutex;
 static pthread_cond_t cli_cond;
 static int cli_count;
@@ -58,6 +59,8 @@ static void block_all_signals(void)
 
 
 
+#ifndef ALLEGRO_MACOSX
+
 /* bg_man_pthreads_threadfunc:
  *  Thread function for the background thread.
  */
@@ -65,23 +68,26 @@ static void *bg_man_pthreads_threadfunc(void *arg)
 {
    struct timeval old_time, new_time;
    struct timeval delay;
-   unsigned long interval, i;
+   unsigned long interval;
    int n;
 
    block_all_signals();
 
+   interval = 0;
    gettimeofday(&old_time, 0);
 
-   while (1) {
+   while (thread_alive) {
       gettimeofday(&new_time, 0);
-      interval = ((new_time.tv_sec - old_time.tv_sec) * 1000000L +
-		  (new_time.tv_usec - old_time.tv_usec));
+      /* add the new time difference to the remainder of the old difference */
+      interval += ((new_time.tv_sec - old_time.tv_sec) * 1000000L +
+		   (new_time.tv_usec - old_time.tv_usec));
       old_time = new_time;
 
-      while (interval) {
-	 i = MIN (interval, INT_MAX/(TIMERS_PER_SECOND/100));
-	 interval -= i;
-	 i = i * (TIMERS_PER_SECOND/100) / 10000L;
+      /* run the callbacks for each 10ms elapsed, but limit to 18ms */
+      if (interval > 18000)
+         interval = 18000;      
+      while (interval > 10000) {
+	 interval -= 10000;	  
 
 	 pthread_mutex_lock(&cli_mutex);
 
@@ -98,10 +104,10 @@ static void *bg_man_pthreads_threadfunc(void *arg)
 	 pthread_mutex_unlock(&cli_mutex);
       }
 
+      /* rest a little bit before checking again */
       delay.tv_sec = 0;
-      delay.tv_usec = 10000;
+      delay.tv_usec = 1000;
       select(0, NULL, NULL, NULL, &delay);
-      pthread_testcancel();
    }
 
    return NULL;
@@ -116,6 +122,9 @@ static int bg_man_pthreads_init(void)
 {
    int i;
 
+   ASSERT(thread == 0);
+   ASSERT(!thread_alive);
+
    for (i = 0; i < MAX_FUNCS; i++)
       funcs[i] = NULL;
 
@@ -125,9 +134,12 @@ static int bg_man_pthreads_init(void)
    pthread_mutex_init(&cli_mutex, NULL);
    pthread_cond_init(&cli_cond, NULL);
 
+   thread_alive = TRUE;
    if (pthread_create(&thread, NULL, bg_man_pthreads_threadfunc, NULL)) {
+      thread_alive = FALSE;
       pthread_mutex_destroy(&cli_mutex);
       pthread_cond_destroy(&cli_cond);
+      thread = 0;
       return -1;
    }
 
@@ -141,10 +153,15 @@ static int bg_man_pthreads_init(void)
  */
 static void bg_man_pthreads_exit(void)
 {
-   pthread_cancel(thread);
-   pthread_join(thread, NULL);
-   pthread_mutex_destroy(&cli_mutex);
-   pthread_cond_destroy(&cli_cond);
+   ASSERT(!!thread == !!thread_alive);
+
+   if (thread) {
+      thread_alive = FALSE;
+      pthread_join(thread, NULL);
+      pthread_mutex_destroy(&cli_mutex);
+      pthread_cond_destroy(&cli_cond);
+      thread = 0;
+   }
 }
 
 
@@ -283,6 +300,8 @@ struct bg_manager _bg_man_pthreads = {
    bg_man_pthreads_interrupts_disabled
 };
 
+#endif
+
 
 
 /* custom mutex that supports nested locking */
