@@ -22,9 +22,13 @@
 
 #include "wddraw.h"
 
+#define PREFIX_I                "al-wddmode INFO: "
+#define PREFIX_W                "al-wddmode WARNING: "
+#define PREFIX_E                "al-wddmode ERROR: "
 
-int desktop_depth;
-RGB_MAP desktop_rgb_map;  /* for 8-bit desktops */
+
+int _win_desktop_depth;
+RGB_MAP _win_desktop_rgb_map;  /* for 8-bit desktops */
 
 
 int mode_supported;
@@ -66,11 +70,12 @@ static int _wnd_width, _wnd_height, _wnd_depth, _wnd_refresh_rate, _wnd_flags;
 static int wnd_set_video_mode(void)
 {
    HRESULT hr;
+   HWND allegro_wnd = win_get_window();
 
    /* set the cooperative level to allow fullscreen access */
    hr = IDirectDraw2_SetCooperativeLevel(directdraw, allegro_wnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
    if (FAILED(hr)) {
-      _TRACE("SetCooperativeLevel() failed (%x)\n", hr);
+      _TRACE(PREFIX_E "SetCooperativeLevel() failed (%x)\n", hr);
       return -1;
    }
 
@@ -78,7 +83,7 @@ static int wnd_set_video_mode(void)
    hr = IDirectDraw2_SetDisplayMode(directdraw, _wnd_width, _wnd_height, _wnd_depth,
                                                 _wnd_refresh_rate, _wnd_flags);
    if (FAILED(hr)) {
-      _TRACE("SetDisplayMode(%u, %u, %u, %u, %u) failed (%x)\n", _wnd_width, _wnd_height, _wnd_depth,
+      _TRACE(PREFIX_E "SetDisplayMode(%u, %u, %u, %u, %u) failed (%x)\n", _wnd_width, _wnd_height, _wnd_depth,
                                                                  _wnd_refresh_rate, _wnd_flags, hr);
       return -1;
    }
@@ -123,10 +128,10 @@ void build_desktop_rgb_map(void)
       pal[i].b = shift_gamma(system_palette[i].peBlue);
    }
 
-   create_rgb_table(&desktop_rgb_map, pal, NULL);
+   create_rgb_table(&_win_desktop_rgb_map, pal, NULL);
 
    /* create_rgb_table() never maps RGB triplets to index 0 */
-   desktop_rgb_map.data[pal[0].r>>1][pal[0].g>>1][pal[0].b>>1] = 0;
+   _win_desktop_rgb_map.data[pal[0].r>>1][pal[0].g>>1][pal[0].b>>1] = 0;
 }
 
 
@@ -179,18 +184,18 @@ int gfx_directx_compare_color_depth(int color_depth)
    surf_desc.dwSize = sizeof(surf_desc);
    hr = IDirectDraw2_GetDisplayMode(directdraw, &surf_desc);
    if (FAILED(hr)) {
-      _TRACE("Can't get color format.\n");
+      _TRACE(PREFIX_E "Can't get color format.\n");
       return -1;
    }
 
    /* get the *real* color depth of the desktop */
-   desktop_depth = surf_desc.ddpfPixelFormat.dwRGBBitCount;
-   if (desktop_depth == 16) /* sure? */
-      desktop_depth = get_color_bits(surf_desc.ddpfPixelFormat.dwRBitMask) +
+   _win_desktop_depth = surf_desc.ddpfPixelFormat.dwRGBBitCount;
+   if (_win_desktop_depth == 16) /* sure? */
+      _win_desktop_depth = get_color_bits(surf_desc.ddpfPixelFormat.dwRBitMask) +
                       get_color_bits(surf_desc.ddpfPixelFormat.dwGBitMask) +
                       get_color_bits(surf_desc.ddpfPixelFormat.dwBBitMask);
 
-   if (color_depth == desktop_depth) {
+   if (color_depth == _win_desktop_depth) {
       ddpixel_format = NULL;
       return 0;
    }
@@ -200,7 +205,7 @@ int gfx_directx_compare_color_depth(int color_depth)
          if ((pixel_realdepth[i] == color_depth) &&
             ((surf_desc.ddpfPixelFormat.dwRBitMask & pixel_format[i].dwRBitMask) ||
                 (surf_desc.ddpfPixelFormat.dwBBitMask & pixel_format[i].dwBBitMask) ||
-                   (desktop_depth == 8) || (color_depth == 8))) {
+                   (_win_desktop_depth == 8) || (color_depth == 8))) {
                       ddpixel_format = &pixel_format[i];
                       break;
          }
@@ -225,7 +230,7 @@ int gfx_directx_update_color_format(DDRAW_SURFACE *surf, int color_depth)
    pixel_format.dwSize = sizeof(DDPIXELFORMAT);
    hr = IDirectDrawSurface2_GetPixelFormat(surf->id, &pixel_format);
    if (FAILED(hr)) {
-      _TRACE("Can't get color format.\n");
+      _TRACE(PREFIX_E "Can't get color format.\n");
       return -1;
    }
 
@@ -363,6 +368,7 @@ GFX_MODE_LIST *gfx_directx_fetch_mode_list(void)
  */
 int set_video_mode(int w, int h, int v_w, int v_h, int color_depth)
 {
+   HWND allegro_wnd = win_get_window();
    _wnd_width = w;
    _wnd_height = h;
    _wnd_depth = (color_depth == 15 ? 16 : color_depth);
@@ -376,7 +382,7 @@ int set_video_mode(int w, int h, int v_w, int v_h, int color_depth)
 
    while (TRUE) {
       /* let the window thread do the hard work */ 
-      _TRACE("setting display mode(%u, %u, %u, %u)\n",
+      _TRACE(PREFIX_I "setting display mode(%u, %u, %u, %u)\n",
              _wnd_width, _wnd_height, _wnd_depth, _wnd_refresh_rate);
 
       if (wnd_call_proc(wnd_set_video_mode) == 0) {
@@ -392,7 +398,7 @@ int set_video_mode(int w, int h, int v_w, int v_h, int color_depth)
          break;
    }
       
-   _TRACE("Unable to set any display mode.\n");
+   _TRACE(PREFIX_E "Unable to set any display mode.\n");
    return -1;
 
  Done:
