@@ -20,10 +20,14 @@
 
 #include "wddraw.h"
 
+#define PREFIX_I                "al-wddwin INFO: "
+#define PREFIX_W                "al-wddwin WARNING: "
+#define PREFIX_E                "al-wddwin ERROR: "
+
 
 /* exported only for asmlock.s */
-char* wd_dirty_lines = NULL;    /* used in WRITE_BANK()       */
-void (*update_window) (RECT* rect) = NULL;  /* window updater */
+char *wd_dirty_lines = NULL;  /* used in WRITE_BANK() */
+void (*update_window) (RECT *rect) = NULL;  /* window updater */
 
 
 static void gfx_directx_set_palette_win(AL_CONST struct RGB *p, int from, int to, int vsync);
@@ -54,13 +58,14 @@ GFX_DRIVER gfx_directx_win =
    NULL,
    gfx_directx_create_system_bitmap,
    gfx_directx_destroy_system_bitmap,
-   NULL,                        // AL_METHOD(int, set_mouse_sprite, (struct BITMAP *sprite, int xfocus, int yfocus));
-   NULL,                        // AL_METHOD(int, show_mouse, (struct BITMAP *bmp, int x, int y));
-   NULL,                        // AL_METHOD(void, hide_mouse, (void));
-   NULL,                        // AL_METHOD(void, move_mouse, (int x, int y));
+   gfx_directx_set_mouse_sprite,
+   gfx_directx_show_mouse,
+   gfx_directx_hide_mouse,
+   gfx_directx_move_mouse,
    NULL,                        // AL_METHOD(void, drawing_mode, (void));
    NULL,                        // AL_METHOD(void, save_video_state, (void*));
    NULL,                        // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                        // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    NULL,                        // AL_METHOD(int, fetch_mode_list, (void));
    0, 0,                        // int w, h;
    TRUE,                        // int linear;
@@ -136,7 +141,7 @@ static void handle_window_enter_sysmode_win(void)
    if (direct_updating_mode_enabled && colorconv_blit) {
       if (direct_updating_mode_on) {
          direct_updating_mode_on = FALSE;
-         _TRACE("direct updating mode off\n");
+         _TRACE(PREFIX_I "direct updating mode off\n");
       }
    }
 }
@@ -151,7 +156,7 @@ static void handle_window_exit_sysmode_win(void)
    if (direct_updating_mode_enabled && colorconv_blit) {
       if (!direct_updating_mode_on) {
          direct_updating_mode_on = TRUE;
-         _TRACE("direct updating mode on\n");
+         _TRACE(PREFIX_I "direct updating mode on\n");
       }
    }
 }
@@ -165,15 +170,16 @@ static void handle_window_move_win(int x, int y, int w, int h)
 {
    int xmod;
    RECT window_rect;
+   HWND allegro_wnd = win_get_window();
 
    if (colorconv_blit) {
-      if (((BYTES_PER_PIXEL(desktop_depth) == 1) && (xmod=ABS(x)%4)) ||
-          ((BYTES_PER_PIXEL(desktop_depth) == 2) && (xmod=ABS(x)%2)) ||
-          ((BYTES_PER_PIXEL(desktop_depth) == 3) && (xmod=ABS(x)%4))) {
+      if (((BYTES_PER_PIXEL(_win_desktop_depth) == 1) && (xmod=ABS(x)%4)) ||
+          ((BYTES_PER_PIXEL(_win_desktop_depth) == 2) && (xmod=ABS(x)%2)) ||
+          ((BYTES_PER_PIXEL(_win_desktop_depth) == 3) && (xmod=ABS(x)%4))) {
          GetWindowRect(allegro_wnd, &window_rect);
          SetWindowPos(allegro_wnd, 0, window_rect.left + (x > 0 ? -xmod : xmod),
                       window_rect.top, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
-         _TRACE("window shifted by %d pixe%s to the %s to enforce alignment\n",
+         _TRACE(PREFIX_I "window shifted by %d pixe%s to the %s to enforce alignment\n",
                 xmod, xmod > 1 ? "ls" : "l", x > 0 ? "left" : "right");
       }
    }
@@ -193,7 +199,7 @@ static void paint_win(RECT *rect)
    /* we may have lost the DirectDraw surfaces
     * (e.g after the monitor has gone to low power)
     */
-   if (IDirectDrawSurface2_IsLost(primary_surface->id))
+   if (IDirectDrawSurface2_IsLost(gfx_directx_primary_surface->id))
       switch_in_win();
 
    /* clip the rectangle */
@@ -208,31 +214,35 @@ static void paint_win(RECT *rect)
 /* update_matching_window:
  *  Updates a portion of the window when the color depths match.
  */
-static void update_matching_window(RECT* rect)
+static void update_matching_window(RECT *rect)
 {
-   RECT dest_rect;
+   HWND allegro_wnd = win_get_window();
+   union {
+     POINT p;
+     RECT r;
+   } dest_rect;
 
    _enter_gfx_critical();
 
-   if (!forefront_bitmap) {
+   if (!gfx_directx_forefront_bitmap) {
       _exit_gfx_critical();
       return;
    }
 
    if (rect)
-      dest_rect = *rect;
+      dest_rect.r = *rect;
    else {
-      dest_rect.left   = 0;
-      dest_rect.right  = gfx_directx_win.w;
-      dest_rect.top    = 0;
-      dest_rect.bottom = gfx_directx_win.h;
+      dest_rect.r.left   = 0;
+      dest_rect.r.right  = gfx_directx_win.w;
+      dest_rect.r.top    = 0;
+      dest_rect.r.bottom = gfx_directx_win.h;
    }
 
-   ClientToScreen(allegro_wnd, (LPPOINT)&dest_rect);
-   ClientToScreen(allegro_wnd, (LPPOINT)&dest_rect + 1);
+   ClientToScreen(allegro_wnd, &dest_rect.p);
+   ClientToScreen(allegro_wnd, &dest_rect.p + 1);
  
    /* blit offscreen backbuffer to the window */
-   IDirectDrawSurface2_Blt(primary_surface->id, &dest_rect,
+   IDirectDrawSurface2_Blt(gfx_directx_primary_surface->id, &dest_rect.r,
                            offscreen_surface->id, rect, 0, NULL);
 
    _exit_gfx_critical();
@@ -307,16 +317,21 @@ static int ddsurf_blit_ex(LPDIRECTDRAWSURFACE2 dest_surf, RECT *dest_rect,
 /* update_colorconv_window:
  *  Updates a portion of the window when the color depths don't match.
  */
-static void update_colorconv_window(RECT* rect)
+static void update_colorconv_window(RECT *rect)
 {
-   RECT src_rect, dest_rect;
+   RECT src_rect;
+   union {
+     POINT p;
+     RECT r;
+   } dest_rect;
    HDC src_dc, dest_dc;
    HRESULT hr;
    int direct;
+   HWND allegro_wnd = win_get_window();
 
    _enter_gfx_critical();
 
-   if (!forefront_bitmap) {
+   if (!gfx_directx_forefront_bitmap) {
       _exit_gfx_critical();
       return;
    }
@@ -339,17 +354,17 @@ static void update_colorconv_window(RECT* rect)
       src_rect.bottom = gfx_directx_win.h;
    }
 
-   dest_rect = src_rect;
-   ClientToScreen(allegro_wnd, (LPPOINT)&dest_rect);
-   ClientToScreen(allegro_wnd, (LPPOINT)&dest_rect + 1);
+   dest_rect.r = src_rect;
+   ClientToScreen(allegro_wnd, &dest_rect.p);
+   ClientToScreen(allegro_wnd, &dest_rect.p + 1);
 
    direct = (direct_updating_mode_on &&
-             is_contained(&dest_rect, &working_area) &&
+             is_contained(&dest_rect.r, &working_area) &&
              GetForegroundWindow() == allegro_wnd);
 
    if (direct) {
       /* blit directly to the primary surface without clipping */
-      ddsurf_blit_ex(primary_surface->id, &dest_rect,
+      ddsurf_blit_ex(gfx_directx_primary_surface->id, &dest_rect.r,
                      offscreen_surface->id, &src_rect);
    }
    else {
@@ -392,9 +407,9 @@ static BITMAP *gfx_directx_create_video_bitmap_win(int width, int height)
    BITMAP *bmp;
 
    /* try to detect page flipping and triple buffering patterns */
-   if ((width == forefront_bitmap->w) && (height == forefront_bitmap->h) && (!reused_offscreen_surface)) {
+   if ((width == gfx_directx_forefront_bitmap->w) && (height == gfx_directx_forefront_bitmap->h) && (!reused_offscreen_surface)) {
       /* recycle the offscreen surface as a video bitmap */
-      bmp = make_bitmap_from_surface(offscreen_surface, width, height, BMP_ID_VIDEO);
+      bmp = gfx_directx_make_bitmap_from_surface(offscreen_surface, width, height, BMP_ID_VIDEO);
       if (bmp) {
          bmp->vtable = &_special_vtable;
          bmp->write_bank = gfx_directx_write_bank_win;
@@ -418,7 +433,7 @@ static BITMAP *gfx_directx_create_video_bitmap_win(int width, int height)
       return NULL;
 
    /* create the bitmap that wraps up the surface */
-   bmp = make_bitmap_from_surface(surf, width, height, BMP_ID_VIDEO);
+   bmp = gfx_directx_make_bitmap_from_surface(surf, width, height, BMP_ID_VIDEO);
    if (!bmp) {
       gfx_directx_destroy_surface(surf);
       return NULL;
@@ -468,7 +483,7 @@ static int gfx_directx_show_video_bitmap_win(BITMAP *bmp)
    former_visible_bmp = offscreen_surface->parent_bmp;
 
    /* manually flip the offscreen surface */
-   DDRAW_SURFACE_OF(forefront_bitmap) = surf;
+   gfx_directx_forefront_bitmap->extra = surf;
    offscreen_surface = surf;
 
    /* restore regular methods for video bitmaps */
@@ -502,7 +517,7 @@ static void gfx_directx_set_palette_win_8(AL_CONST struct RGB *p, int from, int
    cmap = _get_colorconv_map();
 
    for (n = from; n <= to; n++)
-      cmap[n] = desktop_rgb_map.data[p[n].r>>1][p[n].g>>1][p[n].b>>1];
+      cmap[n] = _win_desktop_rgb_map.data[p[n].r>>1][p[n].g>>1][p[n].b>>1];
 
    update_window(NULL);
 }
@@ -531,10 +546,11 @@ static void gfx_directx_set_palette_win(AL_CONST struct RGB *p, int from, int to
 static int wnd_set_windowed_coop(void)
 {
    HRESULT hr;
-
+   HWND allegro_wnd = win_get_window();
+   
    hr = IDirectDraw2_SetCooperativeLevel(directdraw, allegro_wnd, DDSCL_NORMAL);
    if (FAILED(hr)) {
-      _TRACE("SetCooperative level = %s (%x), hwnd = %x\n", win_err_str(hr), hr, allegro_wnd);
+      _TRACE(PREFIX_E "SetCooperative level = %s (%x), hwnd = %x\n", win_err_str(hr), hr, allegro_wnd);
       return -1;
    }
 
@@ -558,11 +574,11 @@ static int verify_color_depth (int color_depth)
    }
    else {
       /* disallow cross-conversion between 15-bit and 16-bit colors */
-      if ((BYTES_PER_PIXEL(color_depth) == 2) && (BYTES_PER_PIXEL(desktop_depth) == 2))
+      if ((BYTES_PER_PIXEL(color_depth) == 2) && (BYTES_PER_PIXEL(_win_desktop_depth) == 2))
          return -1;
 
       /* the color depths don't match, need color conversion */
-      colorconv_blit = _get_colorconv_blitter(color_depth, desktop_depth);
+      colorconv_blit = _get_colorconv_blitter(color_depth, _win_desktop_depth);
 
       if (!colorconv_blit)
          return -1;
@@ -601,7 +617,7 @@ static int create_offscreen(int w, int h, int color_depth)
       offscreen_surface = gfx_directx_create_surface(w, h, NULL, DDRAW_SURFACE_VIDEO);
 
       if (!offscreen_surface) {
-         _TRACE("Can't create offscreen surface in video memory.\n");
+         _TRACE(PREFIX_E "Can't create offscreen surface in video memory.\n");
 
          /* create offscreen surface in system memory */
          offscreen_surface = gfx_directx_create_surface(w, h, NULL, DDRAW_SURFACE_SYSTEM);
@@ -609,7 +625,7 @@ static int create_offscreen(int w, int h, int color_depth)
    }
 
    if (!offscreen_surface) {
-      _TRACE("Can't create offscreen surface.\n");
+      _TRACE(PREFIX_E "Can't create offscreen surface.\n");
       return -1;
    }
 
@@ -617,17 +633,17 @@ static int create_offscreen(int w, int h, int color_depth)
 }
 
 
-/* setup_driver_desc:
+/* gfx_directx_setup_driver_desc:
  *  Sets the driver description string.
  */
-static void setup_driver_desc(void)
+static void gfx_directx_setup_driver_desc(void)
 {
    char tmp1[128], tmp2[128];
 
    uszprintf(gfx_driver_desc, sizeof(gfx_driver_desc),
 	     uconvert_ascii("DirectDraw, in %s, %d bpp window", tmp1),
 	     uconvert_ascii((colorconv_blit ? "color conversion" : "matching"), tmp2),
-	     desktop_depth);
+	     _win_desktop_depth);
    
    gfx_directx_win.desc = gfx_driver_desc;
 }
@@ -642,6 +658,7 @@ static struct BITMAP *init_directx_win(int w, int h, int v_w, int v_h, int color
    unsigned char *cmap;
    HRESULT hr;
    int i;
+   HWND allegro_wnd = win_get_window();
 
    /* flipping is impossible in windowed mode */
    if ((v_w != w && v_w != 0) || (v_h != h && v_h != 0)) {
@@ -669,7 +686,7 @@ static struct BITMAP *init_directx_win(int w, int h, int v_w, int v_h, int color
       goto Error;
 
    if (adjust_window(w, h) != 0) {
-      _TRACE("window size not supported.\n");
+      _TRACE(PREFIX_E "window size not supported.\n");
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Resolution not supported"));
       goto Error;
    }
@@ -678,14 +695,14 @@ static struct BITMAP *init_directx_win(int w, int h, int v_w, int v_h, int color
    get_working_area(&working_area);
 
    /* create primary surface */
-   if (create_primary() != 0)
+   if (gfx_directx_create_primary() != 0)
       goto Error;
 
    /* create clipper */
-   if (create_clipper(allegro_wnd) != 0)
+   if (gfx_directx_create_clipper(allegro_wnd) != 0)
       goto Error;
 
-   hr = IDirectDrawSurface_SetClipper(primary_surface->id, ddclipper);
+   hr = IDirectDrawSurface_SetClipper(gfx_directx_primary_surface->id, ddclipper);
    if (FAILED(hr))
       goto Error;
 
@@ -696,7 +713,7 @@ static struct BITMAP *init_directx_win(int w, int h, int v_w, int v_h, int color
    }
 
    /* setup color management */
-   if (desktop_depth == 8) {
+   if (_win_desktop_depth == 8) {
       build_desktop_rgb_map();
 
       if (color_depth == 8) {
@@ -706,7 +723,7 @@ static struct BITMAP *init_directx_win(int w, int h, int v_w, int v_h, int color
          cmap = _get_colorconv_map();
 
          for (i=0; i<4096; i++)
-            cmap[i] = desktop_rgb_map.data[((i&0xF00)>>7) | ((i&0x800)>>11)]
+            cmap[i] = _win_desktop_rgb_map.data[((i&0xF00)>>7) | ((i&0x800)>>11)]
                                           [((i&0x0F0)>>3) | ((i&0x080)>>7)]
                                           [((i&0x00F)<<1) | ((i&0x008)>>3)];
 
@@ -718,11 +735,11 @@ static struct BITMAP *init_directx_win(int w, int h, int v_w, int v_h, int color
       if (color_depth == 8) {
          gfx_directx_win.set_palette = gfx_directx_set_palette_win;
 
-         if (create_palette(offscreen_surface) != 0)
+         if (gfx_directx_create_palette(offscreen_surface) != 0)
             goto Error;
 
          /* init the core library color conversion functions */ 
-         if (gfx_directx_update_color_format(primary_surface, desktop_depth) != 0)
+         if (gfx_directx_update_color_format(gfx_directx_primary_surface, _win_desktop_depth) != 0)
             goto Error;
       }
       else {
@@ -732,18 +749,18 @@ static struct BITMAP *init_directx_win(int w, int h, int v_w, int v_h, int color
    }
 
    /* setup Allegro gfx driver */
-   setup_driver_desc();
-   if (setup_driver(&gfx_directx_win, w, h, color_depth) != 0)
+   gfx_directx_setup_driver_desc();
+   if (gfx_directx_setup_driver(&gfx_directx_win, w, h, color_depth) != 0)
       goto Error;
 
-   forefront_bitmap = make_bitmap_from_surface(offscreen_surface, w, h, BMP_ID_VIDEO);
+   gfx_directx_forefront_bitmap = gfx_directx_make_bitmap_from_surface(offscreen_surface, w, h, BMP_ID_VIDEO);
 
-   enable_acceleration(&gfx_directx_win);
+   gfx_directx_enable_acceleration(&gfx_directx_win);
    memcpy(&_special_vtable, &_screen_vtable, sizeof (GFX_VTABLE));
    _special_vtable.release = gfx_directx_unlock_win;
    _special_vtable.unwrite_bank = gfx_directx_unwrite_bank_win;
-   forefront_bitmap->vtable = &_special_vtable;
-   forefront_bitmap->write_bank = gfx_directx_write_bank_win;
+   gfx_directx_forefront_bitmap->vtable = &_special_vtable;
+   gfx_directx_forefront_bitmap->write_bank = gfx_directx_write_bank_win;
 
    /* the last flag serves as end of loop delimiter */
    wd_dirty_lines = calloc(h+1, sizeof(char));
@@ -760,7 +777,7 @@ static struct BITMAP *init_directx_win(int w, int h, int v_w, int v_h, int color
 
    _exit_critical();
 
-   return forefront_bitmap;
+   return gfx_directx_forefront_bitmap;
 
  Error:
    _exit_critical();
@@ -799,7 +816,7 @@ static void gfx_directx_win_exit(struct BITMAP *bmp)
       gfx_directx_destroy_surface(offscreen_surface);
       offscreen_surface = NULL;
       reused_offscreen_surface = FALSE;
-      forefront_bitmap = NULL;
+      gfx_directx_forefront_bitmap = NULL;
    }
 
    /* release the color conversion blitter */
