@@ -43,6 +43,10 @@
 #error something is wrong with the makefile
 #endif
 
+#define PREFIX_I                "al-dsinput INFO: "
+#define PREFIX_W                "al-dsinput WARNING: "
+#define PREFIX_E                "al-dsinput ERROR: "
+
 
 /* sound driver globals */
 static LPDIRECTSOUNDCAPTURE ds_capture = NULL;
@@ -65,39 +69,39 @@ static char *ds_err(long err)
    switch (err) {
 
       case DS_OK:
-         strcpy(err_str, "DS_OK");
+         _al_sane_strncpy(err_str, "DS_OK", sizeof(err_str));
          break;
 
       case DSERR_ALLOCATED:
-         strcpy(err_str, "DSERR_ALLOCATED");
+         _al_sane_strncpy(err_str, "DSERR_ALLOCATED", sizeof(err_str));
          break;
 
       case DSERR_BADFORMAT:
-         strcpy(err_str, "DSERR_BADFORMAT");
+         _al_sane_strncpy(err_str, "DSERR_BADFORMAT", sizeof(err_str));
          break;
 
       case DSERR_INVALIDPARAM:
-         strcpy(err_str, "DSERR_INVALIDPARAM");
+         _al_sane_strncpy(err_str, "DSERR_INVALIDPARAM", sizeof(err_str));
          break;
 
       case DSERR_NOAGGREGATION:
-         strcpy(err_str, "DSERR_NOAGGREGATION");
+         _al_sane_strncpy(err_str, "DSERR_NOAGGREGATION", sizeof(err_str));
          break;
 
       case DSERR_OUTOFMEMORY:
-         strcpy(err_str, "DSERR_OUTOFMEMORY");
+         _al_sane_strncpy(err_str, "DSERR_OUTOFMEMORY", sizeof(err_str));
          break;
 
       case DSERR_UNINITIALIZED:
-         strcpy(err_str, "DSERR_UNINITIALIZED");
+         _al_sane_strncpy(err_str, "DSERR_UNINITIALIZED", sizeof(err_str));
          break;
 
       case DSERR_UNSUPPORTED:
-         strcpy(err_str, "DSERR_UNSUPPORTED");
+         _al_sane_strncpy(err_str, "DSERR_UNSUPPORTED", sizeof(err_str));
          break;
 
       default:
-         strcpy(err_str, "DSERR_UNKNOWN");
+         _al_sane_strncpy(err_str, "DSERR_UNKNOWN", sizeof(err_str));
          break;
    }
 
@@ -209,7 +213,7 @@ static int get_capture_format_support(int bits, int stereo, int rate,
    dsCaps.dwSize = sizeof(DSCCAPS);
    hr = IDirectSoundCapture_GetCaps(ds_capture, &dsCaps);
    if (FAILED(hr)) {
-      _TRACE("Can't get input device caps (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_E "Can't get input device caps (%s).\n", ds_err(hr));
       return -1;
    }
 
@@ -250,7 +254,7 @@ static int get_capture_format_support(int bits, int stereo, int rate,
    if (!wfx)
       free(test_wfx);
 
-   _TRACE("No valid recording formats found.\n");
+   _TRACE(PREFIX_W "No valid recording formats found.\n");
    return -1;
 }
 
@@ -263,6 +267,7 @@ int digi_directsound_capture_init(LPGUID guid)
    DSCCAPS dsCaps;
    WAVEFORMATEX wfx;
    HRESULT hr;
+   LPVOID temp;
 
    /* the DirectSoundCapture interface is not part of DirectX 3 */
    if (_dx_ver < 0x0500)
@@ -272,21 +277,23 @@ int digi_directsound_capture_init(LPGUID guid)
     *  we use CoCreateInstance() instead of DirectSoundCaptureCreate() to avoid
     *  the dll loader blocking the start of Allegro under DirectX 3.
     */
-   hr = CoCreateInstance(&CLSID_DirectSoundCapture, NULL, CLSCTX_INPROC_SERVER,
-                         &IID_IDirectSoundCapture, (LPVOID *)&ds_capture);
 
+   hr = CoCreateInstance(&CLSID_DirectSoundCapture, NULL, CLSCTX_INPROC_SERVER,
+                         &IID_IDirectSoundCapture, &temp);
    if (FAILED(hr)) {
-      _TRACE("Can't create DirectSoundCapture interface (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_E "Can't create DirectSoundCapture interface (%s).\n", ds_err(hr));
       goto Error;
    }
 
+   ds_capture = temp;
+
    /* initialize the device */
    hr = IDirectSoundCapture_Initialize(ds_capture, guid);
 
    if (FAILED(hr)) {
       hr = IDirectSoundCapture_Initialize(ds_capture, NULL);
       if (FAILED(hr)) {
-         _TRACE("Can't initialize DirectSoundCapture interface (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "Can't initialize DirectSoundCapture interface (%s).\n", ds_err(hr));
          goto Error;
       }
    }
@@ -295,13 +302,13 @@ int digi_directsound_capture_init(LPGUID guid)
    dsCaps.dwSize = sizeof(DSCCAPS);
    hr = IDirectSoundCapture_GetCaps(ds_capture, &dsCaps);
    if (FAILED(hr)) {
-      _TRACE("Can't get input device caps (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_E "Can't get input device caps (%s).\n", ds_err(hr));
       goto Error;
    }
 
    /* cool little 'autodetection' process :) */
    if (get_capture_format_support(0, FALSE, 0, TRUE, &wfx) != 0) {
-      _TRACE("The DirectSound hardware doesn't support any capture types.\n");
+      _TRACE(PREFIX_E "The DirectSound hardware doesn't support any capture types.\n");
       goto Error;
    }
 
@@ -341,6 +348,7 @@ void digi_directsound_capture_exit(void)
 int digi_directsound_capture_detect(LPGUID guid)
 {
    HRESULT hr;
+   LPVOID temp;
 
    /* the DirectSoundCapture interface is not part of DirectX 3 */
    if (_dx_ver < 0x500)
@@ -352,25 +360,27 @@ int digi_directsound_capture_detect(LPGUID guid)
        *  the dll loader blocking the start of Allegro under DirectX 3.
        */
       hr = CoCreateInstance(&CLSID_DirectSoundCapture, NULL, CLSCTX_INPROC_SERVER,
-                            &IID_IDirectSoundCapture, (LPVOID *)&ds_capture);
+                            &IID_IDirectSoundCapture, &temp);
 
       if (FAILED(hr)) {
-         _TRACE("DirectSoundCapture interface creation failed during detect (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "DirectSoundCapture interface creation failed during detect (%s).\n", ds_err(hr));
          return 0;
       }
 
+      ds_capture = temp;
+
       /* initialize the device */
       hr = IDirectSoundCapture_Initialize(ds_capture, guid);
 
       if (FAILED(hr)) {
          hr = IDirectSoundCapture_Initialize(ds_capture, NULL);
          if (FAILED(hr)) {
-            _TRACE("DirectSoundCapture interface initialization failed during detect (%s).\n", ds_err(hr));
+            _TRACE(PREFIX_E "DirectSoundCapture interface initialization failed during detect (%s).\n", ds_err(hr));
             return 0;
          }
       }
 
-      _TRACE("DirectSoundCapture interface successfully created.\n");
+      _TRACE(PREFIX_I "DirectSoundCapture interface successfully created.\n");
 
       /* release DirectSoundCapture interface */
       IDirectSoundCapture_Release(ds_capture);
@@ -464,7 +474,7 @@ int digi_directsound_rec_start(int rate, int bits, int stereo)
 
    hr = IDirectSoundCapture_CreateCaptureBuffer(ds_capture, &dscBufDesc, &ds_capture_buf, NULL);
    if (FAILED(hr)) {
-      _TRACE("Can't create the DirectSound capture buffer (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_E "Can't create the DirectSound capture buffer (%s).\n", ds_err(hr));
       return 0;
    }
 
@@ -473,7 +483,7 @@ int digi_directsound_rec_start(int rate, int bits, int stereo)
       IDirectSoundCaptureBuffer_Release(ds_capture_buf);
       ds_capture_buf = NULL;
 
-      _TRACE("Can't start the DirectSound capture buffer (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_E "Can't start the DirectSound capture buffer (%s).\n", ds_err(hr));
       return 0;
    }
 
@@ -515,6 +525,7 @@ int digi_directsound_rec_read(void *buf)
    unsigned long int capture_pos;
    HRESULT hr;
    BOOL buffer_filled = FALSE;
+   LPVOID temp1, temp2;
 
    if (!ds_capture || !ds_capture_buf || !input_wave_data)
       return 0;
@@ -535,12 +546,15 @@ int digi_directsound_rec_read(void *buf)
    }
 
    hr = IDirectSoundCaptureBuffer_Lock(ds_capture_buf, last_capture_pos,
-                                       bytes_to_lock, (LPVOID *)&input_ptr1,
-                                       &input_bytes1, (LPVOID *)&input_ptr2,
+                                       bytes_to_lock, &temp1,
+                                       &input_bytes1, &temp2,
                                        &input_bytes2, 0);
    if (FAILED(hr))
       return 0;
 
+   input_ptr1 = temp1;
+   input_ptr2 = temp2;
+
    /* let's get the data aligned linearly */
    linear_input_ptr = malloc(bytes_to_lock);
    memcpy(linear_input_ptr, input_ptr1, input_bytes1);
