@@ -47,6 +47,10 @@
 #error something is wrong with the makefile
 #endif
 
+#define PREFIX_I                "al-dsound INFO: "
+#define PREFIX_W                "al-dsound WARNING: "
+#define PREFIX_E                "al-dsound ERROR: "
+
 
 static int digi_directsound_detect(int input);
 static int digi_directsound_init(int input, int voices);
@@ -194,8 +198,11 @@ static BOOL CALLBACK DSEnumCallback(LPGUID lpGuid, LPCSTR lpcstrDescription, LPC
    if (lpGuid) {
       driver_guids[num_drivers] = lpGuid;
       driver_names[num_drivers] = malloc(strlen(lpcstrDescription)+1);
-      strcpy(driver_names[num_drivers], lpcstrDescription);
-      num_drivers++;
+      if(driver_names[num_drivers])
+      {
+         _al_sane_strncpy(driver_names[num_drivers], lpcstrDescription, strlen(lpcstrDescription)+1);
+         num_drivers++;
+      }
    }
 
    return (num_drivers < MAX_DRIVERS);
@@ -221,6 +228,13 @@ _DRIVER_INFO *_get_win_digi_driver_list(void)
       hr = DirectSoundEnumerate(DSEnumCallback, NULL);
 
       if (hr == DS_OK) {
+         /* Allegro mixer to DirectSound drivers */
+         for (i=0; i<num_drivers; i++) {
+            driver = _get_dsalmix_driver(driver_names[i], driver_guids[i], i);
+
+            _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
+         }
+
          /* pure DirectSound drivers */
          for (i=0; i<num_drivers; i++) {
             driver = malloc(sizeof(DIGI_DRIVER));
@@ -230,13 +244,6 @@ _DRIVER_INFO *_get_win_digi_driver_list(void)
 
             _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
          }
-
-         /* Allegro mixer to DirectSound drivers */
-         for (i=0; i<num_drivers; i++) {
-            driver = _get_dsalmix_driver(driver_names[i], driver_guids[i], i);
-
-            _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
-         }
       }
 
       /* Allegro mixer to WaveOut drivers */
@@ -283,39 +290,39 @@ static char *ds_err(long err)
    switch (err) {
 
       case DS_OK:
-         strcpy(err_str, "DS_OK");
+         _al_sane_strncpy(err_str, "DS_OK", sizeof(err_str));
          break;
 
       case DSERR_ALLOCATED:
-         strcpy(err_str, "DSERR_ALLOCATED");
+         _al_sane_strncpy(err_str, "DSERR_ALLOCATED", sizeof(err_str));
          break;
 
       case DSERR_BADFORMAT:
-         strcpy(err_str, "DSERR_BADFORMAT");
+         _al_sane_strncpy(err_str, "DSERR_BADFORMAT", sizeof(err_str));
          break;
 
       case DSERR_INVALIDPARAM:
-         strcpy(err_str, "DSERR_INVALIDPARAM");
+         _al_sane_strncpy(err_str, "DSERR_INVALIDPARAM", sizeof(err_str));
          break;
 
       case DSERR_NOAGGREGATION:
-         strcpy(err_str, "DSERR_NOAGGREGATION");
+         _al_sane_strncpy(err_str, "DSERR_NOAGGREGATION", sizeof(err_str));
          break;
 
       case DSERR_OUTOFMEMORY:
-         strcpy(err_str, "DSERR_OUTOFMEMORY");
+         _al_sane_strncpy(err_str, "DSERR_OUTOFMEMORY", sizeof(err_str));
          break;
 
       case DSERR_UNINITIALIZED:
-         strcpy(err_str, "DSERR_UNINITIALIZED");
+         _al_sane_strncpy(err_str, "DSERR_UNINITIALIZED", sizeof(err_str));
          break;
 
       case DSERR_UNSUPPORTED:
-         strcpy(err_str, "DSERR_UNSUPPORTED");
+         _al_sane_strncpy(err_str, "DSERR_UNSUPPORTED", sizeof(err_str));
          break;
 
       default:
-         strcpy(err_str, "DSERR_UNKNOWN");
+         _al_sane_strncpy(err_str, "DSERR_UNKNOWN", sizeof(err_str));
          break;
    }
 
@@ -344,11 +351,11 @@ static int digi_directsound_detect(int input)
       /* initialize DirectSound interface */
       hr = DirectSoundCreate(driver_guids[id], &directsound, NULL);
       if (FAILED(hr)) {
-         _TRACE("DirectSound interface creation failed during detect (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "DirectSound interface creation failed during detect (%s).\n", ds_err(hr));
          return 0;
       }
 
-      _TRACE("DirectSound interface successfully created.\n");
+      _TRACE(PREFIX_I "DirectSound interface successfully created.\n");
 
       /* release DirectSound interface */
       IDirectSound_Release(directsound);
@@ -369,7 +376,8 @@ static int digi_directsound_init(int input, int voices)
    DSBUFFERDESC desc;
    WAVEFORMATEX format;
    int v, id;
-
+   HWND allegro_wnd = win_get_window();
+   
    /* deduce our device number from the driver ID code */
    id = ((digi_driver->id >> 8) & 0xFF) - 'A';
 
@@ -381,41 +389,42 @@ static int digi_directsound_init(int input, int voices)
    /* initialize DirectSound interface */
    hr = DirectSoundCreate(driver_guids[id], &directsound, NULL);
    if (FAILED(hr)) { 
-      _TRACE("Can't create DirectSound interface (%s).\n", ds_err(hr)); 
+      _TRACE(PREFIX_E "Can't create DirectSound interface (%s).\n", ds_err(hr));
       goto Error; 
    }
 
    /* set cooperative level */
    hr = IDirectSound_SetCooperativeLevel(directsound, allegro_wnd, DSSCL_PRIORITY);
    if (FAILED(hr))
-      _TRACE("Can't set DirectSound cooperative level (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_W "Can't set DirectSound cooperative level (%s).\n", ds_err(hr));
    else
-      _TRACE("DirectSound cooperation level set to PRIORITY.\n");
+      _TRACE(PREFIX_I "DirectSound cooperation level set to PRIORITY.\n");
 
    /* get hardware capabilities */
    dscaps.dwSize = sizeof(DSCAPS);
    hr = IDirectSound_GetCaps(directsound, &dscaps);
    if (FAILED(hr)) { 
-      _TRACE("Can't get DirectSound caps (%s).\n", ds_err(hr)); 
+      _TRACE(PREFIX_E "Can't get DirectSound caps (%s).\n", ds_err(hr));
       goto Error; 
    }
 
-   if (dscaps.dwFlags & DSCAPS_PRIMARY16BIT)
+   if ((dscaps.dwFlags & DSCAPS_PRIMARY16BIT) &&
+       ((_sound_bits >= 16) || (_sound_bits <= 0)))
       _bits = 16;
    else
       _bits = 8;
 
-   if (dscaps.dwFlags & DSCAPS_PRIMARYSTEREO)
+   if ((dscaps.dwFlags & DSCAPS_PRIMARYSTEREO) && _sound_stereo)
       _stereo = 1;
    else
       _stereo = 0;
 
-   if (dscaps.dwMaxSecondarySampleRate > 44000)
-      _freq = 44100;
+   if ((dscaps.dwMaxSecondarySampleRate > (DWORD)_sound_freq) && (_sound_freq > 0))
+      _freq = _sound_freq;
    else
-      _freq = 22050;
+      _freq = dscaps.dwMaxSecondarySampleRate;
 
-   _TRACE("DirectSound caps: %u bits, %s, %uHz\n", _bits, _stereo ? "stereo" : "mono", _freq);
+   _TRACE(PREFIX_I "DirectSound caps: %u bits, %s, %uHz\n", _bits, _stereo ? "stereo" : "mono", _freq);
 
    /* create primary buffer */
    memset(&desc, 0, sizeof(DSBUFFERDESC));
@@ -424,14 +433,14 @@ static int digi_directsound_init(int input, int voices)
 
    hr = IDirectSound_CreateSoundBuffer(directsound, &desc, &prim_buf, NULL);
    if (hr != DS_OK) { 
-      _TRACE("Can't create primary buffer (%s)\nGlobal volume control won't be available.\n", ds_err(hr)); 
+      _TRACE(PREFIX_W "Can't create primary buffer (%s)\nGlobal volume control won't be available.\n", ds_err(hr));
    }
 
    /* get current format */
    if (prim_buf) {
       hr = IDirectSoundBuffer_GetFormat(prim_buf, &format, sizeof(format), NULL);
       if (FAILED(hr)) {
-         _TRACE("Can't get primary buffer format (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_W "Can't get primary buffer format (%s).\n", ds_err(hr));
       }
       else {
          format.nChannels = _stereo ? 2 : 1;
@@ -442,17 +451,17 @@ static int digi_directsound_init(int input, int voices)
 
          hr = IDirectSoundBuffer_SetFormat(prim_buf, &format);
          if (FAILED(hr)) {
-            _TRACE("Can't set primary buffer format (%s).\n", ds_err(hr));
+            _TRACE(PREFIX_W "Can't set primary buffer format (%s).\n", ds_err(hr));
          }
 
          /* output primary format */
          hr = IDirectSoundBuffer_GetFormat(prim_buf, &format, sizeof(format), NULL);
          if (FAILED(hr)) {
-            _TRACE("Can't get primary buffer format (%s).\n", ds_err(hr));
+            _TRACE(PREFIX_W "Can't get primary buffer format (%s).\n", ds_err(hr));
          }
          else {
-            _TRACE("primary format:\n");
-            _TRACE("  %u channels\n  %u Hz\n  %u AvgBytesPerSec\n  %u BlockAlign\n  %u bits\n  %u size\n",
+            _TRACE(PREFIX_I "primary format:\n");
+            _TRACE(PREFIX_I "  %u channels\n  %u Hz\n  %u AvgBytesPerSec\n  %u BlockAlign\n  %u bits\n  %u size\n",
             format.nChannels, format.nSamplesPerSec, format.nAvgBytesPerSec,
             format.nBlockAlign, format.wBitsPerSample, format.cbSize);
          }
@@ -487,7 +496,7 @@ static int digi_directsound_init(int input, int voices)
    return 0;
 
  Error:
-   _TRACE("digi_directsound_init() failed.\n");
+   _TRACE(PREFIX_E "digi_directsound_init() failed.\n");
    digi_directsound_exit(FALSE);
    return -1;
 }
@@ -557,19 +566,19 @@ static int digi_directsound_mixer_volume(int volume)
 static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int stereo, int vol, int pan)
 {
    LPDIRECTSOUNDBUFFER snd_buf;
-   PCMWAVEFORMAT pcmwf;
+   WAVEFORMATEX wf;
    DSBUFFERDESC dsbdesc;
    HRESULT hr;
    int switch_mode;
 
    /* setup wave format structure */
-   memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));
-   pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
-   pcmwf.wf.nChannels = stereo ? 2 : 1;
-   pcmwf.wf.nSamplesPerSec = freq;
-   pcmwf.wBitsPerSample = bits;
-   pcmwf.wf.nBlockAlign = bits * (stereo ? 2 : 1) / 8;
-   pcmwf.wf.nAvgBytesPerSec = pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
+   memset(&wf, 0, sizeof(WAVEFORMATEX));
+   wf.wFormatTag = WAVE_FORMAT_PCM;
+   wf.nChannels = stereo ? 2 : 1;
+   wf.nSamplesPerSec = freq;
+   wf.wBitsPerSample = bits;
+   wf.nBlockAlign = bits * (stereo ? 2 : 1) / 8;
+   wf.nAvgBytesPerSec = wf.nSamplesPerSec * wf.nBlockAlign;
 
    /* setup DSBUFFERDESC structure */
    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
@@ -583,13 +592,13 @@ static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int
       dsbdesc.dwFlags |= DSBCAPS_GLOBALFOCUS;
 
    dsbdesc.dwBufferBytes = len * (bits / 8) * (stereo ? 2 : 1);
-   dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)&pcmwf;
+   dsbdesc.lpwfxFormat = &wf;
 
    /* create buffer */
    hr = IDirectSound_CreateSoundBuffer(directsound, &dsbdesc, &snd_buf, NULL);
    if (FAILED(hr)) {
-      _TRACE("create_directsound_buffer() failed (%s).\n", ds_err(hr));
-      _TRACE(" - %d Hz, %s, %d bits\n", freq, stereo ? "stereo" : "mono", bits);
+      _TRACE(PREFIX_E "create_directsound_buffer() failed (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_E " - %d Hz, %s, %d bits\n", freq, stereo ? "stereo" : "mono", bits);
       return NULL;
    }
 
@@ -609,8 +618,8 @@ static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int
  */
 static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len, int bits, int stereo, int reversed, char *data)
 {
-   void *buf_a, *buf_b;
-   long int size, size_a, size_b;
+   void *buf_a;
+   long int size, size_a;
    HRESULT hr;
 
    /* transform from samples to bytes */
@@ -626,7 +635,7 @@ static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len,
 
       hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, NULL, NULL, 0);
       if (FAILED(hr)) {
-         _TRACE("fill_dsound_buffer() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "fill_dsound_buffer() failed (%s).\n", ds_err(hr));
          return -1;
       }
    }
@@ -924,7 +933,7 @@ static void *digi_directsound_lock_voice(int voice, int start, int end)
 
       hr = IDirectSoundBuffer_Lock(ds_locked_buffer, start, end - start, &buf_a, &size_a, NULL, NULL, 0);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_lock_voice() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_lock_voice() failed (%s).\n", ds_err(hr));
          return NULL;
       }
    }
@@ -963,7 +972,7 @@ static void digi_directsound_unlock_voice(int voice)
                                      0);
 
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_unlock_voice() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_unlock_voice() failed (%s).\n", ds_err(hr));
       }
 
       ds_voices[voice].ds_locked_buffer = NULL;
@@ -993,7 +1002,7 @@ static int digi_directsound_get_position(int voice)
       hr = IDirectSoundBuffer_GetCurrentPosition(ds_voices[voice].ds_loop_buffer,
                                                  &play_cursor, &write_cursor);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_get_position() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_get_position() failed (%s).\n", ds_err(hr));
          return -1;
       }
 
@@ -1018,7 +1027,7 @@ static int digi_directsound_get_position(int voice)
       hr = IDirectSoundBuffer_GetCurrentPosition(ds_voices[voice].ds_buffer,
                                                  &play_cursor, &write_cursor);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_get_position() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_get_position() failed (%s).\n", ds_err(hr));
          return -1;
       }
 
@@ -1055,7 +1064,7 @@ static void digi_directsound_set_position(int voice, int position)
       hr = IDirectSoundBuffer_SetCurrentPosition(ds_voices[voice].ds_loop_buffer,
                                                  pos * ds_voices[voice].bytes_per_sample);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_set_position() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_set_position() failed (%s).\n", ds_err(hr));
       }
    }
 
@@ -1069,7 +1078,7 @@ static void digi_directsound_set_position(int voice, int position)
       hr = IDirectSoundBuffer_SetCurrentPosition(ds_voices[voice].ds_buffer,
                                                  pos * ds_voices[voice].bytes_per_sample);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_set_position() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_set_position() failed (%s).\n", ds_err(hr));
       }
    }
 }
