@@ -27,6 +27,10 @@
 #error something is wrong with the makefile
 #endif
 
+#define PREFIX_I                "al-wgdi INFO: "
+#define PREFIX_W                "al-wgdi WARNING: "
+#define PREFIX_E                "al-wgdi ERROR: "
+
 
 /* function from asmlock.s */ 
 extern void gfx_gdi_write_bank(void);
@@ -73,6 +77,7 @@ GFX_DRIVER gfx_gdi =
    NULL,                        // AL_METHOD(void, drawing_mode, (void));
    NULL,                        // AL_METHOD(void, save_video_state, (void*));
    NULL,                        // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                        // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    NULL,                        // AL_METHOD(int, fetch_mode_list, (void));
    0, 0,                        // int w, h;
    TRUE,                        // int linear;
@@ -156,6 +161,7 @@ static int gfx_gdi_set_mouse_sprite(struct BITMAP *sprite, int xfocus, int yfocu
 static void update_mouse_pointer(int x, int y, int retrace)
 {
    HDC hdc;
+   HWND allegro_wnd = win_get_window();
 
    /* put the screen contents located at the new position into the frontbuffer */
    blit(gdi_screen, mouse_frontbuffer, x, y, 0, 0, mouse_frontbuffer->w, mouse_frontbuffer->h);
@@ -213,6 +219,7 @@ static int gfx_gdi_show_mouse(struct BITMAP *bmp, int x, int y)
 static void gfx_gdi_hide_mouse(void)
 {
    HDC hdc;
+   HWND allegro_wnd = win_get_window();
 
    if (!mouse_on)
       return; 
@@ -257,6 +264,7 @@ static void render_proc(void)
 {
    int top_line, bottom_line;
    HDC hdc = NULL;
+   HWND allegro_wnd = win_get_window();
 
    /* to prevent reentrant calls */
    if (render_semaphore)
@@ -337,7 +345,7 @@ static void gdi_enter_sysmode(void)
    if (mouse_on) {
       mouse_was_on = TRUE;
       gfx_gdi_hide_mouse();
-      _TRACE("mouse pointer off\n");
+      _TRACE(PREFIX_I "mouse pointer off\n");
    }
 }
 
@@ -350,7 +358,7 @@ static void gdi_exit_sysmode(void)
    if (mouse_was_on) {
       mouse_on = TRUE;
       mouse_was_on = FALSE;
-      _TRACE("mouse pointer on\n");
+      _TRACE(PREFIX_I "mouse pointer on\n");
    }
 }
 
@@ -362,6 +370,7 @@ static void gdi_exit_sysmode(void)
 static void gdi_update_window(RECT *rect)
 {
    HDC hdc;
+   HWND allegro_wnd = win_get_window();
 
    _enter_gfx_critical();
 
@@ -395,14 +404,14 @@ static void gfx_gdi_lock(struct BITMAP *bmp)
    _enter_gfx_critical();
 
    /* arrange for drawing requests to pause when we are in the background */
-   if (!app_foreground) {
+   if (!_win_app_foreground) {
       /* stop timer */
       remove_int(render_proc);
 
       _exit_gfx_critical();
 
       if (GFX_CRITICAL_RELEASED)
-         thread_switch_out();
+         _win_thread_switch_out();
 
       _enter_gfx_critical();
 
@@ -456,7 +465,7 @@ static struct BITMAP *gfx_gdi_init(int w, int h, int v_w, int v_h, int color_dep
    gfx_gdi.h = h;
 
    if (adjust_window(w, h) != 0) {
-      _TRACE("window size not supported.\n");
+      _TRACE(PREFIX_E "window size not supported.\n");
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Resolution not supported"));
       goto Error;
    }
