@@ -12,6 +12,8 @@
  *
  *      By Stefan Schimanski.
  *
+ *      Midi input added by Daniel Verkamp.
+ *
  *      See readme.txt for copyright information.
  */
 
@@ -36,6 +38,10 @@
 #error something is wrong with the makefile
 #endif
 
+#define PREFIX_I                "al-wmidi INFO: "
+#define PREFIX_W                "al-wmidi WARNING: "
+#define PREFIX_E                "al-wmidi ERROR: "
+
 
 int midi_win32_detect(int input);
 int midi_win32_init(int input, int voices);
@@ -43,15 +49,31 @@ void midi_win32_exit(int input);
 int midi_win32_mixer_volume(int volume);
 void midi_win32_raw_midi(int data);
 
+int midi_win32_in_detect(int input);
+int midi_win32_in_init(int input, int voices);
+void midi_win32_in_exit(int input);
+static void CALLBACK midi_in_proc(HMIDIIN, UINT, DWORD, DWORD, DWORD);
+
 
 /* driver globals */
 static HMIDIOUT midi_device = NULL;
+static HMIDIIN  midi_in_device = NULL;
 
 
 /* dynamically generated driver list */
 static _DRIVER_INFO *driver_list = NULL;
 
 
+/* MIDI recording callback */
+static void CALLBACK midi_in_proc(HMIDIIN hMidiIn, UINT wMsg, DWORD dwInstance, 
+				  DWORD dwParam1, DWORD dwParam2)
+{
+   if ((midi_in_device == NULL) || (midi_recorder == NULL)) return;
+   midi_recorder((unsigned char)(dwParam1 & 0xff));         /* status byte */
+   midi_recorder((unsigned char)((dwParam1 >> 8) & 0xff));  /* data byte 1 */
+   midi_recorder((unsigned char)((dwParam1 >> 16) & 0xff)); /* data byte 2 */
+}
+
 
 /* _get_win_midi_driver_list:
  *  System driver hook for listing the available MIDI drivers. This generates
@@ -62,6 +84,7 @@ _DRIVER_INFO *_get_win_midi_driver_list(void)
 {
    MIDI_DRIVER *driver;
    MIDIOUTCAPS caps;
+   MIDIINCAPS  caps_in;
    int num_drivers, i;
 
    if (!driver_list) {
@@ -85,8 +108,7 @@ _DRIVER_INFO *_get_win_midi_driver_list(void)
 
          midiOutGetDevCaps(i-1, &caps, sizeof(caps));
 
-	 driver->ascii_name = malloc(strlen(caps.szPname)+1);
-	 _al_sane_strncpy((char *)driver->ascii_name, caps.szPname, strlen(caps.szPname)+1);
+	 driver->ascii_name = strdup(caps.szPname);
 
 	 driver->detect = midi_win32_detect;
 	 driver->init = midi_win32_init;
@@ -97,6 +119,25 @@ _DRIVER_INFO *_get_win_midi_driver_list(void)
          _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
       }
 
+      /* MidiIn drivers */
+      num_drivers = midiInGetNumDevs();
+      for (i=0; i<num_drivers; i++) {
+	 driver = malloc(sizeof(MIDI_DRIVER));
+	 memcpy(driver, &_midi_none, sizeof(MIDI_DRIVER));
+
+	 driver->id = MIDI_WIN32_IN(i); /* added MIDI_WIN32_IN to alwin.h */
+
+	 midiInGetDevCaps(i, &caps_in, sizeof(caps_in));
+
+	 driver->ascii_name = strdup(caps_in.szPname);
+
+	 driver->detect = midi_win32_in_detect;
+	 driver->init = midi_win32_in_init;
+	 driver->exit = midi_win32_in_exit;
+
+	 _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
+      }
+
       /* cross-platform DIGital MIDi driver */
       _driver_list_append_driver(&driver_list, MIDI_DIGMID, &midi_digmid, TRUE);
    }
@@ -131,7 +172,7 @@ void _free_win_midi_driver_list(void)
  */
 int midi_win32_detect(int input)
 {
-   /* the current driver doesn't support input */
+   /* the input drivers are separate from the output drivers */
    if (input)
       return FALSE;
 
@@ -140,6 +181,18 @@ int midi_win32_detect(int input)
 
 
 
+/* midi_win32_in_detect:
+ */
+int midi_win32_in_detect(int input)
+{
+   if (input)
+      return TRUE;
+
+   return FALSE;
+}
+
+
+
 /* midi_win32_init:
  */
 int midi_win32_init(int input, int voices)
@@ -158,7 +211,7 @@ int midi_win32_init(int input, int voices)
    /* open midi mapper */
    hr = midiOutOpen(&midi_device, id, 0, 0, CALLBACK_NULL);
    if (hr != MMSYSERR_NOERROR) {
-      _TRACE("midiOutOpen failed (%x)\n", hr);
+      _TRACE(PREFIX_E "midiOutOpen failed (%x)\n", hr);
       goto Error;
    }
 
@@ -174,6 +227,34 @@ int midi_win32_init(int input, int voices)
 
 
 
+/* midi_win32_in_init:
+ */
+int midi_win32_in_init(int input, int voices)
+{
+   MMRESULT hr;
+   int id;
+
+   /* deduce our device number from the driver ID code */
+   id = (midi_input_driver->id & 0xFF) - 'A';
+
+   /* open midi input device */
+   hr = midiInOpen(&midi_in_device, id, (DWORD)midi_in_proc,
+		   (DWORD)NULL, CALLBACK_FUNCTION);
+   if (hr != MMSYSERR_NOERROR) {
+      _TRACE(PREFIX_E "midiInOpen failed (%x)\n", hr);
+      midi_win32_in_exit(input);
+      return -1;
+   }
+
+
+   midiInReset(midi_in_device);
+   midiInStart(midi_in_device);
+
+   return 0;
+}
+
+
+
 /* midi_win32_exit:
  */
 void midi_win32_exit(int input)
@@ -189,6 +270,20 @@ void midi_win32_exit(int input)
 
 
 
+/* midi_win32_in_exit:
+ */
+void midi_win32_in_exit(int input)
+{
+   if (midi_in_device != NULL) {
+      midiInStop(midi_in_device);
+      midiInReset(midi_in_device);
+      midiInClose(midi_in_device);
+      midi_in_device = NULL;
+   }
+}
+
+
+
 /* mixer_volume:
  */
 int midi_win32_mixer_volume(int volume)
@@ -233,7 +328,7 @@ void midi_win32_raw_midi(int data)
             switch (get_display_switch_mode()) {
                case SWITCH_AMNESIA:
                case SWITCH_PAUSE:
-	          if (app_foreground)
+	          if (_win_app_foreground)
 	             midiOutShortMsg(midi_device, midi_msg);
 	          else
 	             midiOutReset(midi_device);
