@@ -31,6 +31,13 @@
    #error something is wrong with the makefile
 #endif
 
+#define PREFIX_I                "al-wmouse INFO: "
+#define PREFIX_W                "al-wmouse WARNING: "
+#define PREFIX_E                "al-wmouse ERROR: "
+
+
+static MOUSE_DRIVER mouse_directx;
+
 
 _DRIVER_INFO _mouse_driver_list[] =
 {
@@ -46,9 +53,10 @@ static void mouse_directx_position(int x, int y);
 static void mouse_directx_set_range(int x1, int y1, int x2, int y2);
 static void mouse_directx_set_speed(int xspeed, int yspeed);
 static void mouse_directx_get_mickeys(int *mickeyx, int *mickeyy);
+static void mouse_directx_enable_hardware_cursor(int mode);
+static int mouse_directx_select_system_cursor(int cursor);
 
-
-MOUSE_DRIVER mouse_directx =
+static MOUSE_DRIVER mouse_directx =
 {
    MOUSE_DIRECTX,
    empty_string,
@@ -62,10 +70,15 @@ MOUSE_DRIVER mouse_directx =
    mouse_directx_set_range,
    mouse_directx_set_speed,
    mouse_directx_get_mickeys,
-   NULL                        // AL_METHOD(int, analyse_data, (AL_CONST char *buffer, int size));
+   NULL,                       // AL_METHOD(int, analyse_data, (AL_CONST char *buffer, int size));
+   mouse_directx_enable_hardware_cursor,
+   mouse_directx_select_system_cursor
 };
 
 
+HCURSOR _win_hcursor = NULL;	/* Hardware cursor to display */
+
+
 #define DINPUT_BUFFERSIZE 256
 static HANDLE mouse_input_event = NULL;
 static LPDIRECTINPUT mouse_dinput = NULL;
@@ -88,8 +101,8 @@ static int mouse_sy = 2;
 #define MAF_DEFAULT 1                 /* mouse acceleration parameters */
 static int mouse_accel_fact = MAF_DEFAULT;
 static int mouse_accel_mult = MAF_DEFAULT;
-static int mouse_accel_thr1 = 5;
-static int mouse_accel_thr2 = 16;
+static int mouse_accel_thr1 = 5 * 5;
+static int mouse_accel_thr2 = 16 * 16;
 
 static int mouse_minx = 0;            /* mouse range */
 static int mouse_miny = 0;
@@ -165,30 +178,30 @@ static char* dinput_err_str(long err)
    switch (err) {
 
       case DIERR_ACQUIRED:
-         strcpy(err_str, "the device is acquired");
+         _al_sane_strncpy(err_str, "the device is acquired", sizeof(err_str));
          break;
 
       case DIERR_NOTACQUIRED:
-         strcpy(err_str, "the device is not acquired");
+         _al_sane_strncpy(err_str, "the device is not acquired", sizeof(err_str));
          break;
 
       case DIERR_INPUTLOST:
-         strcpy(err_str, "access to the device was not granted");
+         _al_sane_strncpy(err_str, "access to the device was not granted", sizeof(err_str));
          break;
 
       case DIERR_INVALIDPARAM:
-         strcpy(err_str, "the device does not have a selected data format");
+         _al_sane_strncpy(err_str, "the device does not have a selected data format", sizeof(err_str));
          break;
 
 #ifdef DIERR_OTHERAPPHASPRIO
       /* this is not a legacy DirectX 3 error code */
       case DIERR_OTHERAPPHASPRIO:
-         strcpy(err_str, "can't acquire the device in background");
+         _al_sane_strncpy(err_str, "can't acquire the device in background", sizeof(err_str));
          break;
 #endif
 
       default:
-         strcpy(err_str, "unknown error");
+         _al_sane_strncpy(err_str, "unknown error", sizeof(err_str));
    }
 
    return err_str;
@@ -204,14 +217,24 @@ static char* dinput_err_str(long err)
  */
 static void mouse_dinput_handle_event(int ofs, int data)
 {
+   static int last_data_x = 0;
+   static int last_data_y = 0;
+   static int last_was_x = 0;
+   int mag;
+
    switch (ofs) {
 
       case DIMOFS_X:
          if (!gfx_driver || !gfx_driver->windowed) {
+            if (last_was_x)
+               last_data_y = 0;
+            last_data_x = data;
+            last_was_x = 1;
             if (mouse_accel_mult) {
-               if (ABS(data) >= mouse_accel_thr2)
+               mag = last_data_x*last_data_x + last_data_y*last_data_y;
+               if (mag >= mouse_accel_thr2)
                   data *= (mouse_accel_mult<<1);
-               else if (ABS(data) >= mouse_accel_thr1) 
+               else if (mag >= mouse_accel_thr1) 
                   data *= mouse_accel_mult;
             }
 
@@ -221,10 +244,15 @@ static void mouse_dinput_handle_event(int ofs, int data)
 
       case DIMOFS_Y:
          if (!gfx_driver || !gfx_driver->windowed) {
+            if (!last_was_x)
+               last_data_x = 0;
+            last_data_y = data;
+            last_was_x = 0;
             if (mouse_accel_mult) {
-               if (ABS(data) >= mouse_accel_thr2)
+               mag = last_data_x*last_data_x + last_data_y*last_data_y;
+               if (mag >= mouse_accel_thr2)
                   data *= (mouse_accel_mult<<1);
-               else if (ABS(data) >= mouse_accel_thr1) 
+               else if (mag >= mouse_accel_thr1) 
                   data *= mouse_accel_mult;
             }
 
@@ -263,6 +291,15 @@ static void mouse_dinput_handle_event(int ofs, int data)
          else
             _mouse_b &= ~4;
          break;
+
+      case DIMOFS_BUTTON3:
+         if (data & 0x80) {
+            if (_mouse_on)
+               _mouse_b |= 8;
+         }
+         else
+            _mouse_b &= ~8;
+         break;
    }
 }
 
@@ -291,11 +328,11 @@ static void mouse_dinput_handle(void)
    /* was device lost ? */
    if ((hr == DIERR_NOTACQUIRED) || (hr == DIERR_INPUTLOST)) {
       /* reacquire device */
-      _TRACE("mouse device not acquired or lost\n");
+      _TRACE(PREFIX_W "mouse device not acquired or lost\n");
       wnd_schedule_proc(mouse_dinput_acquire);
    }
    else if (FAILED(hr)) {  /* other error? */
-      _TRACE("unexpected error while filling mouse message buffer\n");
+      _TRACE(PREFIX_E "unexpected error while filling mouse message buffer\n");
    }
    else {
       /* normally only this case should happen */
@@ -368,7 +405,7 @@ int mouse_dinput_acquire(void)
       hr = IDirectInputDevice_Acquire(mouse_dinput_device);
 
       if (FAILED(hr)) {
-	 _TRACE("acquire mouse failed: %s\n", dinput_err_str(hr));
+	 _TRACE(PREFIX_E "acquire mouse failed: %s\n", dinput_err_str(hr));
 	 return -1;
       }
 
@@ -412,6 +449,7 @@ int mouse_dinput_grab(void)
 {
    HRESULT hr;
    DWORD level;
+   HWND allegro_wnd = win_get_window();
 
    if (mouse_dinput_device) {
       /* necessary in order to set the cooperative level */
@@ -419,17 +457,17 @@ int mouse_dinput_grab(void)
 
       if (gfx_driver && !gfx_driver->windowed) {
          level = DISCL_FOREGROUND | DISCL_EXCLUSIVE;
-         _TRACE("foreground exclusive cooperative level requested for mouse\n");
+         _TRACE(PREFIX_I "foreground exclusive cooperative level requested for mouse\n");
       }
       else {
          level = DISCL_FOREGROUND | DISCL_NONEXCLUSIVE;
-         _TRACE("foreground non-exclusive cooperative level requested for mouse\n");
+         _TRACE(PREFIX_I "foreground non-exclusive cooperative level requested for mouse\n");
       }
 
       /* set cooperative level */
       hr = IDirectInputDevice_SetCooperativeLevel(mouse_dinput_device, allegro_wnd, level);
       if (FAILED(hr)) {
-         _TRACE("set cooperative level failed: %s\n", dinput_err_str(hr));
+         _TRACE(PREFIX_E "set cooperative level failed: %s\n", dinput_err_str(hr));
          return -1;
       }
 
@@ -455,8 +493,9 @@ int mouse_dinput_grab(void)
  */
 int mouse_set_syscursor(void)
 {
+   HWND allegro_wnd = win_get_window();
    if ((mouse_dinput_device && _mouse_on) || (gfx_driver && !gfx_driver->windowed)) {
-      SetCursor(NULL);
+      SetCursor(_win_hcursor);
       /* Make sure the cursor is removed by the system. */
       PostMessage(allegro_wnd, WM_MOUSEMOVE, 0, 0);
    }
@@ -501,7 +540,7 @@ static int mouse_dinput_exit(void)
 {
    if (mouse_dinput_device) {
       /* unregister event handler first */
-      input_unregister_event(mouse_input_event);
+      _win_input_unregister_event(mouse_input_event);
 
       /* unacquire device */
       wnd_call_proc(mouse_dinput_unacquire);
@@ -604,7 +643,7 @@ static int mouse_dinput_init(void)
       goto Error;
 
    /* Register event handler */
-   if (input_register_event(mouse_input_event, mouse_dinput_handle) != 0)
+   if (_win_input_register_event(mouse_input_event, mouse_dinput_handle) != 0)
       goto Error;
 
    /* Grab the device */
@@ -633,12 +672,12 @@ static int mouse_directx_init(void)
 
    if (mouse_dinput_init() != 0) {
       /* something has gone wrong */
-      _TRACE("mouse handler init failed\n");
+      _TRACE(PREFIX_E "mouse handler init failed\n");
       return -1;
    }
 
    /* the mouse handler has successfully set up */
-   _TRACE("mouse handler starts\n");
+   _TRACE(PREFIX_I "mouse handler starts\n");
    return dinput_buttons;
 }
 
@@ -650,7 +689,7 @@ static void mouse_directx_exit(void)
 {
    if (mouse_dinput_device) {
       /* command mouse handler shutdown */
-      _TRACE("mouse handler exits\n");
+      _TRACE(PREFIX_I "mouse handler exits\n");
       mouse_dinput_exit();
    }
 }
@@ -744,3 +783,49 @@ static void mouse_directx_get_mickeys(int *mickeyx, int *mickeyy)
    *mickeyy = temp_y;
 }
 
+
+
+/* mouse_directx_enable_hardware_cursor:
+ *  enable the hardware cursor; actually a no-op in Windows, but we need to
+ *  put something in the vtable.
+ */
+static void mouse_directx_enable_hardware_cursor(int mode)
+{
+   /* Do nothing */
+   (void)mode;
+}
+
+
+
+/* mouse_directx_select_system_cursor:
+ *  Select an OS native cursor 
+ */
+static int mouse_directx_select_system_cursor (int cursor)
+{
+   HCURSOR wc;
+   HWND allegro_wnd = win_get_window();
+   
+   wc = NULL;
+   switch(cursor) {
+      case MOUSE_CURSOR_ARROW:
+         wc = LoadCursor(NULL, IDC_ARROW);
+         break;
+      case MOUSE_CURSOR_BUSY:
+         wc = LoadCursor(NULL, IDC_WAIT);
+         break;
+      case MOUSE_CURSOR_QUESTION:
+         wc = LoadCursor(NULL, IDC_HELP);
+         break;
+      case MOUSE_CURSOR_EDIT:
+         wc = LoadCursor(NULL, IDC_IBEAM);
+         break;
+      default:
+         return 0;
+   }
+
+   _win_hcursor = wc;
+   SetCursor(_win_hcursor);
+   PostMessage(allegro_wnd, WM_MOUSEMOVE, 0, 0);
+   
+   return cursor;
+}
