@@ -30,9 +30,13 @@
    #error something is wrong with the makefile
 #endif
 
+#define PREFIX_I                "al-wwnd INFO: "
+#define PREFIX_W                "al-wwnd WARNING: "
+#define PREFIX_E                "al-wwnd ERROR: "
+
 
 /* general */
-HWND allegro_wnd = NULL;
+static HWND allegro_wnd = NULL;
 char wnd_title[WND_TITLE_SIZE];  /* ASCII string */
 int wnd_x = 0;
 int wnd_y = 0;
@@ -43,6 +47,8 @@ int wnd_sysmenu = FALSE;
 static int last_wnd_x = -1;
 static int last_wnd_y = -1;
 
+static int window_is_initialized = FALSE;
+
 /* graphics */
 WIN_GFX_DRIVER *win_gfx_driver;
 CRITICAL_SECTION gfx_crit_sect;
@@ -78,6 +84,16 @@ struct WINDOW_MODULES {
    int midi_input_card;
 };
 
+/* Used in adjust_window(). */
+#ifndef CCHILDREN_TITLEBAR
+   #define CCHILDREN_TITLEBAR 5
+typedef struct {
+   DWORD cbSize;
+   RECT  rcTitleBar;
+   DWORD rgstate[CCHILDREN_TITLEBAR + 1];
+} TITLEBARINFO, *PTITLEBARINFO, *LPTITLEBARINFO;
+#endif /* ifndef CCHILDREN_TITLEBAR */
+
 
 
 /* init_window_modules:
@@ -117,7 +133,7 @@ static int init_window_modules(struct WINDOW_MODULES *wm)
 static void exit_window_modules(struct WINDOW_MODULES *wm)
 {
    if (wm)
-      memset(wm, 0, sizeof(wm));
+      memset(wm, 0, sizeof(*wm));
 
    if (_keyboard_installed) {
      if (wm)
@@ -212,7 +228,7 @@ static LRESULT CALLBACK directx_wnd_proc(HWND wnd, UINT message, WPARAM wparam,
       case WM_DESTROY:
          if (user_wnd_proc) {
             exit_window_modules(NULL);
-            sys_reset_switch_mode();
+            _win_reset_switch_mode();
          }
          else {
             PostQuitMessage(0);
@@ -230,7 +246,7 @@ static LRESULT CALLBACK directx_wnd_proc(HWND wnd, UINT message, WPARAM wparam,
 
       case WM_ACTIVATE:
          if (LOWORD(wparam) == WA_INACTIVE) {
-            sys_switch_out();
+            _win_switch_out();
          }
          else {
             if (gfx_driver && !gfx_driver->windowed) {
@@ -239,7 +255,7 @@ static LRESULT CALLBACK directx_wnd_proc(HWND wnd, UINT message, WPARAM wparam,
             }
             else {
                /* no delay in windowed mode */
-               sys_switch_in();
+               _win_switch_in();
             }
          }
          break;
@@ -247,7 +263,7 @@ static LRESULT CALLBACK directx_wnd_proc(HWND wnd, UINT message, WPARAM wparam,
       case WM_TIMER:
          if (wparam == SWITCH_TIMER) {
             KillTimer(allegro_wnd, SWITCH_TIMER);
-            sys_switch_in();
+            _win_switch_in();
             return 0;
          }
          break;
@@ -303,6 +319,15 @@ static LRESULT CALLBACK directx_wnd_proc(HWND wnd, UINT message, WPARAM wparam,
             return 0;
          break;
 
+      case WM_SYSCOMMAND:
+         if (wparam == SC_MONITORPOWER || wparam == SC_SCREENSAVE) {
+            if (_screensaver_policy == ALWAYS_DISABLED
+                || (_screensaver_policy == FULLSCREEN_DISABLED
+                    && gfx_driver && !gfx_driver->windowed))
+            return 0;
+         }
+         break;
+
       case WM_INITMENUPOPUP:
          wnd_sysmenu = TRUE;
          mouse_set_sysmenu(TRUE);
@@ -321,6 +346,9 @@ static LRESULT CALLBACK directx_wnd_proc(HWND wnd, UINT message, WPARAM wparam,
          }
          break;
 
+      case WM_MENUCHAR :
+         return (MNC_CLOSE<<16)|(wparam&0xffff);
+         
       case WM_CLOSE:
          if (!user_wnd_proc) {
             if (user_close_proc)
@@ -385,7 +413,7 @@ static HWND create_directx_window(void)
                         -100, -100, 0, 0,
                         NULL, NULL, allegro_inst, NULL);
    if (!wnd) {
-      _TRACE("CreateWindowEx() failed (%s)\n", win_err_str(GetLastError()));
+      _TRACE(PREFIX_E "CreateWindowEx() failed (%s)\n", win_err_str(GetLastError()));
       return NULL;
    }
 
@@ -406,8 +434,8 @@ static void wnd_thread_proc(HANDLE setup_event)
    int result;
    MSG msg;
 
-   thread_init();
-   _TRACE("window thread starts\n");
+   _win_thread_init();
+   _TRACE(PREFIX_I "window thread starts\n");
 
    /* setup window */
    if (wnd_create_proc)
@@ -423,16 +451,15 @@ static void wnd_thread_proc(HANDLE setup_event)
 
    /* message loop */
    while (TRUE) {
-      result = MsgWaitForMultipleObjects(input_events, input_event_id, FALSE, INFINITE, QS_ALLINPUT);
-      if ((result >= WAIT_OBJECT_0) && (result < WAIT_OBJECT_0 + input_events)) {
+      result = MsgWaitForMultipleObjects(_win_input_events, _win_input_event_id, FALSE, INFINITE, QS_ALLINPUT);
+      if ((result >= WAIT_OBJECT_0) && (result < WAIT_OBJECT_0 + _win_input_events)) {
          /* one of the registered events is in signaled state */
-         (*input_event_handler[result - WAIT_OBJECT_0])();
+         (*_win_input_event_handler[result - WAIT_OBJECT_0])();
       }
-      else if (result == WAIT_OBJECT_0 + input_events) {
+      else if (result == WAIT_OBJECT_0 + _win_input_events) {
          /* messages are waiting in the queue */
          while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
             if (GetMessage(&msg, NULL, 0, 0)) {
-               TranslateMessage(&msg);
                DispatchMessage(&msg);
             }
             else {
@@ -443,8 +470,8 @@ static void wnd_thread_proc(HANDLE setup_event)
    }
 
  End:
-   _TRACE("window thread exits\n");
-   thread_exit();
+   _TRACE(PREFIX_I "window thread exits\n");
+   _win_thread_exit();
 }
 
 
@@ -455,7 +482,10 @@ static void wnd_thread_proc(HANDLE setup_event)
  */
 int init_directx_window(void)
 {
-   RECT win_rect;
+   union {
+     POINT p;
+     RECT r;
+   } win_rect;
    HANDLE events[2];
    long result;
 
@@ -465,7 +495,7 @@ int init_directx_window(void)
 
    if (user_wnd) {
       /* initializes input module and requests dedicated thread */
-      input_init(TRUE);
+      _win_input_init(TRUE);
 
       /* hook the user window */
       user_wnd_proc = (WNDPROC) SetWindowLong(user_wnd, GWL_WNDPROC, (long)directx_wnd_proc);
@@ -475,17 +505,17 @@ int init_directx_window(void)
       allegro_wnd = user_wnd;
 
       /* retrieve the window dimensions */
-      GetWindowRect(allegro_wnd, &win_rect);
-      ClientToScreen(allegro_wnd, (LPPOINT)&win_rect);
-      ClientToScreen(allegro_wnd, (LPPOINT)&win_rect + 1);
-      wnd_x = win_rect.left;
-      wnd_y = win_rect.top;
-      wnd_width = win_rect.right - win_rect.left;
-      wnd_height = win_rect.bottom - win_rect.top;
+      GetWindowRect(allegro_wnd, &win_rect.r);
+      ClientToScreen(allegro_wnd, &win_rect.p);
+      ClientToScreen(allegro_wnd, &win_rect.p + 1);
+      wnd_x = win_rect.r.left;
+      wnd_y = win_rect.r.top;
+      wnd_width = win_rect.r.right - win_rect.r.left;
+      wnd_height = win_rect.r.bottom - win_rect.r.top;
    }
    else {
       /* initializes input module without dedicated thread */
-      input_init(FALSE);
+      _win_input_init(FALSE);
 
       /* create window thread */
       events[0] = CreateEvent(NULL, FALSE, FALSE, NULL);        /* acknowledges that thread is up */
@@ -542,7 +572,9 @@ void exit_directx_window(void)
 
    DeleteCriticalSection(&gfx_crit_sect);
 
-   input_exit();
+   _win_input_exit();
+   
+   window_is_initialized = FALSE;
 }
 
 
@@ -564,24 +596,63 @@ void restore_window_style(void)
 int adjust_window(int w, int h)
 {
    RECT working_area, win_size;
+   TITLEBARINFO tb_info;
+   HMODULE user32_handle;
+   typedef BOOL (WINAPI *func)(HWND, PTITLEBARINFO);
+   func get_title_bar_info = NULL;
+   int tb_height = 0;
+   static int last_w=-1, last_h=-1;
 
    if (!user_wnd) {
-      if (last_wnd_x < 0) {
+      SystemParametersInfo(SPI_GETWORKAREA, 0, &working_area, 0);
+
+      if ((last_w == -1) && (last_h == -1)) {
          /* first window placement: try to center it */
-         SystemParametersInfo(SPI_GETWORKAREA, 0, &working_area, 0);
          last_wnd_x = (working_area.left + working_area.right - w)/2;
          last_wnd_y = (working_area.top + working_area.bottom - h)/2;
+      }
+      else {
+         /* try to get the height of the window's title bar */
+         user32_handle = GetModuleHandle("user32");
+         if (user32_handle) {
+            get_title_bar_info
+               = (func)GetProcAddress(user32_handle, "GetTitleBarInfo");
+            if (get_title_bar_info) {
+               tb_info.cbSize = sizeof(TITLEBARINFO);
+               if (get_title_bar_info(allegro_wnd, &tb_info))
+                  tb_height
+                     = tb_info.rcTitleBar.bottom - tb_info.rcTitleBar.top;
+            }
+         }
+         if (!user32_handle || !get_title_bar_info)
+            tb_height = GetSystemMetrics(SM_CYSIZE);
+         
+	 /* try to center the window relative to its last position */
+	 last_wnd_x += (last_w - w)/2;
+	 last_wnd_y += (last_h - h)/2;
+	 
+	 if (last_wnd_x + w >= working_area.right)
+	    last_wnd_x = working_area.right - w;
+	 if (last_wnd_y + h >= working_area.bottom)
+	    last_wnd_y = working_area.bottom - h;
+	 if (last_wnd_x < working_area.left)
+	    last_wnd_x = working_area.left;
+	 if (last_wnd_y - tb_height < working_area.top)
+	    last_wnd_y = working_area.top + tb_height;
+      }
 
 #ifdef ALLEGRO_COLORCONV_ALIGNED_WIDTH
-         last_wnd_x &= 0xfffffffc;
+      last_wnd_x &= 0xfffffffc;
 #endif
-      }
 
       win_size.left = last_wnd_x;
       win_size.top = last_wnd_y;
       win_size.right = last_wnd_x+w;
       win_size.bottom = last_wnd_y+h;
 
+      last_w = w;
+      last_h = h;
+
       /* retrieve the size of the decorated window */
       AdjustWindowRect(&win_size, GetWindowLong(allegro_wnd, GWL_STYLE), FALSE);
    
@@ -626,7 +697,7 @@ void win_set_window(HWND wnd)
    static int (*saved_scbc)(void (*proc)(void)) = NULL;
    struct WINDOW_MODULES wm;
 
-   if (_allegro_count > 0) {
+   if (window_is_initialized || !wnd) {
       exit_window_modules(&wm);
       exit_directx_window();
    }
@@ -646,10 +717,12 @@ void win_set_window(HWND wnd)
          system_directx.set_close_button_callback = saved_scbc;
    }
 
-   if (_allegro_count > 0) {
+   if (window_is_initialized || !wnd) {
       init_directx_window();
       init_window_modules(&wm);
    }
+   
+   window_is_initialized = TRUE;
 }
 
 
