@@ -139,8 +139,8 @@ static int scale_icon(ICON_DATA *icon)
    if (!shape)
       return -1;
    blit(icon->original, shape, 0, 0, 0, 0, icon->original->w, icon->original->h);
-   clear_to_color(icon->workspace, makeacol32(0, 0, 0, 0));
-   shape->vtable->mask_color = makeacol32(255, 0, 255, 255);
+   clear_to_color(icon->workspace, makeacol32(0, 0, 0, 255));
+   shape->vtable->mask_color = makeacol32(255, 0, 255, 0);
    masked_blit(shape, icon->workspace, 0, 0, x_ofs, y_ofs, shape->w, shape->h);
    destroy_bitmap(shape);
    
@@ -164,7 +164,7 @@ static int scale_icon(ICON_DATA *icon)
 	       alpha += ((float)geta32(color) * r1 * r2);
             }
          }
-	 color = makeacol32((int)floor(red), (int)floor(green), (int)floor(blue), (int)floor(alpha));
+	 color = makeacol32((int)floor(red), (int)floor(green), (int)floor(blue), 255 - (int)floor(alpha));
          ((unsigned int *)(icon->scaled->line[y]))[x] = color;
       }
    }
@@ -205,7 +205,6 @@ static int load_resource(char *datafile, char *name, ICON_DATA *icon)
 	    break;
 	    
 	 case DAT_PALETTE:
-	    unselect_palette();
 	    select_palette((RGB *)data->dat);
 	    unload_datafile_object(data);
 	    return 0;
@@ -219,7 +218,6 @@ static int load_resource(char *datafile, char *name, ICON_DATA *icon)
    }
    else {
       bitmap = load_bitmap(name, palette);
-      unselect_palette();
       select_palette(palette);
       if (!bitmap) {
          fprintf(stderr, "Unable to load '%s'\n", name);
@@ -424,7 +422,6 @@ int main(int argc, char *argv[])
 	 }
       }
    }
-   unselect_palette();
    
    buffer = malloc(4096);
    if (!buffer) {
@@ -482,6 +479,11 @@ int main(int argc, char *argv[])
 	 result = -1;
 	 goto exit_error_bundle;
       }
+      if (!exists("/Library/Frameworks/Allegro.framework/Resources/Embeddable")) {
+         fprintf(stderr, "Cannot embed system wide Allegro framework; install embeddable version first!\n");
+	 result = -1;
+	 goto exit_error_bundle;
+      }
       sprintf(buffer, "/Developer/Tools/pbxcp -exclude .DS_Store -exclude CVS -resolve-src-symlinks /Library/Frameworks/Allegro.framework %s", bundle_contents_frameworks_dir);
       if ((mkdir(bundle_contents_frameworks_dir, 0777) && (errno != EEXIST)) ||
 	  (system(buffer))) {
@@ -512,7 +514,7 @@ int main(int argc, char *argv[])
 	    DisposeHandle(raw_data);
 	    /* Set 8bit mask */
             raw_data = NewHandle(icon_data[i].size * icon_data[i].size);
-	    data = *raw_data;
+	    data = *(unsigned char **)raw_data;
 	    for (y = 0; y < icon_data[i].size; y++) {
 	       for (x = 0; x < icon_data[i].size; x++) {
 	          *data++ = geta32(((unsigned int *)(icon_data[i].scaled->line[y]))[x]);
@@ -529,7 +531,7 @@ int main(int argc, char *argv[])
 	    if (icon_data[i].mask1) {
 	       size = ((icon_data[i].size * icon_data[i].size) + 7) / 8;
 	       raw_data = NewHandle(size * 2);
-	       data = *raw_data;
+	       data = *(unsigned char **)raw_data;
 	       mask_byte = 0;
 	       mask_bit = 7;
 	       for (y = 0; y < icon_data[i].size; y++) {
@@ -564,7 +566,7 @@ int main(int argc, char *argv[])
       }
       pack_fclose(f);
       
-      cf_url_ref = CFURLCreateWithBytes(kCFAllocatorDefault, bundle_icns, strlen(bundle_icns), 0, NULL);
+      cf_url_ref = CFURLCreateWithBytes(kCFAllocatorDefault, (unsigned char *)bundle_icns, strlen(bundle_icns), 0, NULL);
       if (!cf_url_ref) {
          fprintf(stderr, "Cannot create %s\n", bundle_icns);
 	 result = -1;
